/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples;

import com.alee.extended.ninepatch.NinePatchEditorPanel;
import com.alee.laf.WebLookAndFeel;
import com.alee.utils.ImageUtils;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * User: mgarin Date: 20.12.11 Time: 13:43
 */

public class NinePatchEditorDialog extends JFrame
{
    private NinePatchEditorPanel ninePatchEditorPanel = null;

    public NinePatchEditorDialog (  )
    {
        super (  );

        updateTitle ();
        setIconImages ( NinePatchEditorPanel.icons );

        getContentPane ().setLayout ( new BorderLayout () );

        ninePatchEditorPanel = new NinePatchEditorPanel ();
        ninePatchEditorPanel.addChangeListener ( new ChangeListener ()
        {
            public void stateChanged ( ChangeEvent e )
            {
                updateTitle ();
            }
        } );
        getContentPane ().add ( ninePatchEditorPanel );
    }

    private void updateTitle ()
    {
        String imageSrc = ninePatchEditorPanel != null ? ninePatchEditorPanel.getImageSrc () : null;
        setTitle ( "Nine-patch editor" + ( imageSrc != null ? " - [" + imageSrc + "]" : "" ) );
    }

    public NinePatchEditorPanel getNinePatchEditorPanel ()
    {
        return ninePatchEditorPanel;
    }

    public void setNinePatchImage ( BufferedImage ninePatchImage )
    {
        ninePatchEditorPanel.setNinePatchImage ( ninePatchImage );
    }

    public BufferedImage getNinePatchImage ()
    {
        return ninePatchEditorPanel.getNinePatchImage ();
    }

    public static void main ( String[] args )
    {
        WebLookAndFeel.install ();

        NinePatchEditorDialog nped = new NinePatchEditorDialog ();
        nped.setNinePatchImage ( ImageUtils.getBufferedImage (
                NinePatchEditorDialog.class.getResource ( "icons/example.png" ) ) );
        nped.setDefaultCloseOperation ( JDialog.DISPOSE_ON_CLOSE );
        nped.pack ();
        nped.setLocationRelativeTo ( null );
        nped.setVisible ( true );
        nped.getNinePatchEditorPanel ().getNinePatchEditor ().requestFocusInWindow ();
    }
}
