/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.date;

import com.alee.extended.image.WebImageTransition;
import com.alee.extended.image.transition.Direction;
import com.alee.extended.image.transition.SlideType;
import com.alee.extended.image.transition.TransitionEffect;
import com.alee.extended.panel.TransitionPanel;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.laf.button.WebToggleButton;
import com.alee.laf.label.WebLabel;
import com.alee.laf.panel.WebPanel;
import com.alee.laf.separator.WebSeparator;
import com.alee.utils.LafUtils;
import com.alee.utils.TimeUtils;
import com.alee.utils.laf.ShapeProvider;
import info.clearthought.layout.TableLayout;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * User: mgarin Date: 01.11.11 Time: 14:56
 */

public class WebCalendar extends WebPanel implements ShapeProvider
{
    public static final ImageIcon previousSkipIcon =
            new ImageIcon ( WebCalendar.class.getResource ( "icons/previous_skip.png" ) );
    public static final ImageIcon previousIcon =
            new ImageIcon ( WebCalendar.class.getResource ( "icons/previous.png" ) );
    public static final ImageIcon nextIcon =
            new ImageIcon ( WebCalendar.class.getResource ( "icons/next.png" ) );
    public static final ImageIcon nextSkipIcon =
            new ImageIcon ( WebCalendar.class.getResource ( "icons/next_skip.png" ) );

    private List<DateSelectionListener> dateSelectionListeners =
            new ArrayList<DateSelectionListener> ();

    private SimpleDateFormat titleFormat = new SimpleDateFormat ( "MMMM yyyy" );
    private Date date = new Date ( System.currentTimeMillis () );
    private Date shownDate = new Date ( date.getTime () );
    private Date oldShownDate = new Date ( System.currentTimeMillis () );

    private boolean startWeekFromSunday = false;

    private boolean animate = true;
    private boolean horizontalSlide = true;

    private DateCustomizer dateCustomizer = null;

    private Color otherMonthColor = new Color ( 90, 90, 90 );
    private Color weekendsColor = new Color ( 160, 0, 0 );

    private WebButton previousSkip;
    private WebButton previous;
    private WebLabel titleLabel;
    private WebButton next;
    private WebButton nextSkip;

    private JPanel weekHeaders;

    private JPanel monthDays;
    private TransitionPanel monthDaysTransition;

    public WebCalendar ()
    {
        this ( null );
    }

    public WebCalendar ( Date date )
    {
        super ();
        setUndecorated ( false );
        setRound ( StyleConstants.smallRound );

        if ( date != null )
        {
            this.date = date;
            this.shownDate = date;
        }

        // Main layout

        setLayout ( new BorderLayout ( 0, 0 ) );

        JPanel centerPanel = new JPanel ();
        centerPanel.setOpaque ( false );
        centerPanel.setLayout ( new BorderLayout ( 0, 0 ) );
        add ( centerPanel, BorderLayout.CENTER );


        // Header panel

        JPanel header = new JPanel ();
        header.setOpaque ( false );
        TableLayout headerLayout = new TableLayout ( new double[][]{
                { TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL,
                        TableLayout.PREFERRED, TableLayout.PREFERRED },
                { TableLayout.PREFERRED } } );
        headerLayout.setHGap ( 0 );
        headerLayout.setVGap ( 0 );
        header.setLayout ( headerLayout );
        add ( header, BorderLayout.NORTH );

        previousSkip =
                WebButton.createIconWebButton ( previousSkipIcon, StyleConstants.smallRound, true );
        previousSkip.setDrawFocus ( false );
        previousSkip.setFocusable ( true );
        previousSkip.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                Calendar calendar = Calendar.getInstance ();
                calendar.setTime ( shownDate );
                calendar.set ( Calendar.YEAR, calendar.get ( Calendar.YEAR ) - 1 );
                setShownDate ( calendar.getTime () );
            }
        } );
        header.add ( previousSkip, "0,0" );

        previous = WebButton.createIconWebButton ( previousIcon, StyleConstants.smallRound, true );
        previous.setDrawFocus ( false );
        previous.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                Calendar calendar = Calendar.getInstance ();
                calendar.setTime ( shownDate );
                calendar.set ( Calendar.MONTH, calendar.get ( Calendar.MONTH ) - 1 );
                setShownDate ( calendar.getTime () );
            }
        } );
        header.add ( previous, "1,0" );

        titleLabel = new WebLabel ();
        titleLabel.setHorizontalAlignment ( WebLabel.CENTER );
        titleLabel.setFont ( titleLabel.getFont ().deriveFont ( Font.BOLD ) );
        updateTitleLabel ();
        header.add ( titleLabel, "2,0" );

        next = WebButton.createIconWebButton ( nextIcon, StyleConstants.smallRound, true );
        next.setDrawFocus ( false );
        next.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                Calendar calendar = Calendar.getInstance ();
                calendar.setTime ( shownDate );
                calendar.set ( Calendar.MONTH, calendar.get ( Calendar.MONTH ) + 1 );
                setShownDate ( calendar.getTime () );
            }
        } );
        header.add ( next, "3,0" );

        nextSkip = WebButton.createIconWebButton ( nextSkipIcon, StyleConstants.smallRound, true );
        nextSkip.setDrawFocus ( false );
        nextSkip.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                Calendar calendar = Calendar.getInstance ();
                calendar.setTime ( shownDate );
                calendar.set ( Calendar.YEAR, calendar.get ( Calendar.YEAR ) + 1 );
                setShownDate ( calendar.getTime () );
            }
        } );
        header.add ( nextSkip, "4,0" );


        // Week days

        weekHeaders = new JPanel ()
        {
            protected void paintComponent ( Graphics g )
            {
                super.paintComponent ( g );

                LafUtils.drawCustomWebBorder ( ( Graphics2D ) g, this,
                        new Rectangle ( -1, 0, getWidth () + 2,
                                getHeight () - StyleConstants.shadeWidth ),
                        StyleConstants.shadeColor, StyleConstants.shadeWidth, true, true );
            }
        };
        weekHeaders.setOpaque ( false );
        weekHeaders.setBorder ( BorderFactory
                .createEmptyBorder ( StyleConstants.shadeWidth, StyleConstants.shadeWidth - 1,
                        StyleConstants.shadeWidth + 1, StyleConstants.shadeWidth - 1 ) );
        weekHeaders.setLayout ( new TableLayout ( new double[][]{
                { TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED,
                        TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL,
                        TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED,
                        TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL },
                { TableLayout.PREFERRED } } ) );
        centerPanel.add ( weekHeaders, BorderLayout.NORTH );

        updateWeekHeaders ();


        // Month days

        monthDays = createMonthDaysPanel ();
        updateMonthDays ( monthDays );

        monthDaysTransition = new TransitionPanel ( monthDays );
        monthDaysTransition.setOpaque ( false );
        monthDaysTransition.setTransitionEffect ( TransitionEffect.slide );
        monthDaysTransition.put ( WebImageTransition.SLIDE_TYPE, SlideType.moveBoth );
        centerPanel.add ( monthDaysTransition, BorderLayout.CENTER );
    }

    private JPanel createMonthDaysPanel ()
    {
        JPanel monthDays = new JPanel ();
        monthDays.setOpaque ( false );
        monthDays.setBorder ( BorderFactory
                .createEmptyBorder ( StyleConstants.shadeWidth - 1, StyleConstants.shadeWidth - 1,
                        StyleConstants.shadeWidth - 1, StyleConstants.shadeWidth - 1 ) );
        final TableLayout layout = new TableLayout ( new double[][]{
                { TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED,
                        TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL,
                        TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED,
                        TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL },
                { TableLayout.FILL, TableLayout.FILL, TableLayout.FILL, TableLayout.FILL,
                        TableLayout.FILL, TableLayout.FILL } } );
        layout.setHGap ( 0 );
        layout.setVGap ( 0 );
        monthDays.setLayout ( layout );
        return monthDays;
    }

    private void switchMonthDays ()
    {
        if ( animate )
        {
            JPanel newMonthDays = createMonthDaysPanel ();
            updateMonthDays ( newMonthDays );

            monthDaysTransition.put ( WebImageTransition.DIRECTION,
                    oldShownDate.getTime () > shownDate.getTime () ?
                            ( horizontalSlide ? Direction.right : Direction.down ) :
                            ( horizontalSlide ? Direction.left : Direction.up ) );
            monthDaysTransition.switchContent ( newMonthDays );
        }
        else
        {
            updateMonthDays ( monthDays );
        }
    }

    private void updateMonthDays ( JPanel monthDays )
    {
        monthDays.removeAll ();

        monthDays.add ( new WebSeparator ( WebSeparator.VERTICAL ), "1,0,1,5" );
        monthDays.add ( new WebSeparator ( WebSeparator.VERTICAL ), "3,0,3,5" );
        monthDays.add ( new WebSeparator ( WebSeparator.VERTICAL ), "5,0,5,5" );
        monthDays.add ( new WebSeparator ( WebSeparator.VERTICAL ), "7,0,7,5" );
        monthDays.add ( new WebSeparator ( WebSeparator.VERTICAL ), "9,0,9,5" );
        monthDays.add ( new WebSeparator ( WebSeparator.VERTICAL ), "11,0,11,5" );

        ButtonGroup dates = new ButtonGroup ();

        Calendar calendar = Calendar.getInstance ();
        calendar.setTime ( shownDate );
        calendar.set ( Calendar.DAY_OF_MONTH, 1 );

        int col = 0;
        int row = 0;

        // Month before
        int dayOfWeek = calendar.get ( Calendar.DAY_OF_WEEK );
        int shift;
        switch ( dayOfWeek )
        {
            case Calendar.MONDAY:
                shift = startWeekFromSunday ? 1 : 7;
                break;
            case Calendar.TUESDAY:
                shift = startWeekFromSunday ? 2 : 1;
                break;
            case Calendar.WEDNESDAY:
                shift = startWeekFromSunday ? 3 : 2;
                break;
            case Calendar.THURSDAY:
                shift = startWeekFromSunday ? 4 : 3;
                break;
            case Calendar.FRIDAY:
                shift = startWeekFromSunday ? 5 : 4;
                break;
            case Calendar.SATURDAY:
                shift = startWeekFromSunday ? 6 : 5;
                break;
            case Calendar.SUNDAY:
                shift = startWeekFromSunday ? 7 : 6;
                break;
            default:
                shift = 0;
                break;
        }
        TimeUtils.changeByDays ( calendar, -shift );
        while ( calendar.get ( Calendar.DAY_OF_MONTH ) > 1 )
        {
            final Date thisDate = calendar.getTime ();
            final WebToggleButton day = new WebToggleButton ();
            day.setFont ( day.getFont ().deriveFont ( Font.PLAIN ) );
            day.setForeground ( otherMonthColor );
            day.setText ( "" + calendar.get ( Calendar.DAY_OF_MONTH ) );
            day.setRolloverDecoratedOnly ( true );
            day.setHorizontalAlignment ( WebButton.RIGHT );
            day.setRound ( StyleConstants.smallRound );
            day.addItemListener ( new ItemListener ()
            {
                public void itemStateChanged ( ItemEvent e )
                {
                    day.setFont ( day.isSelected () ? day.getFont ().deriveFont ( Font.BOLD ) :
                            day.getFont ().deriveFont ( Font.PLAIN ) );
                    if ( day.isSelected () )
                    {
                        setDate ( thisDate );
                    }
                }
            } );
            if ( dateCustomizer != null )
            {
                dateCustomizer.customize ( day, thisDate );
            }
            monthDays.add ( day, col * 2 + "," + row );
            dates.add ( day );

            TimeUtils.increaseByDay ( calendar );

            col++;
            if ( col > 6 )
            {
                col = 0;
                row++;
            }
        }

        // Current month
        do
        {
            final boolean weekend = calendar.get ( Calendar.DAY_OF_WEEK ) == 1 ||
                    calendar.get ( Calendar.DAY_OF_WEEK ) == 7;
            final boolean selected = TimeUtils.isSameDay ( calendar, date.getTime () );

            final Date thisDate = calendar.getTime ();
            final WebToggleButton day = new WebToggleButton ();
            day.setFont ( selected ? day.getFont ().deriveFont ( Font.BOLD ) :
                    day.getFont ().deriveFont ( Font.PLAIN ) );
            day.setForeground ( weekend ? weekendsColor : Color.BLACK );
            day.setText ( "" + calendar.get ( Calendar.DAY_OF_MONTH ) );
            day.setSelected ( selected );
            day.setRolloverDecoratedOnly ( true );
            day.setHorizontalAlignment ( WebButton.RIGHT );
            day.setRound ( StyleConstants.smallRound );
            day.addItemListener ( new ItemListener ()
            {
                public void itemStateChanged ( ItemEvent e )
                {
                    day.setFont ( day.isSelected () ? day.getFont ().deriveFont ( Font.BOLD ) :
                            day.getFont ().deriveFont ( Font.PLAIN ) );
                    if ( day.isSelected () )
                    {
                        setDate ( thisDate );
                    }
                }
            } );
            if ( dateCustomizer != null )
            {
                dateCustomizer.customize ( day, thisDate );
            }
            monthDays.add ( day, col * 2 + "," + row );
            dates.add ( day );

            TimeUtils.increaseByDay ( calendar );

            col++;
            if ( col > 6 )
            {
                col = 0;
                row++;
            }
        }
        while ( calendar.get ( Calendar.DAY_OF_MONTH ) > 1 );

        // Month after
        int left = 6 * 7 - ( monthDays.getComponentCount () - 6 );
        for ( int i = 1; i <= left; i++ )
        {
            final Date thisDate = calendar.getTime ();
            final WebToggleButton day = new WebToggleButton ();
            day.setFont ( day.getFont ().deriveFont ( Font.PLAIN ) );
            day.setForeground ( otherMonthColor );
            day.setText ( "" + calendar.get ( Calendar.DAY_OF_MONTH ) );
            day.setRolloverDecoratedOnly ( true );
            day.setHorizontalAlignment ( WebButton.RIGHT );
            day.setRound ( StyleConstants.smallRound );
            day.addItemListener ( new ItemListener ()
            {
                public void itemStateChanged ( ItemEvent e )
                {
                    day.setFont ( day.isSelected () ? day.getFont ().deriveFont ( Font.BOLD ) :
                            day.getFont ().deriveFont ( Font.PLAIN ) );
                    if ( day.isSelected () )
                    {
                        setDate ( thisDate );
                    }
                }
            } );
            if ( dateCustomizer != null )
            {
                dateCustomizer.customize ( day, thisDate );
            }
            monthDays.add ( day, col * 2 + "," + row );
            dates.add ( day );

            TimeUtils.increaseByDay ( calendar );

            col++;
            if ( col > 6 )
            {
                col = 0;
                row++;
            }
        }

        monthDays.revalidate ();
    }

    private void updateWeekHeaders ()
    {
        weekHeaders.removeAll ();
        for ( int i = 1; i <= 7; i++ )
        {
            int day = startWeekFromSunday ? ( i == 1 ? 7 : i - 1 ) : i;
            WebLabel dayOfWeekLabel =
                    new WebLabel ( UIManager.getString ( "WebCalendar.dayOfWeek." + day ) );
            dayOfWeekLabel.setHorizontalAlignment ( WebLabel.CENTER );
            dayOfWeekLabel.setFont (
                    dayOfWeekLabel.getFont ().deriveFont ( 10f ).deriveFont ( Font.BOLD ) );
            weekHeaders.add ( dayOfWeekLabel, ( i - 1 ) * 2 + ",0" );

            if ( i < 7 )
            {
                weekHeaders.add ( new WebSeparator ( WebSeparator.VERTICAL ),
                        ( ( i - 1 ) * 2 + 1 ) + ",0" );
            }
        }
        weekHeaders.revalidate ();
    }

    private void updateTitleLabel ()
    {
        titleLabel.setText ( titleFormat.format ( shownDate ) );
    }

    public Date getDate ()
    {
        return date;
    }

    public void setDate ( Date date )
    {
        if ( this.date.getTime () != date.getTime () )
        {
            this.date = date;
            setShownDate ( date );
            fireDateSelected ( date );
        }
    }

    public Date getShownDate ()
    {
        return shownDate;
    }

    public void setShownDate ( Date shownDate )
    {
        this.oldShownDate = this.shownDate;
        this.shownDate = shownDate;

        Calendar calendar = Calendar.getInstance ();
        calendar.setTime ( oldShownDate );
        int oldMonth = calendar.get ( Calendar.MONTH );
        int oldYear = calendar.get ( Calendar.YEAR );
        calendar.setTime ( shownDate );
        int newMonth = calendar.get ( Calendar.MONTH );
        int newYear = calendar.get ( Calendar.YEAR );
        if ( oldMonth != newMonth || oldYear != newYear )
        {
            updateTitleLabel ();
            switchMonthDays ();
        }
    }

    public boolean isStartWeekFromSunday ()
    {
        return startWeekFromSunday;
    }

    public void setStartWeekFromSunday ( boolean startWeekFromSunday )
    {
        this.startWeekFromSunday = startWeekFromSunday;
        updateWeekHeaders ();
        updateMonthDays ( monthDays );
    }

    public boolean isAnimate ()
    {
        return animate;
    }

    public void setAnimate ( boolean animate )
    {
        this.animate = animate;
    }

    public boolean isHorizontalSlide ()
    {
        return horizontalSlide;
    }

    public void setHorizontalSlide ( boolean horizontalSlide )
    {
        this.horizontalSlide = horizontalSlide;
    }

    public Color getOtherMonthColor ()
    {
        return otherMonthColor;
    }

    public void setOtherMonthColor ( Color otherMonthColor )
    {
        this.otherMonthColor = otherMonthColor;
        updateMonthDays ( monthDays );
    }

    public Color getWeekendsColor ()
    {
        return weekendsColor;
    }

    public void setWeekendsColor ( Color weekendsColor )
    {
        this.weekendsColor = weekendsColor;
        updateMonthDays ( monthDays );
    }

    public DateCustomizer getDateCustomizer ()
    {
        return dateCustomizer;
    }

    public void setDateCustomizer ( DateCustomizer dateCustomizer )
    {
        this.dateCustomizer = dateCustomizer;
        updateMonthDays ( monthDays );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( WebCalendar.this, getShadeWidth (), getRound () );
    }

    public void addDateSelectionListener ( DateSelectionListener listener )
    {
        dateSelectionListeners.add ( listener );
    }

    public void removeDateSelectionListener ( DateSelectionListener listener )
    {
        dateSelectionListeners.remove ( listener );
    }

    private void fireDateSelected ( Date date )
    {
        List<DateSelectionListener> listeners = new ArrayList<DateSelectionListener> ();
        listeners.addAll ( dateSelectionListeners );
        for ( DateSelectionListener listener : listeners )
        {
            listener.dateSelected ( date );
        }
        listeners.clear ();
    }
}
