/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.alee.extended.filechooser;

import com.alee.extended.panel.GroupPanel;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.managers.hotkey.Hotkey;
import com.alee.managers.hotkey.HotkeyManager;
import com.alee.managers.hotkey.HotkeyRunnable;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 13.10.11 Time: 15:07
 */

public class WebDirectoryChooser extends JDialog
{
    private static final ImageIcon ICON =
            new ImageIcon ( WebDirectoryChooser.class.getResource ( "icons/dir_icon.png" ) );

    private static final ImageIcon SETTINGS_ICON =
            new ImageIcon ( WebDirectoryChooser.class.getResource ( "icons/settings.png" ) );
    private static final ImageIcon OK_ICON =
            new ImageIcon ( WebDirectoryChooser.class.getResource ( "icons/ok.png" ) );
    private static final ImageIcon CANCEL_ICON =
            new ImageIcon ( WebDirectoryChooser.class.getResource ( "icons/cancel.png" ) );

    private List<ActionListener> listeners = new ArrayList<ActionListener> ();

    private int result = StyleConstants.NONE_OPTION;

    private WebDirectoryChooserPanel directoryChooserPanel;

    private WebButton ok;
    private WebButton cancel;

    public WebDirectoryChooser ( Window parent, String title )
    {
        super ( parent, title );
        setIconImage ( ICON.getImage () );

        // Hotkeys preview action
        HotkeyManager.installShowAllHotkeysAction ( this, Hotkey.F1 );

        getContentPane ().setBackground ( Color.WHITE );
        getContentPane ().setLayout ( new BorderLayout ( 0, 0 ) );

        directoryChooserPanel = new WebDirectoryChooserPanel ();
        getContentPane ().add ( directoryChooserPanel, BorderLayout.CENTER );

        JPanel buttonsPanel = new JPanel ();
        buttonsPanel.setOpaque ( false );
        buttonsPanel.setLayout ( new BorderLayout ( 0, 0 ) );
        buttonsPanel.setBorder ( BorderFactory.createEmptyBorder ( 0, 3, 3, 3 ) );
        getContentPane ().add ( buttonsPanel, BorderLayout.SOUTH );


        ok = new WebButton ( UIManager.getString ( "WebFileChooser.choose" ), OK_ICON );
        HotkeyManager.registerHotkey ( WebDirectoryChooser.this, ok, Hotkey.CTRL_ENTER,
                new HotkeyRunnable ()
                {
                    public void run ( KeyEvent e )
                    {
                        if ( directoryChooserPanel.allowHotkeys () )
                        {
                            ok.doClick ();
                        }
                    }
                } );
        if ( StyleConstants.highlightControlButtons )
        {
            ok.setShineColor ( StyleConstants.greenHighlight );
        }
        ok.putClientProperty ( GroupPanel.FILL_CELL, true );
        ok.setEnabled ( false );
        ok.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                result = StyleConstants.OK_OPTION;
                WebDirectoryChooser.this.dispose ();
            }
        } );

        cancel = new WebButton ( UIManager.getString ( "WebFileChooser.cancel" ), CANCEL_ICON );
        HotkeyManager.registerHotkey ( WebDirectoryChooser.this, cancel, Hotkey.ESCAPE,
                new HotkeyRunnable ()
                {
                    public void run ( KeyEvent e )
                    {
                        if ( directoryChooserPanel.allowHotkeys () )
                        {
                            cancel.doClick ();
                        }
                    }
                } );
        if ( StyleConstants.highlightControlButtons )
        {
            cancel.setShineColor ( StyleConstants.redHighlight );
        }
        cancel.putClientProperty ( GroupPanel.FILL_CELL, true );
        cancel.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                result = StyleConstants.CANCEL_OPTION;
                WebDirectoryChooser.this.dispose ();
            }
        } );

        buttonsPanel.add ( new GroupPanel ( 4, ok, cancel ), BorderLayout.EAST );


        directoryChooserPanel.addWebDirectoryChooserListener ( new WebDirectoryChooserListener ()
        {
            public void selectionChanged ( File file )
            {
                updateButtons ( file );
            }
        } );
        updateButtons ( directoryChooserPanel.getSelectedFolder () );

        addWindowListener ( new WindowAdapter ()
        {
            public void windowClosed ( WindowEvent e )
            {
                result = StyleConstants.CLOSE_OPTION;
            }
        } );

        setModal ( true );
        pack ();
        setDefaultCloseOperation ( JDialog.DISPOSE_ON_CLOSE );
    }

    private void updateButtons ( File file )
    {
        ok.setEnabled ( file != null );
    }

    public int getResult ()
    {
        return result;
    }

    public File getSelectedFolder ()
    {
        return directoryChooserPanel.getSelectedFolder ();
    }

    public void setSelectedFolder ( File selectedFolder )
    {
        directoryChooserPanel.setSelectedFolder ( selectedFolder );
    }

    public void addWebDirectoryChooserListener ( WebDirectoryChooserListener listener )
    {
        directoryChooserPanel.addWebDirectoryChooserListener ( listener );
    }

    public void removeWebDirectoryChooserListener ( WebDirectoryChooserListener listener )
    {
        directoryChooserPanel.removeWebDirectoryChooserListener ( listener );
    }

    public int showDialog ()
    {
        setVisible ( true );
        return getResult ();
    }

    public static File showDialog ( Window parent, String title )
    {
        WebDirectoryChooser wdc = new WebDirectoryChooser ( parent, title );
        wdc.setVisible ( true );
        if ( wdc.getResult () == StyleConstants.OK_OPTION )
        {
            return wdc.getSelectedFolder ();
        }
        else
        {
            return null;
        }
    }

    public void setVisible ( boolean b )
    {
        if ( b )
        {
            result = StyleConstants.NONE_OPTION;
            setLocationRelativeTo ( getOwner () );
        }
        super.setVisible ( b );
    }
}
