/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.layout;

import java.awt.*;

/**
 * User: mgarin Date: 09.11.11 Time: 12:38
 */

public class OverlapFlowLayout implements LayoutManager
{
    private int gap = 0;

    public OverlapFlowLayout ()
    {
        super ();
    }

    public OverlapFlowLayout ( int gap )
    {
        super ();
        this.gap = gap;
    }

    public int getGap ()
    {
        return gap;
    }

    public void setGap ( int gap )
    {
        this.gap = gap;
    }

    public void addLayoutComponent ( String name, Component comp )
    {

    }

    public void removeLayoutComponent ( Component comp )
    {

    }

    public Dimension preferredLayoutSize ( Container parent )
    {
        Insets insets = parent.getInsets ();
        Dimension maxSize =
                new Dimension ( insets.left + insets.right, insets.top + insets.bottom );
        for ( int i = 0; i < parent.getComponentCount (); i++ )
        {
            Dimension ps = parent.getComponent ( i ).getPreferredSize ();
            maxSize.width += ps.width + ( i < parent.getComponentCount () - 1 ? gap : 0 );
            maxSize.height = Math.max ( maxSize.height, insets.top + ps.height + insets.bottom );
        }
        return maxSize;
    }

    public Dimension minimumLayoutSize ( Container parent )
    {
        return preferredLayoutSize ( parent );
    }

    public void layoutContainer ( Container parent )
    {
        Insets insets = parent.getInsets ();
        int x = insets.left;
        for ( Component component : parent.getComponents () )
        {
            Dimension ps = component.getPreferredSize ();
            component.setBounds ( x, insets.top, ps.width,
                    parent.getHeight () - insets.top - insets.bottom );
            x += ps.width + gap;
        }
    }
}
