/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.layout;

import java.awt.*;

/**
 * User: mgarin Date: 07.10.11 Time: 15:14
 */

public class WrapFlowLayout implements LayoutManager
{
    private boolean fitWidth = true;
    private boolean fillWidth = true;
    private int hgap = 0;
    private int vgap = 0;

    //    private int maxWidth = 0;
    private int maxHeight = 0;

    public WrapFlowLayout ()
    {
        this ( true );
    }

    public WrapFlowLayout ( boolean fillWidth )
    {
        this ( fillWidth, 0, 0 );
    }

    public WrapFlowLayout ( boolean fillWidth, int hgap, int vgap )
    {
        super ();
        this.fillWidth = fillWidth;
        this.hgap = hgap;
        this.vgap = vgap;
    }

    public boolean isFitWidth ()
    {
        return fitWidth;
    }

    public void setFitWidth ( boolean fitWidth )
    {
        this.fitWidth = fitWidth;
    }

    public boolean isFillWidth ()
    {
        return fillWidth;
    }

    public void setFillWidth ( boolean fillWidth )
    {
        this.fillWidth = fillWidth;
    }

    public int getHgap ()
    {
        return hgap;
    }

    public void setHgap ( int hgap )
    {
        this.hgap = hgap;
    }

    public int getVgap ()
    {
        return vgap;
    }

    public void setVgap ( int vgap )
    {
        this.vgap = vgap;
    }

    public void addLayoutComponent ( String name, Component comp )
    {
        //
    }

    public void removeLayoutComponent ( Component comp )
    {
        //
    }

    public void layoutContainer ( Container parent )
    {
        // Current processed row component number
        int rowComponent = 0;

        // Current processed row height
        int currentRowMaxHeight = 0;

        // Sizes
        //        maxWidth = 0;
        maxHeight = 0;

        // Component scroll
        //        int minusWidth = 0;
        //        JScrollPane scrollPane = SwingUtils.getComponentScroll ( parent );
        //        if ( scrollPane != null && scrollPane.getVerticalScrollBar () != null &&
        //                scrollPane.getVerticalScrollBar ().isVisible () &&
        //                scrollPane.getVerticalScrollBar ().isShowing () )
        //        {
        //            minusWidth = 0;
        //        }
        Insets insets = parent.getInsets ();
        int parentWidth = parent.getWidth () - insets.left - insets.right/*- minusWidth*/;

        int x = insets.left;
        int y = insets.top;
        for ( int i = 0; i < parent.getComponentCount (); i++ )
        {
            Component component = parent.getComponent ( i );
            Dimension ps = component.getPreferredSize ();
            if ( rowComponent > 0 && x + ps.width > insets.left + parentWidth )
            {
                x = insets.left;
                y += currentRowMaxHeight;
                maxHeight += currentRowMaxHeight;
                rowComponent = 0;
                currentRowMaxHeight = 0;
            }

            int componentWidth = fitWidth ? Math.min ( ps.width, parentWidth ) : ps.width;
            if ( fillWidth && ( i + 1 == parent.getComponentCount () ||
                    x + ps.width + parent.getComponent ( i + 1 ).getPreferredSize ().width >
                            insets.left + parentWidth ) &&
                    x + ps.width < insets.left + parentWidth )
            {
                componentWidth = insets.left + parentWidth - x;
            }
            //            maxWidth = Math.max ( componentWidth, maxWidth );
            component.setBounds ( x, y, componentWidth, ps.height );

            currentRowMaxHeight = Math.max ( currentRowMaxHeight, ps.height );
            x += componentWidth;
            rowComponent++;
        }
        maxHeight += currentRowMaxHeight;
        maxHeight += insets.top + insets.bottom;
    }

    public Dimension minimumLayoutSize ( Container parent )
    {
        return preferredLayoutSize ( parent );
    }

    public Dimension preferredLayoutSize ( Container parent )
    {
        layoutContainer ( parent );
        return new Dimension ( 0, maxHeight );
    }
}
