/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.ninepatch;

import com.alee.extended.colorchooser.WebColorChooser;
import com.alee.extended.drag.FileDropHandler;
import com.alee.extended.filechooser.SelectionMode;
import com.alee.extended.filechooser.WebFileChooser;
import com.alee.extended.layout.ToolbarLayout;
import com.alee.extended.panel.BorderPanel;
import com.alee.extended.panel.ResizablePanel;
import com.alee.extended.panel.WebButtonGroup;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.laf.button.WebToggleButton;
import com.alee.laf.label.WebLabel;
import com.alee.laf.panel.WebPanel;
import com.alee.laf.scroll.WebScrollPane;
import com.alee.laf.slider.WebSlider;
import com.alee.laf.splitpane.WebSplitPane;
import com.alee.laf.text.WebTextField;
import com.alee.laf.toolbar.ToolbarStyle;
import com.alee.laf.toolbar.WebToolBar;
import com.alee.laf.toolbar.WebToolbarStyle;
import com.alee.managers.hotkey.Hotkey;
import com.alee.managers.hotkey.HotkeyManager;
import com.alee.managers.settings.SettingsManager;
import com.alee.managers.tooltip.TooltipManager;
import com.alee.utils.ImageUtils;
import com.alee.utils.image.NinePatchBackgroundPainter;
import com.alee.utils.image.NinePatchIcon;
import com.alee.utils.image.NinePatchInterval;
import com.alee.utils.laf.AlphaBackgroundPainter;
import com.alee.utils.laf.ColorBackgroundPainter;
import info.clearthought.layout.TableLayout;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * User: mgarin Date: 22.12.11 Time: 13:31
 */

public class NinePatchEditorPanel extends WebPanel
{
    public static final BufferedImage icon16 = ImageUtils
            .getBufferedImage ( NinePatchEditorPanel.class.getResource ( "icons/icon16.png" ) );
    public static final BufferedImage icon256 = ImageUtils
            .getBufferedImage ( NinePatchEditorPanel.class.getResource ( "icons/icon256.png" ) );
    public static final List<BufferedImage> icons =
            Arrays.asList ( icon16, ImageUtils.createPreviewImage ( icon256, 32 ),
                    ImageUtils.createPreviewImage ( icon256, 48 ),
                    ImageUtils.createPreviewImage ( icon256, 64 ),
                    ImageUtils.createPreviewImage ( icon256, 128 ), icon256 );

    public static final ImageIcon OPEN_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/open.png" ) );
    public static final ImageIcon SAVE_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/save.png" ) );
    public static final ImageIcon SAVE_AS_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/saveas.png" ) );
    public static final ImageIcon COPY_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/copy.png" ) );
    public static final ImageIcon PASTE_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/paste.png" ) );

    public static final ImageIcon GUIDES_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/guides.png" ) );
    public static final ImageIcon RULER_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/ruler.png" ) );
    public static final ImageIcon STRETCH_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/stretch.png" ) );
    public static final ImageIcon CONTENT_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/content.png" ) );
    public static final ImageIcon MIN_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/min.png" ) );
    public static final ImageIcon MAX_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/max.png" ) );

    public static final ImageIcon ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/icon.png" ) );
    public static final ImageIcon SHOW_ICON_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/show_icon.png" ) );
    public static final ImageIcon SHOW_TEXT_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/show_text.png" ) );
    public static final ImageIcon FOREGROUND_COLOR_ICON = new ImageIcon (
            NinePatchEditorPanel.class.getResource ( "icons/foreground_color.png" ) );
    public static final ImageIcon TRANSPARENT_ICON =
            new ImageIcon ( NinePatchEditorPanel.class.getResource ( "icons/transparent.png" ) );
    public static final ImageIcon BACKGROUND_COLOR_ICON = new ImageIcon (
            NinePatchEditorPanel.class.getResource ( "icons/background_color.png" ) );

    private List<ChangeListener> changeListeners = new ArrayList<ChangeListener> ();

    private String imageSrc = null;

    private NinePatchEditor ninePatchEditor;
    private WebToolBar toolBar;
    private WebButton save;

    private ChangeListener changeListener;
    private ZoomChangeListener zoomChangeListener;

    private AlphaBackgroundPainter abp;
    private ColorBackgroundPainter cbp;

    private WebPanel previewPanel;
    private WebLabel preview;

    public NinePatchEditorPanel ()
    {
        super ();

        SettingsManager.alias ( "NinePatchInfo", NinePatchInfo.class );
        SettingsManager.alias ( "NinePatchInterval", NinePatchInterval.class );

        setLayout ( new BorderLayout () );

        ninePatchEditor = new NinePatchEditor ();
        ninePatchEditor.setTransferHandler ( new FileDropHandler ()
        {
            protected boolean filesImported ( List<File> files )
            {
                for ( File file : files )
                {
                    if ( ImageUtils.isImageLoadable ( file.getName () ) )
                    {
                        final BufferedImage image = ImageUtils.getBufferedImage (
                                ImageUtils.loadImage ( file.getAbsolutePath () ) );
                        if ( image != null )
                        {
                            imageSrc = file.getAbsolutePath ();
                            save.setEnabled ( true );

                            ninePatchEditor.setNinePatchImage ( ImageUtils.copy ( image ) );
                            image.flush ();

                            fireStateChanged ();

                            return true;
                        }
                    }
                }
                return false;
            }
        } );

        WebScrollPane editorView = ninePatchEditor.getView ();
        editorView.setBorder (
                BorderFactory.createMatteBorder ( 0, 0, 0, 1, WebToolbarStyle.borderColor ) );

        WebPanel previewView = createPreviewPanel ();
        previewView.setBorder (
                BorderFactory.createMatteBorder ( 0, 1, 0, 0, WebToolbarStyle.borderColor ) );

        WebSplitPane split = new WebSplitPane ( WebSplitPane.HORIZONTAL_SPLIT );
        split.setLeftComponent ( editorView );
        split.setRightComponent ( previewView );
        split.setOneTouchExpandable ( true );
        split.setContinuousLayout ( true );
        split.setResizeWeight ( 1 );
        add ( split, BorderLayout.CENTER );

        ///////////////////////////////////////////////////////////////////////////////////

        toolBar = new WebToolBar ( WebToolBar.HORIZONTAL );
        toolBar.setToolbarStyle ( ToolbarStyle.attached );
        toolBar.setFloatable ( false );
        add ( toolBar, BorderLayout.NORTH );

        WebButton open = WebButton.createIconWebButton ( OPEN_ICON );
        TooltipManager.setTooltip ( open, UIManager.getString ( "NinePatchEditor.openImage" ) );
        HotkeyManager.registerHotkey ( open, Hotkey.CTRL_O );
        open.setRolloverDecoratedOnly ( true );
        open.setDrawFocus ( false );
        open.addActionListener ( new ActionListener ()
        {
            private WebFileChooser wfc = null;

            public void actionPerformed ( ActionEvent e )
            {
                if ( wfc == null )
                {
                    wfc = new WebFileChooser (
                            SwingUtilities.getWindowAncestor ( NinePatchEditorPanel.this ),
                            UIManager.getString ( "NinePatchEditor.openImage.title" ) );
                    wfc.setSelectionMode ( SelectionMode.SINGLE_SELECTION );
                }
                if ( imageSrc != null )
                {
                    wfc.setCurrentDirectory ( imageSrc );
                }
                int done = wfc.showDialog ();
                if ( done == StyleConstants.OK_OPTION )
                {
                    imageSrc = wfc.getSelectedFile ().getAbsolutePath ();
                    ninePatchEditor.setNinePatchImage ( ImageUtils
                            .getBufferedImage ( wfc.getSelectedFile ().getAbsolutePath () ) );
                    save.setEnabled ( true );
                    fireStateChanged ();
                }
            }
        } );
        toolBar.add ( open );

        toolBar.addSeparator ();

        save = WebButton.createIconWebButton ( SAVE_ICON );
        TooltipManager.setTooltip ( save, UIManager.getString ( "NinePatchEditor.saveImage" ) );
        HotkeyManager.registerHotkey ( save, Hotkey.CTRL_S );
        save.setRolloverDecoratedOnly ( true );
        save.setDrawFocus ( false );
        save.setEnabled ( false );
        save.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( imageSrc != null )
                {
                    try
                    {
                        saveImage ( new File ( imageSrc ) );
                    }
                    catch ( IOException e1 )
                    {
                        e1.printStackTrace ();
                    }
                }
            }
        } );
        toolBar.add ( save );

        WebButton saveAs = WebButton.createIconWebButton ( SAVE_AS_ICON );
        TooltipManager.setTooltip ( saveAs, UIManager.getString ( "NinePatchEditor.saveImageAs" ) );
        HotkeyManager.registerHotkey ( saveAs, Hotkey.CTRL_SHIFT_S );
        saveAs.setRolloverDecoratedOnly ( true );
        saveAs.setDrawFocus ( false );
        saveAs.addActionListener ( new ActionListener ()
        {
            private WebFileChooser wfc = null;

            public void actionPerformed ( ActionEvent e )
            {
                if ( wfc == null )
                {
                    wfc = new WebFileChooser (
                            SwingUtilities.getWindowAncestor ( NinePatchEditorPanel.this ),
                            UIManager.getString ( "NinePatchEditor.saveImageAs.title" ) );
                    wfc.setSelectionMode ( SelectionMode.SINGLE_SELECTION );
                }
                if ( imageSrc != null )
                {
                    wfc.setCurrentDirectory ( imageSrc );
                }
                int done = wfc.showDialog ();
                if ( done == StyleConstants.OK_OPTION )
                {
                    try
                    {
                        saveImage ( wfc.getSelectedFile () );
                        save.setEnabled ( true );
                    }
                    catch ( IOException e1 )
                    {
                        e1.printStackTrace ();
                    }
                }
            }
        } );
        toolBar.add ( saveAs );

        toolBar.addSeparator ();

        WebButton copy = WebButton.createIconWebButton ( COPY_ICON );
        TooltipManager.setTooltip ( copy, UIManager.getString ( "NinePatchEditor.copyInfo" ) );
        HotkeyManager.registerHotkey ( copy, Hotkey.CTRL_C );
        copy.setRolloverDecoratedOnly ( true );
        copy.setDrawFocus ( false );
        copy.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( !getNinePatchEditor ().isSomeDragged () )
                {
                    NinePatchIcon npi = getNinePatchEditor ().getNinePatchIcon ();
                    NinePatchInfo info = new NinePatchInfo ();
                    info.setHorizontalStretch ( npi.getHorizontalStretch () );
                    info.setVerticalStretch ( npi.getVerticalStretch () );
                    info.setMargin ( npi.getMargin () );
                    info.setImageSize ( getRealImageSize ( npi ) );
                    Toolkit.getDefaultToolkit ().getSystemClipboard ().setContents (
                            new StringSelection ( SettingsManager.getXStream ().toXML ( info ) ),
                            null );
                }
            }
        } );
        toolBar.add ( copy );

        WebButton paste = WebButton.createIconWebButton ( PASTE_ICON );
        TooltipManager.setTooltip ( paste, UIManager.getString ( "NinePatchEditor.pasteInfo" ) );
        HotkeyManager.registerHotkey ( paste, Hotkey.CTRL_V );
        paste.setRolloverDecoratedOnly ( true );
        paste.setDrawFocus ( false );
        paste.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( !getNinePatchEditor ().isSomeDragged () )
                {
                    try
                    {
                        Transferable t = Toolkit.getDefaultToolkit ().getSystemClipboard ()
                                .getContents ( null );
                        if ( t != null && t.isDataFlavorSupported ( DataFlavor.stringFlavor ) )
                        {
                            // Retrieving data from clipboard
                            String data = ( String ) t.getTransferData ( DataFlavor.stringFlavor );
                            NinePatchInfo info = ( NinePatchInfo ) SettingsManager.getXStream ()
                                    .fromXML ( data );

                            // Restoring data if it fits size
                            NinePatchIcon npi = getNinePatchEditor ().getNinePatchIcon ();
                            if ( info != null &&
                                    info.getImageSize ().equals ( getRealImageSize ( npi ) ) )
                            {
                                npi.setHorizontalStretch ( info.getHorizontalStretch () );
                                npi.setVerticalStretch ( info.getVerticalStretch () );
                                npi.setMargin ( info.getMargin () );
                                getNinePatchEditor ().setNinePatchIcon ( npi );
                            }
                        }
                    }
                    catch ( Throwable ex )
                    {
                        //
                    }
                }
            }
        } );
        toolBar.add ( paste );


        //        final WebToggleButton showPreviewDialog =
        //                WebToggleButton.createIconWebButton ( PREVIEW_ICON );
        //        showPreviewDialog.setText ( UIManager.getString ( "NinePatchEditor.showPreview" ) );
        //        showPreviewDialog.setRolloverDecoratedOnly ( true );
        //        showPreviewDialog.setDrawFocus ( false );
        //        showPreviewDialog.setSelected ( false );
        //        showPreviewDialog.addActionListener ( new ActionListener ()
        //        {
        //        } );
        //        toolBar.addToEnd ( showPreviewDialog );


        boolean si = SettingsManager.getBoolean ( "NinePatchEditor", "preview.showIcon", false );
        final WebToggleButton showIcon = WebToggleButton.createIconWebButton ( SHOW_ICON_ICON );
        TooltipManager.setTooltip ( showIcon,
                UIManager.getString ( "NinePatchEditor.preview.showIcon" ) );
        showIcon.setDrawFocus ( false );
        showIcon.setRolloverDecoratedOnly ( true );
        preview.setIcon ( si ? ICON : null );
        showIcon.setSelected ( si );
        showIcon.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                boolean show = showIcon.isSelected ();
                SettingsManager.set ( "NinePatchEditor", "preview.showIcon", show );
                preview.setIcon ( show ? ICON : null );
            }
        } );
        toolBar.addToEnd ( showIcon );

        boolean st = SettingsManager.getBoolean ( "NinePatchEditor", "preview.showText", true );
        final WebToggleButton showText = WebToggleButton.createIconWebButton ( SHOW_TEXT_ICON );
        TooltipManager.setTooltip ( showText,
                UIManager.getString ( "NinePatchEditor.preview.showText" ) );
        showText.setCursor ( Cursor.getDefaultCursor () );
        showText.setDrawFocus ( false );
        showText.setRolloverDecoratedOnly ( true );
        showText.setRolloverDarkBorderOnly ( false );
        showText.setDrawRight ( false );
        showText.setDrawRightLine ( true );
        showText.putClientProperty ( ToolbarLayout.IGNORE_BORDER, true );
        showText.setSelected ( st );

        final WebTextField textField = new WebTextField ( 8 );
        textField.setText ( SettingsManager.getString ( "NinePatchEditor", "preview.text",
                UIManager.getString ( "NinePatchEditor.preview.text" ) ) );
        preview.setText ( st ? parseToMultilineHtml ( textField.getText () ) : "" );
        textField.setHorizontalAlignment ( WebTextField.CENTER );
        textField.setDrawFocus ( false );
        textField.setEditable ( st );
        textField.addCaretListener ( new CaretListener ()
        {
            public void caretUpdate ( CaretEvent e )
            {
                if ( showText.isSelected () )
                {
                    SettingsManager.set ( "NinePatchEditor", "preview.text", textField.getText () );
                    preview.setText ( parseToMultilineHtml ( textField.getText () ) );
                }
            }
        } );
        showText.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                boolean show = showText.isSelected ();
                SettingsManager.set ( "NinePatchEditor", "preview.showText", show );
                textField.setEditable ( show );
                preview.setText ( show ? parseToMultilineHtml ( textField.getText () ) : "" );
            }
        } );
        textField.setLeadingComponent ( showText );
        toolBar.addToEnd ( textField );


        final WebButton chooseColor = WebButton.createIconWebButton ( FOREGROUND_COLOR_ICON );
        TooltipManager.setTooltip ( chooseColor,
                UIManager.getString ( "NinePatchEditor.preview.foregroundColor" ) );
        preview.setForeground ( SettingsManager
                .getColor ( "NinePatchEditor", "preview.foregroundColor", Color.WHITE ) );
        chooseColor.setCursor ( Cursor.getDefaultCursor () );
        chooseColor.setDrawFocus ( false );
        chooseColor.setRolloverDecoratedOnly ( true );
        chooseColor.setRolloverDarkBorderOnly ( false );
        chooseColor.setDrawLeft ( false );
        chooseColor.setDrawLeftLine ( true );
        chooseColor.putClientProperty ( ToolbarLayout.IGNORE_BORDER, true );
        chooseColor.addActionListener ( new ActionListener ()
        {
            private WebColorChooser webColorChooser = null;

            public void actionPerformed ( ActionEvent e )
            {
                if ( webColorChooser == null )
                {
                    webColorChooser = new WebColorChooser (
                            SwingUtilities.getWindowAncestor ( previewPanel ) );
                    webColorChooser.setColor ( preview.getForeground () );
                }
                if ( webColorChooser.showDialog () == StyleConstants.OK_OPTION )
                {
                    Color nfg = webColorChooser.getColor ();
                    SettingsManager.set ( "NinePatchEditor", "preview.foregroundColor", nfg );
                    preview.setForeground ( nfg );
                }
            }
        } );
        textField.setTrailingComponent ( chooseColor );

        abp = new AlphaBackgroundPainter ();

        boolean da = SettingsManager
                .getBoolean ( "NinePatchEditor", "preview.transparentBackground", true );
        final WebToggleButton drawAlphaBackground =
                WebToggleButton.createIconWebButton ( TRANSPARENT_ICON );
        TooltipManager.setTooltip ( drawAlphaBackground,
                UIManager.getString ( "NinePatchEditor.preview.transparentBackground" ) );
        drawAlphaBackground.setSelected ( da );
        drawAlphaBackground.setDrawFocus ( false );
        drawAlphaBackground.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                SettingsManager.set ( "NinePatchEditor", "preview.transparentBackground", true );
                previewPanel.setBackgroundPainter ( abp );
            }
        } );

        cbp = new ColorBackgroundPainter ( SettingsManager
                .getColor ( "NinePatchEditor", "preview.backgroundColor", Color.WHITE ) );

        final WebToggleButton drawColoredBackground = WebToggleButton
                .createIconWebButton ( ImageUtils.createColorIcon ( cbp.getColor () ) );
        TooltipManager.setTooltip ( drawColoredBackground,
                UIManager.getString ( "NinePatchEditor.preview.coloredBackground" ) );
        drawColoredBackground.setSelected ( !da );
        drawColoredBackground.setDrawFocus ( false );
        drawColoredBackground.addActionListener ( new ActionListener ()
        {
            private WebColorChooser wcc = null;

            public void actionPerformed ( ActionEvent e )
            {
                SettingsManager.set ( "NinePatchEditor", "preview.transparentBackground", false );

                if ( wcc == null )
                {
                    wcc = new WebColorChooser ( SwingUtilities.getWindowAncestor ( previewPanel ) );
                    wcc.setColor ( cbp.getColor () );
                }
                wcc.setVisible ( true );

                if ( wcc.getResult () == StyleConstants.OK_OPTION )
                {
                    Color color = wcc.getColor ();
                    SettingsManager.set ( "NinePatchEditor", "preview.backgroundColor", color );
                    drawColoredBackground.setIcon ( ImageUtils.createColorIcon ( color ) );
                    cbp.setColor ( color );
                }

                previewPanel.setBackgroundPainter ( cbp );
            }
        } );

        previewPanel.setBackgroundPainter ( da ? abp : cbp );

        ButtonGroup bgGroup = new ButtonGroup ();
        bgGroup.add ( drawAlphaBackground );
        bgGroup.add ( drawColoredBackground );

        toolBar.addToEnd ( new WebButtonGroup ( drawAlphaBackground, drawColoredBackground ) );

        ///////////////////////////////////////////////////////////////////////////////////

        WebToolBar bottomToolBar = new WebToolBar ( WebToolBar.HORIZONTAL );
        bottomToolBar.setToolbarStyle ( ToolbarStyle.attached );
        bottomToolBar.setFloatable ( false );
        bottomToolBar.setComponentOrientation ( ComponentOrientation.RIGHT_TO_LEFT );
        add ( bottomToolBar, BorderLayout.SOUTH );

        boolean sgs = SettingsManager.getBoolean ( "NinePatchEditor", "showSpacing", true );
        final WebToggleButton showGuidesSpacing =
                WebToggleButton.createIconWebButton ( GUIDES_ICON );
        TooltipManager.setTooltip ( showGuidesSpacing,
                UIManager.getString ( "NinePatchEditor.showSpacing" ) );
        showGuidesSpacing.setRolloverDecoratedOnly ( true );
        showGuidesSpacing.setDrawFocus ( false );
        showGuidesSpacing.setSelected ( sgs );
        ninePatchEditor.setShowGuideSpacing ( sgs );
        showGuidesSpacing.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                boolean sgs = showGuidesSpacing.isSelected ();
                SettingsManager.set ( "NinePatchEditor", "showSpacing", sgs );
                ninePatchEditor.setShowGuideSpacing ( sgs );
            }
        } );

        boolean sr = SettingsManager.getBoolean ( "NinePatchEditor", "showRuler", true );
        final WebToggleButton showRuler = WebToggleButton.createIconWebButton ( RULER_ICON );
        TooltipManager
                .setTooltip ( showRuler, UIManager.getString ( "NinePatchEditor.showRuler" ) );
        showRuler.setRolloverDecoratedOnly ( true );
        showRuler.setDrawFocus ( false );
        showRuler.setSelected ( sr );
        ninePatchEditor.setShowRuler ( sr );
        showRuler.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                boolean sr = showRuler.isSelected ();
                SettingsManager.set ( "NinePatchEditor", "showRuler", sr );
                ninePatchEditor.setShowRuler ( sr );
            }
        } );

        final boolean fc = SettingsManager.getBoolean ( "NinePatchEditor", "fillContent", true );
        final WebToggleButton fillContent = WebToggleButton.createIconWebButton ( CONTENT_ICON );
        TooltipManager
                .setTooltip ( fillContent, UIManager.getString ( "NinePatchEditor.fillContent" ) );
        fillContent.setRolloverDecoratedOnly ( true );
        fillContent.setDrawFocus ( false );
        fillContent.setSelected ( fc );
        ninePatchEditor.setFillContentArea ( fc );
        fillContent.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                boolean fc = fillContent.isSelected ();
                SettingsManager.set ( "NinePatchEditor", "fillContent", fc );
                ninePatchEditor.setFillContentArea ( fc );
            }
        } );

        final boolean fs = SettingsManager.getBoolean ( "NinePatchEditor", "fillStretch", true );
        final WebToggleButton fillStretch = WebToggleButton.createIconWebButton ( STRETCH_ICON );
        TooltipManager
                .setTooltip ( fillStretch, UIManager.getString ( "NinePatchEditor.fillStretch" ) );
        fillStretch.setRolloverDecoratedOnly ( true );
        fillStretch.setDrawFocus ( false );
        fillStretch.setSelected ( fs );
        ninePatchEditor.setFillStretchAreas ( fs );
        fillStretch.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                boolean fs = fillStretch.isSelected ();
                SettingsManager.set ( "NinePatchEditor", "fillStretch", fs );
                ninePatchEditor.setFillStretchAreas ( fs );
            }
        } );

        bottomToolBar.add ( showGuidesSpacing );
        bottomToolBar.add ( showRuler );
        bottomToolBar.addSeparator ();
        bottomToolBar.add ( fillContent );
        bottomToolBar.add ( fillStretch );

        //

        final WebSlider zoomSlider =
                new WebSlider ( NinePatchEditor.MIN_ZOOM, NinePatchEditor.MAX_ZOOM,
                        ninePatchEditor.getZoom () )
                {
                    public Dimension getPreferredSize ()
                    {
                        Dimension ps = super.getPreferredSize ();
                        ps.width = 170;
                        return ps;
                    }
                };
        zoomSlider.setAngledThumb ( false );
        zoomSlider.setPaintTicks ( false );
        zoomSlider.setPaintLabels ( false );
        changeListener = new ChangeListener ()
        {
            public void stateChanged ( ChangeEvent e )
            {
                ninePatchEditor.removeZoomChangeListener ( zoomChangeListener );
                ninePatchEditor.setZoom ( zoomSlider.getValue () );
                ninePatchEditor.addZoomChangeListener ( zoomChangeListener );
            }
        };
        zoomSlider.addChangeListener ( changeListener );
        zoomChangeListener = new ZoomChangeListener ()
        {
            public void zoomChanged ()
            {
                zoomSlider.removeChangeListener ( changeListener );
                zoomSlider.setValue ( ninePatchEditor.getZoom () );
                zoomSlider.addChangeListener ( changeListener );
            }
        };
        ninePatchEditor.addZoomChangeListener ( zoomChangeListener );

        WebButton minZoom = WebButton.createIconWebButton ( MIN_ICON );
        minZoom.setRolloverDecoratedOnly ( true );
        minZoom.setDrawFocus ( false );
        minZoom.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                zoomSlider.setValue ( NinePatchEditor.MIN_ZOOM );
            }
        } );

        WebButton maxZoom = WebButton.createIconWebButton ( MAX_ICON );
        maxZoom.setRolloverDecoratedOnly ( true );
        maxZoom.setDrawFocus ( false );
        maxZoom.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                zoomSlider.setValue ( NinePatchEditor.MAX_ZOOM );
            }
        } );

        bottomToolBar.addToEnd ( minZoom );
        bottomToolBar.addToEnd ( zoomSlider );
        bottomToolBar.addToEnd ( maxZoom );
    }

    private Dimension getRealImageSize ( NinePatchIcon npi )
    {
        return new Dimension ( npi.getImage ().getWidth (), npi.getImage ().getHeight () );
    }

    private WebPanel createPreviewPanel ()
    {
        previewPanel = new WebPanel ();
        previewPanel.setLayout ( new TableLayout (
                new double[][]{ { TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL },
                        { TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL } } ) );
        previewPanel.setPreferredSize ( new Dimension ( 230, 230 ) );
        previewPanel.setBackgroundPainter ( abp );

        preview = new WebLabel ( "", WebLabel.CENTER );
        final ResizablePanel resizablePanel = new ResizablePanel ( preview );
        previewPanel.add ( resizablePanel, "1,1" );

        updatePreviews ();
        getNinePatchEditor ().addChangeListener ( new ChangeListener ()
        {
            public void stateChanged ( ChangeEvent e )
            {
                updatePreviews ();
            }
        } );

        return new BorderPanel ( previewPanel, ( Border ) null );
    }

    private String parseToMultilineHtml ( String text )
    {
        return "<html><center>" + text.replaceAll ( ";", "<br>" ).replaceAll ( "\\\\n", "<br>" ) +
                "</center></html>";
    }

    private void updatePreviews ()
    {
        NinePatchBackgroundPainter bp =
                new NinePatchBackgroundPainter ( getNinePatchEditor ().getNinePatchIcon () );
        preview.setBackgroundPainter ( bp );
        previewPanel.revalidate ();
    }

    private void saveImage ( File imageFile ) throws IOException
    {
        // Checking file name
        String fullName = getSaveFileName ( imageFile );
        imageFile = new File ( imageFile.getParentFile (), fullName );
        imageSrc = imageFile.getAbsolutePath ();

        // Saving image
        BufferedImage image = ninePatchEditor.getNinePatchImage ();
        ImageIO.write ( image, "png", imageFile );

        // Inform about changes
        fireStateChanged ();
    }

    private String getSaveFileName ( File imageFile )
    {
        // Adding .9 to file name if it doesn't exist
        String format = ".png";
        String subFormat = ".9";
        String fullName = imageFile.getName ();
        if ( fullName.endsWith ( format ) )
        {
            String name = fullName.substring ( 0, fullName.lastIndexOf ( format ) );
            if ( !name.endsWith ( subFormat ) )
            {
                fullName = name + subFormat + format;
            }
        }
        else if ( fullName.endsWith ( subFormat ) )
        {
            fullName = fullName + format;
        }
        else
        {
            int dot = fullName.lastIndexOf ( "." );
            if ( dot != -1 )
            {
                fullName = fullName.substring ( 0, dot );
            }
            fullName = fullName + subFormat + format;
        }
        return fullName;
    }

    public NinePatchEditor getNinePatchEditor ()
    {
        return ninePatchEditor;
    }

    public String getImageSrc ()
    {
        return imageSrc;
    }

    public void setNinePatchImage ( BufferedImage ninePatchImage )
    {
        ninePatchEditor.setNinePatchImage ( ninePatchImage );
    }

    public BufferedImage getNinePatchImage ()
    {
        return ninePatchEditor.getNinePatchImage ();
    }

    public List<ChangeListener> getChangeListeners ()
    {
        return changeListeners;
    }

    public void addChangeListener ( ChangeListener changeListener )
    {
        changeListeners.add ( changeListener );
    }

    public void removeChangeListener ( ChangeListener changeListener )
    {
        changeListeners.add ( changeListener );
    }

    private void fireStateChanged ()
    {
        ChangeEvent changeEvent = new ChangeEvent ( NinePatchEditorPanel.this );
        List<ChangeListener> changeListenersC = new ArrayList<ChangeListener> ();
        changeListenersC.addAll ( changeListeners );
        for ( ChangeListener cl : changeListenersC )
        {
            cl.stateChanged ( changeEvent );
        }
    }
}
