/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.panel;

import com.alee.extended.layout.HorizontalFlowLayout;
import com.alee.extended.layout.VerticalFlowLayout;
import com.alee.laf.button.WebButtonUI;
import com.alee.laf.panel.WebPanel;

import javax.swing.*;
import java.awt.*;

/**
 * User: mgarin Date: 11/28/11 Time: 5:07 PM
 */

public class WebButtonGroup extends WebPanel implements SwingConstants
{
    private int orientation;

    public WebButtonGroup ( JComponent... component )
    {
        this ( HORIZONTAL, component );
    }

    public WebButtonGroup ( int orientation, JComponent... components )
    {
        super ();

        setOpaque ( false );

        // Update orientation and layout
        setOrientation ( orientation );

        // Adding component to panel   
        if ( components != null )
        {
            for ( JComponent component : components )
            {
                add ( component );
            }
        }

        // Updating UIs
        updateButtons ();
    }

    public int getOrientation ()
    {
        return orientation;
    }

    public void setOrientation ( int orientation )
    {
        this.orientation = orientation;

        // Updating layout
        boolean horizontal = orientation == HORIZONTAL;
        setLayout ( horizontal ? new HorizontalFlowLayout ( 0, false ) :
                new VerticalFlowLayout ( VerticalFlowLayout.TOP, 0, 0, true, false ) );

        // Updating UIs
        updateButtons ();
    }

    public void updateButtons ()
    {
        Component[] components = getComponents ();
        boolean horizontal = orientation == HORIZONTAL;
        boolean lastWasButton = false;
        for ( int i = 0; i < components.length; i++ )
        {
            if ( isWebStyledButton ( components[ i ] ) )
            {
                // Web UI
                WebButtonUI ui = ( WebButtonUI ) ( ( AbstractButton ) components[ i ] ).getUI ();

                // Hiding left side
                if ( lastWasButton )
                {
                    if ( horizontal )
                    {
                        ui.setDrawLeft ( false );
                        ui.setDrawLeftLine ( false );
                    }
                    else
                    {
                        ui.setDrawTop ( false );
                        ui.setDrawTopLine ( false );
                    }
                }

                // Hiding right side
                if ( i < components.length - 1 && isWebStyledButton ( components[ i + 1 ] ) )
                {
                    if ( horizontal )
                    {
                        ui.setDrawRight ( false );
                        ui.setDrawRightLine ( true );
                    }
                    else
                    {
                        ui.setDrawBottom ( false );
                        ui.setDrawBottomLine ( true );
                    }
                }

                lastWasButton = true;
            }
            else if ( isWebButtonGroup ( components[ i ] ) )
            {
                // WebButtonGroup
                WebButtonGroup wbg = ( WebButtonGroup ) components[ i ];

                // Hiding left side
                if ( lastWasButton )
                {
                    for ( Component component : wbg.getComponents () )
                    {
                        if ( isWebStyledButton ( component ) )
                        {
                            WebButtonUI ui =
                                    ( WebButtonUI ) ( ( AbstractButton ) component ).getUI ();
                            if ( horizontal )
                            {
                                ui.setDrawLeft ( false );
                                ui.setDrawLeftLine ( false );
                            }
                            else
                            {
                                ui.setDrawTop ( false );
                                ui.setDrawTopLine ( false );
                            }
                        }
                    }
                }

                // Hiding right side
                if ( i < components.length - 1 && isWebButtonGroup ( components[ i + 1 ] ) )
                {
                    for ( Component component : wbg.getComponents () )
                    {
                        if ( isWebStyledButton ( component ) )
                        {
                            WebButtonUI ui =
                                    ( WebButtonUI ) ( ( AbstractButton ) component ).getUI ();
                            if ( horizontal )
                            {
                                ui.setDrawRight ( false );
                                ui.setDrawRightLine ( true );
                            }
                            else
                            {
                                ui.setDrawBottom ( false );
                                ui.setDrawBottomLine ( true );
                            }
                        }
                    }
                }

                lastWasButton = true;
            }
            else
            {
                lastWasButton = false;
            }

            // Adding component to panel
            add ( components[ i ] );
        }
    }

    private boolean isWebStyledButton ( Component component )
    {
        return component instanceof AbstractButton &&
                ( ( AbstractButton ) component ).getUI () instanceof WebButtonUI;
    }

    private boolean isWebButtonGroup ( Component component )
    {
        return component instanceof WebButtonGroup;
    }
}
