/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf;

import com.alee.NonObfuscable;
import com.alee.laf.button.WebButtonUI;
import com.alee.laf.button.WebToggleButtonUI;
import com.alee.laf.checkbox.WebCheckBoxUI;
import com.alee.laf.colorchooser.WebColorChooserUI;
import com.alee.laf.combobox.WebComboBoxUI;
import com.alee.laf.desktoppane.WebDesktopIconUI;
import com.alee.laf.desktoppane.WebDesktopPaneUI;
import com.alee.laf.desktoppane.WebInternalFrameUI;
import com.alee.laf.filechooser.WebFileChooserUI;
import com.alee.laf.label.WebLabelUI;
import com.alee.laf.list.WebListCellRenderer;
import com.alee.laf.list.WebListUI;
import com.alee.laf.menu.*;
import com.alee.laf.optionpane.WebOptionPaneUI;
import com.alee.laf.panel.WebPanelUI;
import com.alee.laf.progressbar.WebProgressBarUI;
import com.alee.laf.radiobutton.WebRadioButtonUI;
import com.alee.laf.rootpane.WebRootPaneUI;
import com.alee.laf.scroll.WebScrollBarUI;
import com.alee.laf.scroll.WebScrollPaneUI;
import com.alee.laf.separator.WebSeparatorUI;
import com.alee.laf.slider.WebSliderUI;
import com.alee.laf.spinner.WebSpinnerUI;
import com.alee.laf.splitpane.WebSplitPaneUI;
import com.alee.laf.tabbedpane.WebTabbedPaneUI;
import com.alee.laf.table.WebTableCorner;
import com.alee.laf.table.WebTableHeaderUI;
import com.alee.laf.table.WebTableUI;
import com.alee.laf.text.*;
import com.alee.laf.toolbar.WebToolBarSeparatorUI;
import com.alee.laf.toolbar.WebToolBarUI;
import com.alee.laf.tooltip.WebToolTipUI;
import com.alee.laf.tree.WebTreeUI;
import com.alee.laf.viewport.WebViewportUI;
import com.alee.managers.focus.FocusManager;
import com.alee.managers.hotkey.HotkeyManager;
import com.alee.managers.proxy.ProxyManager;
import com.alee.managers.settings.SettingsManager;
import com.alee.managers.tooltip.TooltipManager;
import jsyntaxpane.DefaultSyntaxKit;
import sun.swing.SwingLazyValue;
import sun.swing.SwingUtilities2;

import javax.swing.*;
import javax.swing.plaf.InsetsUIResource;
import javax.swing.plaf.basic.BasicLookAndFeel;
import javax.swing.text.DefaultEditorKit;
import java.awt.*;
import java.util.Locale;

/**
 * User: mgarin Date: 27.04.11 Time: 17:40
 */

public class WebLookAndFeel extends BasicLookAndFeel implements NonObfuscable
{
    /*
    * Reassignable LookAndFeel UI class names
    */

    // Label
    public static String labelUI = WebLabelUI.class.getCanonicalName ();
    public static String toolTipUI = WebToolTipUI.class.getCanonicalName ();

    // Button
    public static String buttonUI = WebButtonUI.class.getCanonicalName ();
    public static String toggleButtonUI = WebToggleButtonUI.class.getCanonicalName ();
    public static String checkBoxUI = WebCheckBoxUI.class.getCanonicalName ();
    public static String radioButtonUI = WebRadioButtonUI.class.getCanonicalName ();

    // Menu
    public static String menuBarUI = WebMenuBarUI.class.getCanonicalName ();
    public static String menuUI = WebMenuUI.class.getCanonicalName ();
    public static String popupMenuUI = WebPopupMenuUI.class.getCanonicalName ();
    public static String menuItemUI = WebMenuItemUI.class.getCanonicalName ();
    public static String checkBoxMenuItemUI = WebCheckBoxMenuItemUI.class.getCanonicalName ();
    public static String radioButtonMenuItemUI = WebRadioButtonMenuItemUI.class.getCanonicalName ();
    public static String popupMenuSeparatorUI = WebPopupMenuSeparatorUI.class.getCanonicalName ();

    // Separator
    public static String separatorUI = WebSeparatorUI.class.getCanonicalName ();

    // Scroll
    public static String scrollBarUI = WebScrollBarUI.class.getCanonicalName ();
    public static String scrollPaneUI = WebScrollPaneUI.class.getCanonicalName ();

    // Text
    public static String textFieldUI = WebTextFieldUI.class.getCanonicalName ();
    public static String passwordFieldUI = WebPasswordFieldUI.class.getCanonicalName ();
    public static String formattedTextFieldUI = WebFormattedTextFieldUI.class.getCanonicalName ();
    public static String textAreaUI = WebTextAreaUI.class.getCanonicalName ();
    public static String editorPaneUI = WebEditorPaneUI.class.getCanonicalName ();
    public static String textPaneUI = WebTextPaneUI.class.getCanonicalName ();

    // Toolbar
    public static String toolBarUI = WebToolBarUI.class.getCanonicalName ();
    public static String toolBarSeparatorUI = WebToolBarSeparatorUI.class.getCanonicalName ();

    // Table
    public static String tableUI = WebTableUI.class.getCanonicalName ();
    public static String tableHeaderUI = WebTableHeaderUI.class.getCanonicalName ();

    // Chooser
    public static String colorChooserUI = WebColorChooserUI.class.getCanonicalName ();
    public static String fileChooserUI = WebFileChooserUI.class.getCanonicalName ();

    // Container
    public static String panelUI = WebPanelUI.class.getCanonicalName ();
    public static String viewportUI = WebViewportUI.class.getCanonicalName ();
    public static String rootPaneUI = WebRootPaneUI.class.getCanonicalName ();
    public static String tabbedPaneUI = WebTabbedPaneUI.class.getCanonicalName ();
    public static String splitPaneUI = WebSplitPaneUI.class.getCanonicalName ();

    // Complex components
    public static String progressBarUI = WebProgressBarUI.class.getCanonicalName ();
    public static String sliderUI = WebSliderUI.class.getCanonicalName ();
    public static String spinnerUI = WebSpinnerUI.class.getCanonicalName ();
    public static String treeUI = WebTreeUI.class.getCanonicalName ();
    public static String listUI = WebListUI.class.getCanonicalName ();
    public static String comboBoxUI = WebComboBoxUI.class.getCanonicalName ();

    // Desktop pane
    public static String desktopPaneUI = WebDesktopPaneUI.class.getCanonicalName ();
    public static String descktopIconUI = WebDesktopIconUI.class.getCanonicalName ();
    public static String internalFrameUI = WebInternalFrameUI.class.getCanonicalName ();

    // Option pane
    public static String optionPaneUI = WebOptionPaneUI.class.getCanonicalName ();

    /*
    * Look and Feel description
    */

    public String getName ()
    {
        return "WebLookAndFeel";
    }

    public String getID ()
    {
        return "WebLookAndFeel";
    }

    public String getDescription ()
    {
        return "Cross-platform stylish Look and Feel";
    }

    public boolean isNativeLookAndFeel ()
    {
        return false;
    }

    public boolean isSupportedLookAndFeel ()
    {
        return true;
    }

    public boolean getSupportsWindowDecorations ()
    {
        return false;
    }

    /*
    * Styles initialization
    */

    public UIDefaults getDefaults ()
    {
        // Initializing UIDefaults
        UIDefaults defaults = super.getDefaults ();

        // Initializing specific WebLookAndFeel styles
        initializeStyles ();

        return defaults;
    }

    protected void initClassDefaults ( UIDefaults table )
    {
        // Label
        table.put ( "LabelUI", labelUI );
        table.put ( "ToolTipUI", toolTipUI );

        // Button
        table.put ( "ButtonUI", buttonUI );
        table.put ( "ToggleButtonUI", toggleButtonUI );
        table.put ( "CheckBoxUI", checkBoxUI );
        table.put ( "RadioButtonUI", radioButtonUI );

        // Menu
        table.put ( "MenuBarUI", menuBarUI );
        table.put ( "MenuUI", menuUI );
        table.put ( "PopupMenuUI", popupMenuUI );
        table.put ( "MenuItemUI", menuItemUI );
        table.put ( "CheckBoxMenuItemUI", checkBoxMenuItemUI );
        table.put ( "RadioButtonMenuItemUI", radioButtonMenuItemUI );
        table.put ( "PopupMenuSeparatorUI", popupMenuSeparatorUI );

        // Separator
        table.put ( "SeparatorUI", separatorUI );

        // Scroll
        table.put ( "ScrollBarUI", scrollBarUI );
        table.put ( "ScrollPaneUI", scrollPaneUI );

        // Text
        table.put ( "TextFieldUI", textFieldUI );
        table.put ( "PasswordFieldUI", passwordFieldUI );
        table.put ( "FormattedTextFieldUI", formattedTextFieldUI );
        table.put ( "TextAreaUI", textAreaUI );
        table.put ( "EditorPaneUI", editorPaneUI );
        table.put ( "TextPaneUI", textPaneUI );

        // Toolbar
        table.put ( "ToolBarUI", toolBarUI );
        table.put ( "ToolBarSeparatorUI", toolBarSeparatorUI );

        // Table
        table.put ( "TableUI", tableUI );
        table.put ( "TableHeaderUI", tableHeaderUI );

        // Chooser
        table.put ( "ColorChooserUI", colorChooserUI );
        table.put ( "FileChooserUI", fileChooserUI );

        // Container
        table.put ( "PanelUI", panelUI );
        table.put ( "ViewportUI", viewportUI );
        table.put ( "RootPaneUI", rootPaneUI );
        table.put ( "TabbedPaneUI", tabbedPaneUI );
        table.put ( "SplitPaneUI", splitPaneUI );

        // Complex components
        table.put ( "ProgressBarUI", progressBarUI );
        table.put ( "SliderUI", sliderUI );
        table.put ( "SpinnerUI", spinnerUI );
        table.put ( "TreeUI", treeUI );
        table.put ( "ListUI", listUI );
        table.put ( "ComboBoxUI", comboBoxUI );

        // Desktop pane
        table.put ( "DesktopPaneUI", desktopPaneUI );
        table.put ( "DesktopIconUI", descktopIconUI );
        table.put ( "InternalFrameUI", internalFrameUI );

        // Option pane
        table.put ( "OptionPaneUI", optionPaneUI );
    }

    protected void initComponentDefaults ( UIDefaults table )
    {
        super.initComponentDefaults ( table );

        // Initializing specific WebLookAndFeel defaults
        initializeWebDefaults ();
    }

    public void initialize ()
    {
        super.initialize ();

        // Listening to ALT key for menubar quick focusing
        KeyboardFocusManager.getCurrentKeyboardFocusManager ().
                addKeyEventPostProcessor ( WebRootPaneUI.altProcessor );
    }

    /*
    * Methods for proper Alt-focus behavior in menubar
    */

    private static boolean isMnemonicHidden = true;

    public static void setMnemonicHidden ( boolean hide )
    {
        isMnemonicHidden = !UIManager.getBoolean ( "Button.showMnemonics" ) && hide;
    }

    public static boolean isMnemonicHidden ()
    {
        if ( UIManager.getBoolean ( "Button.showMnemonics" ) )
        {
            isMnemonicHidden = false;
        }
        return isMnemonicHidden;
    }

    /*
    * Static LookAndFeel installation method for simple and quick usage
    */

    public static boolean install ()
    {
        return install ( false );
    }

    public static boolean isInstalled ()
    {
        return UIManager.getLookAndFeel ().getClass ().getCanonicalName ()
                .equals ( WebLookAndFeel.class.getCanonicalName () );
    }

    public static boolean install ( boolean updateExistingComponents )
    {
        try
        {
            // Installing LookAndFeel
            UIManager.setLookAndFeel ( new WebLookAndFeel () );

            // Updating already created components tree
            if ( updateExistingComponents )
            {
                for ( Window window : Window.getWindows () )
                {
                    SwingUtilities.updateComponentTreeUI ( window );
                }
            }

            // LookAndFeel installed sucessfully
            return true;
        }
        catch ( Throwable e )
        {
            // LookAndFeel installation failed
            return false;
        }
    }

    /*
    * WebLookAndFeel defaults like standard tree/list renderers, component borders and such
    * initialization method. This method will be called only in case WebLookAndFeel is installed
    * through UIManager as current application LookAndFeel and will change values which might affect
    * other than WebLookAndFeel LookAndFeels behavior or even break them.
    */

    private void initializeWebDefaults ()
    {
        // Global text antialiasing
        UIManager.put ( SwingUtilities2.AA_TEXT_PROPERTY_KEY,
                SwingUtilities2.AATextInfo.getAATextInfo ( true ) );

        // Proper locale for language
        Locale.setDefault ( new Locale ( StyleConstants.language ) );

        // Tree icons
        UIManager.put ( "Tree.closedIcon", WebTreeUI.CLOSED_ICON );
        UIManager.put ( "Tree.openIcon", WebTreeUI.OPEN_ICON );
        UIManager.put ( "Tree.leafIcon", WebTreeUI.LEAF_ICON );
        // Tree default selection style
        UIManager.put ( "Tree.textForeground", Color.BLACK );
        UIManager.put ( "Tree.textBackground", StyleConstants.transparent );
        UIManager.put ( "Tree.selectionForeground", Color.BLACK );
        UIManager.put ( "Tree.selectionBackground", StyleConstants.transparent );
        UIManager.put ( "Tree.selectionBorderColor", StyleConstants.transparent );
        // Tree default renderer content margins
        UIManager.put ( "Tree.rendererMargins", new InsetsUIResource ( 5, 4, 4, 6 ) );
        UIManager.put ( "Tree.rendererFillBackground", Boolean.FALSE );
        UIManager.put ( "Tree.drawsFocusBorderAroundIcon", Boolean.FALSE );
        UIManager.put ( "Tree.drawDashedFocusIndicator", Boolean.FALSE );

        // Accelerator font
        UIManager.put ( "MenuItem.acceleratorFont", new Font ( "Arial", Font.PLAIN, 11 ) );
        // Disabled foreground color
        UIManager.put ( "MenuItem.disabledForeground", StyleConstants.disabledTextColor );
        // We don't need uneven heights
        UIManager.put ( "MenuItem.evenHeight", Boolean.TRUE );

        // JMenu expand spacing
        // Up-down menu expand
        UIManager.put ( "Menu.menuPopupOffsetX", StyleConstants.shadeWidth );
        UIManager.put ( "Menu.menuPopupOffsetY", -StyleConstants.shadeWidth / 2 );
        // Left-right menu expand
        UIManager.put ( "Menu.submenuPopupOffsetX", StyleConstants.shadeWidth / 2 );
        UIManager.put ( "Menu.submenuPopupOffsetY", 0 );
        // Menu item colors
        //        UIManager.put ( "MenuItem.background", StyleConstants.transparent );
        //        UIManager.put ( "MenuItem.foreground", Color.BLACK );
        //        UIManager.put ( "MenuItem.selectionBackground", StyleConstants.transparent );
        //        UIManager.put ( "MenuItem.selectionForeground", Color.BLACK );
        //        UIManager.put ( "RadioButtonMenuItem.background", StyleConstants.transparent );
        //        UIManager.put ( "RadioButtonMenuItem.foreground", Color.BLACK );
        //        UIManager.put ( "RadioButtonMenuItem.selectionBackground", StyleConstants.transparent );
        //        UIManager.put ( "RadioButtonMenuItem.selectionForeground", Color.BLACK );
        //        UIManager.put ( "CheckBoxMenuItem.background", StyleConstants.transparent );
        //        UIManager.put ( "CheckBoxMenuItem.foreground", Color.BLACK );
        //        UIManager.put ( "CheckBoxMenuItem.selectionBackground", StyleConstants.transparent );
        //        UIManager.put ( "CheckBoxMenuItem.selectionForeground", Color.BLACK );
        //        UIManager.put ( "Menu.background", StyleConstants.transparent );
        //        UIManager.put ( "Menu.foreground", Color.BLACK );
        //        UIManager.put ( "Menu.selectionBackground", StyleConstants.transparent );
        //        UIManager.put ( "Menu.selectionForeground", Color.BLACK );

        // WebTable top right scroll corner
        UIManager.put ( "Table.scrollPaneCornerComponent", WebTableCorner.class );
        // WebTable colors
        UIManager.put ( "Table.cellNoFocusBorder", BorderFactory.createEmptyBorder ( 1, 1, 1, 1 ) );
        UIManager.put ( "Table.focusSelectedCellHighlightBorder",
                BorderFactory.createEmptyBorder ( 1, 1, 1, 1 ) );
        UIManager.put ( "Table.focusCellHighlightBorder",
                BorderFactory.createEmptyBorder ( 1, 1, 1, 1 ) );

        // Default list renderer
        UIManager.put ( "List.cellRenderer", new UIDefaults.ActiveValue ()
        {
            public Object createValue ( UIDefaults table )
            {
                return new WebListCellRenderer.UIResource ();
            }
        } );

        // Default components borders
        UIManager.put ( "ProgressBar.border",
                new SwingLazyValue ( "com.alee.laf.WebBorders", "getProgressBarBorder" ) );
        UIManager.put ( "Button.border",
                new SwingLazyValue ( "com.alee.laf.WebBorders", "getButtonBorder" ) );

        // WebTextField actions
        UIManager.put ( "TextField.focusInputMap", new UIDefaults.LazyInputMap (
                new Object[]{ "control C", DefaultEditorKit.copyAction, "control V",
                        DefaultEditorKit.pasteAction, "control X", DefaultEditorKit.cutAction,
                        "COPY", DefaultEditorKit.copyAction, "PASTE", DefaultEditorKit.pasteAction,
                        "CUT", DefaultEditorKit.cutAction, "control INSERT",
                        DefaultEditorKit.copyAction, "shift INSERT", DefaultEditorKit.pasteAction,
                        "shift DELETE", DefaultEditorKit.cutAction, "control A",
                        DefaultEditorKit.selectAllAction, "control BACK_SLASH", "unselect"
                        /*DefaultEditorKit.unselectAction*/, "shift LEFT",
                        DefaultEditorKit.selectionBackwardAction, "shift RIGHT",
                        DefaultEditorKit.selectionForwardAction, "control LEFT",
                        DefaultEditorKit.previousWordAction, "control RIGHT",
                        DefaultEditorKit.nextWordAction, "control shift LEFT",
                        DefaultEditorKit.selectionPreviousWordAction, "control shift RIGHT",
                        DefaultEditorKit.selectionNextWordAction, "HOME",
                        DefaultEditorKit.beginLineAction, "END", DefaultEditorKit.endLineAction,
                        "shift HOME", DefaultEditorKit.selectionBeginLineAction, "shift END",
                        DefaultEditorKit.selectionEndLineAction, "BACK_SPACE",
                        DefaultEditorKit.deletePrevCharAction, "shift BACK_SPACE",
                        DefaultEditorKit.deletePrevCharAction, "ctrl H",
                        DefaultEditorKit.deletePrevCharAction, "DELETE",
                        DefaultEditorKit.deleteNextCharAction, "ctrl DELETE",
                        DefaultEditorKit.deleteNextWordAction, "ctrl BACK_SPACE",
                        DefaultEditorKit.deletePrevWordAction, "RIGHT",
                        DefaultEditorKit.forwardAction, "LEFT", DefaultEditorKit.backwardAction,
                        "KP_RIGHT", DefaultEditorKit.forwardAction, "KP_LEFT",
                        DefaultEditorKit.backwardAction, "ENTER", JTextField.notifyAction,
                        "control shift O", "toggle-componentOrientation"
                        /*DefaultEditorKit.toggleComponentOrientation*/ } ) );

        // WebPasswordField actions
        UIManager.put ( "PasswordField.focusInputMap", new UIDefaults.LazyInputMap (
                new Object[]{ "control C", DefaultEditorKit.copyAction, "control V",
                        DefaultEditorKit.pasteAction, "control X", DefaultEditorKit.cutAction,
                        "COPY", DefaultEditorKit.copyAction, "PASTE", DefaultEditorKit.pasteAction,
                        "CUT", DefaultEditorKit.cutAction, "control INSERT",
                        DefaultEditorKit.copyAction, "shift INSERT", DefaultEditorKit.pasteAction,
                        "shift DELETE", DefaultEditorKit.cutAction, "control A",
                        DefaultEditorKit.selectAllAction, "control BACK_SLASH", "unselect"
                        /*DefaultEditorKit.unselectAction*/, "shift LEFT",
                        DefaultEditorKit.selectionBackwardAction, "shift RIGHT",
                        DefaultEditorKit.selectionForwardAction, "control LEFT",
                        DefaultEditorKit.beginLineAction, "control RIGHT",
                        DefaultEditorKit.endLineAction, "control shift LEFT",
                        DefaultEditorKit.selectionBeginLineAction, "control shift RIGHT",
                        DefaultEditorKit.selectionEndLineAction, "HOME",
                        DefaultEditorKit.beginLineAction, "END", DefaultEditorKit.endLineAction,
                        "shift HOME", DefaultEditorKit.selectionBeginLineAction, "shift END",
                        DefaultEditorKit.selectionEndLineAction, "BACK_SPACE",
                        DefaultEditorKit.deletePrevCharAction, "shift BACK_SPACE",
                        DefaultEditorKit.deletePrevCharAction, "ctrl H",
                        DefaultEditorKit.deletePrevCharAction, "DELETE",
                        DefaultEditorKit.deleteNextCharAction, "RIGHT",
                        DefaultEditorKit.forwardAction, "LEFT", DefaultEditorKit.backwardAction,
                        "KP_RIGHT", DefaultEditorKit.forwardAction, "KP_LEFT",
                        DefaultEditorKit.backwardAction, "ENTER", JTextField.notifyAction,
                        "control shift O", "toggle-componentOrientation"
                        /*DefaultEditorKit.toggleComponentOrientation*/ } ) );

        // WebFormattedTextField actions
        UIManager.put ( "FormattedTextField.focusInputMap", new UIDefaults.LazyInputMap (
                new Object[]{ "ctrl C", DefaultEditorKit.copyAction, "ctrl V",
                        DefaultEditorKit.pasteAction, "ctrl X", DefaultEditorKit.cutAction, "COPY",
                        DefaultEditorKit.copyAction, "PASTE", DefaultEditorKit.pasteAction, "CUT",
                        DefaultEditorKit.cutAction, "control INSERT", DefaultEditorKit.copyAction,
                        "shift INSERT", DefaultEditorKit.pasteAction, "shift DELETE",
                        DefaultEditorKit.cutAction, "shift LEFT",
                        DefaultEditorKit.selectionBackwardAction, "shift KP_LEFT",
                        DefaultEditorKit.selectionBackwardAction, "shift RIGHT",
                        DefaultEditorKit.selectionForwardAction, "shift KP_RIGHT",
                        DefaultEditorKit.selectionForwardAction, "ctrl LEFT",
                        DefaultEditorKit.previousWordAction, "ctrl KP_LEFT",
                        DefaultEditorKit.previousWordAction, "ctrl RIGHT",
                        DefaultEditorKit.nextWordAction, "ctrl KP_RIGHT",
                        DefaultEditorKit.nextWordAction, "ctrl shift LEFT",
                        DefaultEditorKit.selectionPreviousWordAction, "ctrl shift KP_LEFT",
                        DefaultEditorKit.selectionPreviousWordAction, "ctrl shift RIGHT",
                        DefaultEditorKit.selectionNextWordAction, "ctrl shift KP_RIGHT",
                        DefaultEditorKit.selectionNextWordAction, "ctrl A",
                        DefaultEditorKit.selectAllAction, "HOME", DefaultEditorKit.beginLineAction,
                        "END", DefaultEditorKit.endLineAction, "shift HOME",
                        DefaultEditorKit.selectionBeginLineAction, "shift END",
                        DefaultEditorKit.selectionEndLineAction, "BACK_SPACE",
                        DefaultEditorKit.deletePrevCharAction, "shift BACK_SPACE",
                        DefaultEditorKit.deletePrevCharAction, "ctrl H",
                        DefaultEditorKit.deletePrevCharAction, "DELETE",
                        DefaultEditorKit.deleteNextCharAction, "ctrl DELETE",
                        DefaultEditorKit.deleteNextWordAction, "ctrl BACK_SPACE",
                        DefaultEditorKit.deletePrevWordAction, "RIGHT",
                        DefaultEditorKit.forwardAction, "LEFT", DefaultEditorKit.backwardAction,
                        "KP_RIGHT", DefaultEditorKit.forwardAction, "KP_LEFT",
                        DefaultEditorKit.backwardAction, "ENTER", JTextField.notifyAction,
                        "ctrl BACK_SLASH", "unselect", "control shift O",
                        "toggle-componentOrientation", "ESCAPE", "reset-field-edit", "UP",
                        "increment", "KP_UP", "increment", "DOWN", "decrement", "KP_DOWN",
                        "decrement", } ) );

        // Multiline areas actions
        Object multilineInputMap = new UIDefaults.LazyInputMap (
                new Object[]{ "control C", DefaultEditorKit.copyAction, "control V",
                        DefaultEditorKit.pasteAction, "control X", DefaultEditorKit.cutAction,
                        "COPY", DefaultEditorKit.copyAction, "PASTE", DefaultEditorKit.pasteAction,
                        "CUT", DefaultEditorKit.cutAction, "control INSERT",
                        DefaultEditorKit.copyAction, "shift INSERT", DefaultEditorKit.pasteAction,
                        "shift DELETE", DefaultEditorKit.cutAction, "shift LEFT",
                        DefaultEditorKit.selectionBackwardAction, "shift RIGHT",
                        DefaultEditorKit.selectionForwardAction, "control LEFT",
                        DefaultEditorKit.previousWordAction, "control RIGHT",
                        DefaultEditorKit.nextWordAction, "control shift LEFT",
                        DefaultEditorKit.selectionPreviousWordAction, "control shift RIGHT",
                        DefaultEditorKit.selectionNextWordAction, "control A",
                        DefaultEditorKit.selectAllAction, "control BACK_SLASH", "unselect"
                        /*DefaultEditorKit.unselectAction*/, "HOME",
                        DefaultEditorKit.beginLineAction, "END", DefaultEditorKit.endLineAction,
                        "shift HOME", DefaultEditorKit.selectionBeginLineAction, "shift END",
                        DefaultEditorKit.selectionEndLineAction, "control HOME",
                        DefaultEditorKit.beginAction, "control END", DefaultEditorKit.endAction,
                        "control shift HOME", DefaultEditorKit.selectionBeginAction,
                        "control shift END", DefaultEditorKit.selectionEndAction, "UP",
                        DefaultEditorKit.upAction, "DOWN", DefaultEditorKit.downAction,
                        "BACK_SPACE", DefaultEditorKit.deletePrevCharAction, "shift BACK_SPACE",
                        DefaultEditorKit.deletePrevCharAction, "ctrl H",
                        DefaultEditorKit.deletePrevCharAction, "DELETE",
                        DefaultEditorKit.deleteNextCharAction, "ctrl DELETE",
                        DefaultEditorKit.deleteNextWordAction, "ctrl BACK_SPACE",
                        DefaultEditorKit.deletePrevWordAction, "RIGHT",
                        DefaultEditorKit.forwardAction, "LEFT", DefaultEditorKit.backwardAction,
                        "KP_RIGHT", DefaultEditorKit.forwardAction, "KP_LEFT",
                        DefaultEditorKit.backwardAction, "PAGE_UP", DefaultEditorKit.pageUpAction,
                        "PAGE_DOWN", DefaultEditorKit.pageDownAction, "shift PAGE_UP",
                        "selection-page-up", "shift PAGE_DOWN", "selection-page-down",
                        "ctrl shift PAGE_UP", "selection-page-left", "ctrl shift PAGE_DOWN",
                        "selection-page-right", "shift UP", DefaultEditorKit.selectionUpAction,
                        "shift DOWN", DefaultEditorKit.selectionDownAction, "ENTER",
                        DefaultEditorKit.insertBreakAction, "TAB", DefaultEditorKit.insertTabAction,
                        "control T", "next-link-action", "control shift T", "previous-link-action",
                        "control SPACE", "activate-link-action", "control shift O",
                        "toggle-componentOrientation"
                        /*DefaultEditorKit.toggleComponentOrientation*/ } );
        UIManager.put ( "TextArea.focusInputMap", multilineInputMap );
        UIManager.put ( "TextPane.focusInputMap", multilineInputMap );
        UIManager.put ( "EditorPane.focusInputMap", multilineInputMap );

        // WebComboBox actions
        UIManager.put ( "ComboBox.ancestorInputMap", new UIDefaults.LazyInputMap (
                new Object[]{ "ESCAPE", "hidePopup", "PAGE_UP", "pageUpPassThrough", "PAGE_DOWN",
                        "pageDownPassThrough", "HOME", "homePassThrough", "END", "endPassThrough",
                        "DOWN", "selectNext", "KP_DOWN", "selectNext", "alt DOWN", "togglePopup",
                        "alt KP_DOWN", "togglePopup", "alt UP", "togglePopup", "alt KP_UP",
                        "togglePopup", "SPACE", "spacePopup", "ENTER", "enterPressed", "UP",
                        "selectPrevious", "KP_UP", "selectPrevious" } ) );

        // WebFileChooser actions
        UIManager.put ( "FileChooser.ancestorInputMap", new UIDefaults.LazyInputMap (
                new Object[]{ "ESCAPE", "cancelSelection", "F2", "editFileName", "F5", "refresh",
                        "BACK_SPACE", "Go Up", "ENTER", "approveSelection", "ctrl ENTER",
                        "approveSelection" } ) );
    }

    /*
    * Separate static method for components UIs variables initialization.
    * After calling this method (even without settings the WebLookAndFeel
    * as current LaF) you can use separate UIs and Web-components anywhere in the application.
    */

    public static void initializeStyles ()
    {
        // Language constants
        if ( StyleConstants.language.equals ( StyleConstants.RUSSIAN ) )
        {
            // WebFileChooser language
            UIManager.put ( "WebFileChooser.title", "Выбор файлов" );
            UIManager.put ( "WebFileChooser.back", "Назад" );
            UIManager.put ( "WebFileChooser.forward", "Вперёд" );
            UIManager.put ( "WebFileChooser.path", "Расположение:" );
            UIManager.put ( "WebFileChooser.folderup", "На уровень вверх" );
            UIManager.put ( "WebFileChooser.home", "Домашняя директория" );
            UIManager.put ( "WebFileChooser.newfolder", "Создать новую папку" );
            UIManager.put ( "WebFileChooser.newfolder.name", "Новая папка" );
            UIManager.put ( "WebFileChooser.newfolder.error.title", "Ошибка" );
            UIManager.put ( "WebFileChooser.newfolder.error.text",
                    "Не удалось создать здесь новую папку" );
            UIManager.put ( "WebFileChooser.refresh", "Обновить содержимое" );
            UIManager.put ( "WebFileChooser.delete", "Удалить файл или папку" );
            UIManager.put ( "WebFileChooser.delete.confirm.title", "Подтверждение удаления" );
            UIManager.put ( "WebFileChooser.delete.confirm.text",
                    "Вы уверены что хотите удалить все эти файлы?" );
            UIManager.put ( "WebFileChooser.view", "Изменить вид" );
            UIManager.put ( "WebFileChooser.view.table", "Таблица" );
            UIManager.put ( "WebFileChooser.view.icons", "Иконки" );
            UIManager.put ( "WebFileChooser.view.tiles", "Тайлы" );
            UIManager.put ( "WebFileChooser.files.selected", "Выбранные файлы:" );
            UIManager.put ( "WebFileChooser.files.format", "Фильтр:" );
            UIManager.put ( "WebFileChooser.choose", "Выбрать" );
            UIManager.put ( "WebFileChooser.cancel", "Отмена" );

            // WebDirectoryChooser language
            UIManager.put ( "WebDirectoryChooser.delete.confirm.singledir",
                    "Вы уверены что хотите удалить папку" );
            UIManager.put ( "WebDirectoryChooser.drag",
                    "Просто перетащите файл на дерево, чтбы найти его расположение" );

            // WebColorChooser language
            UIManager.put ( "WebColorChooser.title", "Выбор цвета" );
            UIManager.put ( "WebColorChooser.webonly", "Только Web-цвета" );
            UIManager.put ( "WebColorChooser.webonly.mnemonic", 'W' );
            UIManager.put ( "WebColorChooser.choose", "Выбрать" );
            UIManager.put ( "WebColorChooser.choose.mnemonic", 'В' );
            UIManager.put ( "WebColorChooser.reset", "Сбросить" );
            UIManager.put ( "WebColorChooser.reset.mnemonic", 'С' );
            UIManager.put ( "WebColorChooser.cancel", "Отмена" );
            UIManager.put ( "WebColorChooser.cancel.mnemonic", 'О' );

            // File filters language
            UIManager.put ( "WebFileChooser.filter.allfiles", "Все файлы" );
            UIManager.put ( "WebFileChooser.filter.directoriesonly", "Только папки" );
            UIManager.put ( "WebFileChooser.filter.filesonly", "Только файлы" );
            UIManager.put ( "WebFileChooser.filter.imagesonly", "Только изображения" );
            UIManager.put ( "WebFileChooser.filter.nonhiddenonly", "Только видимые файлы" );

            // File sizes language
            UIManager.put ( "FileUtils.file.size.b", "б" );
            UIManager.put ( "FileUtils.file.size.kb", "Кб" );
            UIManager.put ( "FileUtils.file.size.mb", "Мб" );
            UIManager.put ( "FileUtils.file.size.gb", "Гб" );
            UIManager.put ( "FileUtils.file.size.tb", "Тб" );
            UIManager.put ( "FileUtils.file.size.pb", "Пб" );

            // WebOptionPane language
            UIManager.put ( "OptionPane.okButtonText", "Ок" );
            UIManager.put ( "OptionPane.okButtonMnemonic", "к" );
            UIManager.put ( "OptionPane.yesButtonText", "Да" );
            UIManager.put ( "OptionPane.yesButtonMnemonic", "Д" );
            UIManager.put ( "OptionPane.noButtonText", "Нет" );
            UIManager.put ( "OptionPane.noButtonMnemonic", "Н" );
            UIManager.put ( "OptionPane.cancelButtonText", "Отмена" );
            UIManager.put ( "OptionPane.cancelButtonMnemonic", "О" );

            // WebMemoryBar language
            UIManager.put ( "WebMemoryBar.of", "из" );
            UIManager.put ( "WebMemoryBar.allocated", "Выделенная память:" );
            UIManager.put ( "WebMemoryBar.used", "Использованная:" );
            UIManager.put ( "WebMemoryBar.maximum", "Максимальная:" );

            // WebFileDrop language
            UIManager.put ( "WebFileDrop.draghere", "Перетащите сюда файлы" );

            // WebCalendar language
            UIManager.put ( "WebCalendar.dayOfWeek.1", "Пн" );
            UIManager.put ( "WebCalendar.dayOfWeek.2", "Вт" );
            UIManager.put ( "WebCalendar.dayOfWeek.3", "Ср" );
            UIManager.put ( "WebCalendar.dayOfWeek.4", "Чт" );
            UIManager.put ( "WebCalendar.dayOfWeek.5", "Пт" );
            UIManager.put ( "WebCalendar.dayOfWeek.6", "Сб" );
            UIManager.put ( "WebCalendar.dayOfWeek.7", "Вс" );

            // NinePatchEditor language
            UIManager.put ( "NinePatchEditor.openImage", "Открыть изображение" );
            UIManager.put ( "NinePatchEditor.openImage.title", "Выберите изображение" );
            UIManager.put ( "NinePatchEditor.saveImage", "Сохранить изображение" );
            UIManager.put ( "NinePatchEditor.saveImageAs", "Сохранить изображение как" );
            UIManager.put ( "NinePatchEditor.saveImageAs.title", "Выберите файл для сохранения" );
            UIManager.put ( "NinePatchEditor.copyInfo", "Копировать расположение патчей" );
            UIManager.put ( "NinePatchEditor.pasteInfo", "Вставить расположение патчей" );
            UIManager.put ( "NinePatchEditor.showPreview", "Предпросмотр" );
            UIManager.put ( "NinePatchEditor.preview.title", "Предпросмотр" );
            UIManager.put ( "NinePatchEditor.preview.text", "Просто текст" );
            UIManager.put ( "NinePatchEditor.preview.showIcon", "Показывать иконку" );
            UIManager.put ( "NinePatchEditor.preview.showText", "Показывать текст" );
            UIManager.put ( "NinePatchEditor.preview.preferredSize",
                    "Вернуть компонент к исходному размеру" );
            UIManager.put ( "NinePatchEditor.preview.followParent",
                    "Перемещать диалог вместе с окном редактора" );
            UIManager.put ( "NinePatchEditor.preview.transparentBackground",
                    "Показывать прозрачный фон" );
            UIManager.put ( "NinePatchEditor.preview.coloredBackground", "Показывать цветной фон" );
            UIManager.put ( "NinePatchEditor.preview.backgroundColor", "Цвет фона" );
            UIManager.put ( "NinePatchEditor.preview.foregroundColor", "Цвет текста" );
            UIManager.put ( "NinePatchEditor.showSpacing",
                    "Показывать расстояние между направляющими" );
            UIManager.put ( "NinePatchEditor.showRuler", "Показывать линейку" );
            UIManager.put ( "NinePatchEditor.fillContent", "Показывать область содержимого" );
            UIManager.put ( "NinePatchEditor.fillStretch", "Показывать растягиваемые области" );
        }
        else
        {
            // WebFileChooser language
            UIManager.put ( "WebFileChooser.title", "Choose files" );
            UIManager.put ( "WebFileChooser.back", "Back" );
            UIManager.put ( "WebFileChooser.forward", "Forward" );
            UIManager.put ( "WebFileChooser.path", "Placement:" );
            UIManager.put ( "WebFileChooser.folderup", "Folder up" );
            UIManager.put ( "WebFileChooser.home", "Home directory" );
            UIManager.put ( "WebFileChooser.newfolder", "Create new folder" );
            UIManager.put ( "WebFileChooser.newfolder.name", "New folder" );
            UIManager.put ( "WebFileChooser.newfolder.error.title", "Error" );
            UIManager.put ( "WebFileChooser.newfolder.error.text",
                    "Unable to create new folder here" );
            UIManager.put ( "WebFileChooser.refresh", "Reload folder content" );
            UIManager.put ( "WebFileChooser.delete", "Delete selected files" );
            UIManager.put ( "WebFileChooser.delete.confirm.title", "Delete confirmation" );
            UIManager.put ( "WebFileChooser.delete.confirm.text",
                    "Are you sure you want to delete these files?" );
            UIManager.put ( "WebFileChooser.view", "Change view" );
            UIManager.put ( "WebFileChooser.view.table", "Details" );
            UIManager.put ( "WebFileChooser.view.icons", "Icons" );
            UIManager.put ( "WebFileChooser.view.tiles", "Tiles" );
            UIManager.put ( "WebFileChooser.files.selected", "Selected files:" );
            UIManager.put ( "WebFileChooser.files.format", "Filter:" );
            UIManager.put ( "WebFileChooser.choose", "Choose" );
            UIManager.put ( "WebFileChooser.cancel", "Cancel" );

            // WebDirectoryChooser language
            UIManager.put ( "WebDirectoryChooser.delete.confirm.singledir",
                    "Are you sure you want to delete directory" );
            UIManager.put ( "WebDirectoryChooser.drag",
                    "Simply drag and drop file on the tree to locate it" );

            // WebColorChooser language
            UIManager.put ( "WebColorChooser.title", "Choose color" );
            UIManager.put ( "WebColorChooser.webonly", "Web only" );
            UIManager.put ( "WebColorChooser.webonly.mnemonic", 'W' );
            UIManager.put ( "WebColorChooser.choose", "Choose" );
            UIManager.put ( "WebColorChooser.choose.mnemonic", 'C' );
            UIManager.put ( "WebColorChooser.reset", "Reset" );
            UIManager.put ( "WebColorChooser.reset.mnemonic", 'R' );
            UIManager.put ( "WebColorChooser.cancel", "Cancel" );
            UIManager.put ( "WebColorChooser.cancel.mnemonic", 'l' );

            // File filters language
            UIManager.put ( "WebFileChooser.filter.allfiles", "All files" );
            UIManager.put ( "WebFileChooser.filter.directoriesonly", "Folders only" );
            UIManager.put ( "WebFileChooser.filter.filesonly", "Files only" );
            UIManager.put ( "WebFileChooser.filter.imagesonly", "Images only" );
            UIManager.put ( "WebFileChooser.filter.nonhiddenonly", "Non-hidden files only" );

            // File sizes language
            UIManager.put ( "FileUtils.file.size.b", "b" );
            UIManager.put ( "FileUtils.file.size.kb", "Kb" );
            UIManager.put ( "FileUtils.file.size.mb", "Mb" );
            UIManager.put ( "FileUtils.file.size.gb", "Gb" );
            UIManager.put ( "FileUtils.file.size.tb", "Tb" );
            UIManager.put ( "FileUtils.file.size.pb", "Pb" );

            // WebOptionPane language
            UIManager.put ( "OptionPane.okButtonText", "Ok" );
            UIManager.put ( "OptionPane.okButtonMnemonic", "O" );
            UIManager.put ( "OptionPane.yesButtonText", "Yes" );
            UIManager.put ( "OptionPane.yesButtonMnemonic", "Y" );
            UIManager.put ( "OptionPane.noButtonText", "No" );
            UIManager.put ( "OptionPane.noButtonMnemonic", "N" );
            UIManager.put ( "OptionPane.cancelButtonText", "Cancel" );
            UIManager.put ( "OptionPane.cancelButtonMnemonic", "C" );

            // WebMemoryBar language
            UIManager.put ( "WebMemoryBar.of", "of" );
            UIManager.put ( "WebMemoryBar.allocated", "Allocated heap size:" );
            UIManager.put ( "WebMemoryBar.used", "Used:" );
            UIManager.put ( "WebMemoryBar.maximum", "Maximum:" );

            // WebFileDrop language
            UIManager.put ( "WebFileDrop.draghere", "Drag files here" );

            // WebCalendar language
            UIManager.put ( "WebCalendar.dayOfWeek.1", "Mon" );
            UIManager.put ( "WebCalendar.dayOfWeek.2", "Tue" );
            UIManager.put ( "WebCalendar.dayOfWeek.3", "Wed" );
            UIManager.put ( "WebCalendar.dayOfWeek.4", "Thu" );
            UIManager.put ( "WebCalendar.dayOfWeek.5", "Fri" );
            UIManager.put ( "WebCalendar.dayOfWeek.6", "Sat" );
            UIManager.put ( "WebCalendar.dayOfWeek.7", "Sun" );

            // NinePatchEditor language
            UIManager.put ( "NinePatchEditor.openImage", "Open image" );
            UIManager.put ( "NinePatchEditor.openImage.title", "Choose image" );
            UIManager.put ( "NinePatchEditor.saveImage", "Save image" );
            UIManager.put ( "NinePatchEditor.saveImageAs", "Save image as" );
            UIManager.put ( "NinePatchEditor.saveImageAs.title", "Choose save location" );
            UIManager.put ( "NinePatchEditor.copyInfo", "Copy patch locations" );
            UIManager.put ( "NinePatchEditor.pasteInfo", "Paste patch locations" );
            UIManager.put ( "NinePatchEditor.showPreview", "Preview" );
            UIManager.put ( "NinePatchEditor.preview.title", "Nine-patch file preview" );
            UIManager.put ( "NinePatchEditor.preview.text", "Simple text" );
            UIManager.put ( "NinePatchEditor.preview.showIcon", "Show icon" );
            UIManager.put ( "NinePatchEditor.preview.showText", "Show text" );
            UIManager.put ( "NinePatchEditor.preview.preferredSize",
                    "Size component back to preferred size" );
            UIManager.put ( "NinePatchEditor.preview.followParent", "Follow editor window" );
            UIManager.put ( "NinePatchEditor.preview.transparentBackground",
                    "Show transparent background" );
            UIManager
                    .put ( "NinePatchEditor.preview.coloredBackground", "Show colored background" );
            UIManager.put ( "NinePatchEditor.preview.backgroundColor", "Background color" );
            UIManager.put ( "NinePatchEditor.preview.foregroundColor", "Text color" );
            UIManager.put ( "NinePatchEditor.showSpacing", "Show spacing between guidelines" );
            UIManager.put ( "NinePatchEditor.showRuler", "Show ruler" );
            UIManager.put ( "NinePatchEditor.fillContent", "Show content area" );
            UIManager.put ( "NinePatchEditor.fillStretch", "Show stretched areas" );
        }

        // Initializing additional libraries
        // Syntax highlight in JEditorPane
        DefaultSyntaxKit.initKit ();

        // Initializing managers
        // This initialization order is strict so don't change it
        SettingsManager.initializeManager ();
        ProxyManager.initializeManager ();
        HotkeyManager.initializeManager ();
        FocusManager.initializeManager ();
        TooltipManager.initializeManager ();
    }
}
