/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.checkbox;

import com.alee.laf.WebLookAndFeel;
import com.alee.utils.ReflectionUtils;

import javax.swing.*;
import java.awt.*;

/**
 * User: mgarin Date: 28.06.11 Time: 0:50
 */

public class WebCheckBox extends JCheckBox
{
    public WebCheckBox ()
    {
        super ();
    }

    public WebCheckBox ( Icon icon )
    {
        super ( icon );
    }

    public WebCheckBox ( Icon icon, boolean selected )
    {
        super ( icon, selected );
    }

    public WebCheckBox ( String text )
    {
        super ( text );
    }

    public WebCheckBox ( Action a )
    {
        super ( a );
    }

    public WebCheckBox ( String text, boolean selected )
    {
        super ( text, selected );
    }

    public WebCheckBox ( String text, Icon icon )
    {
        super ( text, icon );
    }

    public WebCheckBox ( String text, Icon icon, boolean selected )
    {
        super ( text, icon, selected );
    }

    public boolean isAnimated ()
    {
        return getWebUI ().isAnimated ();
    }

    public void setAnimated ( boolean animated )
    {
        getWebUI ().setAnimated ( animated );
    }

    public boolean isRolloverDarkBorderOnly ()
    {
        return getWebUI ().isRolloverDarkBorderOnly ();
    }

    public void setRolloverDarkBorderOnly ( boolean rolloverDarkBorderOnly )
    {
        getWebUI ().setRolloverDarkBorderOnly ( rolloverDarkBorderOnly );
    }

    public Color getBorderColor ()
    {
        return getWebUI ().getBorderColor ();
    }

    public void setBorderColor ( Color borderColor )
    {
        getWebUI ().setBorderColor ( borderColor );
    }

    public Color getDarkBorderColor ()
    {
        return getWebUI ().getDarkBorderColor ();
    }

    public void setDarkBorderColor ( Color darkBorderColor )
    {
        getWebUI ().setDarkBorderColor ( darkBorderColor );
    }

    public Color getDisabledBorderColor ()
    {
        return getWebUI ().getDisabledBorderColor ();
    }

    public void setDisabledBorderColor ( Color disabledBorderColor )
    {
        getWebUI ().setDisabledBorderColor ( disabledBorderColor );
    }

    public Color getTopBgColor ()
    {
        return getWebUI ().getTopBgColor ();
    }

    public void setTopBgColor ( Color topBgColor )
    {
        getWebUI ().setTopBgColor ( topBgColor );
    }

    public Color getBottomBgColor ()
    {
        return getWebUI ().getBottomBgColor ();
    }

    public void setBottomBgColor ( Color bottomBgColor )
    {
        getWebUI ().setBottomBgColor ( bottomBgColor );
    }

    public Color getTopSelectedBgColor ()
    {
        return getWebUI ().getTopSelectedBgColor ();
    }

    public void setTopSelectedBgColor ( Color topSelectedBgColor )
    {
        getWebUI ().setTopSelectedBgColor ( topSelectedBgColor );
    }

    public Color getBottomSelectedBgColor ()
    {
        return getWebUI ().getBottomSelectedBgColor ();
    }

    public void setBottomSelectedBgColor ( Color bottomSelectedBgColor )
    {
        getWebUI ().setBottomSelectedBgColor ( bottomSelectedBgColor );
    }

    public int getRound ()
    {
        return getWebUI ().getRound ();
    }

    public void setRound ( int round )
    {
        getWebUI ().setRound ( round );
    }

    public int getShadeWidth ()
    {
        return getWebUI ().getShadeWidth ();
    }

    public void setShadeWidth ( int shadeWidth )
    {
        getWebUI ().setShadeWidth ( shadeWidth );
    }

    public void setSelected ( boolean b )
    {
        setSelected ( b, true );
    }

    public void setSelected ( boolean b, boolean withAnimation )
    {
        boolean animated = isAnimated ();
        if ( !withAnimation && animated )
        {
            setAnimated ( false );
        }
        super.setSelected ( b );
        if ( !withAnimation )
        {
            setAnimated ( animated );
        }
    }

    public WebCheckBoxUI getWebUI ()
    {
        return ( WebCheckBoxUI ) getUI ();
    }

    public void updateUI ()
    {
        if ( getUI () == null || !( getUI () instanceof WebCheckBoxUI ) )
        {
            try
            {
                setUI ( ( WebCheckBoxUI ) ReflectionUtils
                        .createInstance ( WebLookAndFeel.checkBoxUI ) );
            }
            catch ( Throwable e )
            {
                setUI ( new WebCheckBoxUI () );
            }
        }
        else
        {
            setUI ( getUI () );
        }
    }
}
