/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.label;

import com.alee.laf.BackgroundPainter;
import com.alee.utils.SwingUtils;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicLabelUI;
import java.awt.*;

/**
 * User: mgarin Date: 26.07.11 Time: 13:16
 */

public class WebLabelUI extends BasicLabelUI
{
    private BackgroundPainter backgroundPainter = WebLabelStyle.backgroundPainter;
    private JLabel label;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebLabelUI ();
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        // Saving label to local variable
        this.label = ( JLabel ) c;

        // Updating border
        updateBorder ( c );
    }

    private void updateBorder ( JComponent component )
    {
        if ( backgroundPainter != null )
        {
            // Background insets
            Insets bi = backgroundPainter.getBackgroundMargin ( component );
            component.setBorder (
                    BorderFactory.createEmptyBorder ( bi.top, bi.left, bi.bottom, bi.right ) );
        }
        else
        {
            // Empty insets
            component.setBorder ( BorderFactory.createEmptyBorder ( 0, 0, 0, 0 ) );
        }
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return backgroundPainter;
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        this.backgroundPainter = backgroundPainter;
        updateBorder ( label );
    }

    public void paint ( Graphics g, JComponent c )
    {
        if ( backgroundPainter != null )
        {
            // Use background painter instead of default UI graphics
            backgroundPainter.paintBackground ( g, SwingUtils.size ( c ), c );
        }

        super.paint ( g, c );
    }

    public Dimension getPreferredSize ( JComponent c )
    {
        Dimension ps = super.getPreferredSize ( c );
        if ( backgroundPainter != null )
        {
            if ( c.getLayout () != null )
            {
                ps = SwingUtils.max ( ps, c.getLayout ().preferredLayoutSize ( c ) );
            }
            ps = SwingUtils.max ( ps, backgroundPainter.getBackgroundPreferredSize ( c ) );
        }
        return ps;
    }
}
