/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.separator;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicSeparatorUI;
import java.awt.*;

/**
 * User: mgarin Date: 28.06.11 Time: 0:34
 */

public class WebSeparatorUI extends BasicSeparatorUI
{
    private static final float[] fractions = new float[]{ 0.0f, 0.5f, 1f };

    private Color separatorLightUpperColor = WebSeparatorStyle.separatorLightUpperColor;
    private Color separatorLightColor = WebSeparatorStyle.separatorLightColor;
    private Color separatorUpperColor = WebSeparatorStyle.separatorUpperColor;
    private Color separatorColor = WebSeparatorStyle.separatorColor;

    private boolean reversedColors = WebSeparatorStyle.reversedColors;
    private boolean drawSideLines = WebSeparatorStyle.drawSideLines;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebSeparatorUI ();
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        c.setOpaque ( false );
    }

    public Color getSeparatorColor ()
    {
        return separatorColor;
    }

    public void setSeparatorColor ( Color separatorColor )
    {
        this.separatorColor = separatorColor;
    }

    public Color getSeparatorUpperColor ()
    {
        return separatorUpperColor;
    }

    public void setSeparatorUpperColor ( Color separatorUpperColor )
    {
        this.separatorUpperColor = separatorUpperColor;
    }

    public Color getSeparatorLightColor ()
    {
        return separatorLightColor;
    }

    public void setSeparatorLightColor ( Color separatorLightColor )
    {
        this.separatorLightColor = separatorLightColor;
    }

    public Color getSeparatorLightUpperColor ()
    {
        return separatorLightUpperColor;
    }

    public void setSeparatorLightUpperColor ( Color separatorLightUpperColor )
    {
        this.separatorLightUpperColor = separatorLightUpperColor;
    }

    public boolean isReversedColors ()
    {
        return reversedColors;
    }

    public void setReversedColors ( boolean reversedColors )
    {
        this.reversedColors = reversedColors;
    }

    public boolean isDrawSideLines ()
    {
        return drawSideLines;
    }

    public void setDrawSideLines ( boolean drawSideLines )
    {
        this.drawSideLines = drawSideLines;
    }

    public void paint ( Graphics g, JComponent c )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

        JSeparator separator = ( JSeparator ) c;

        if ( separator.getOrientation () == WebSeparator.VERTICAL )
        {
            if ( drawSideLines )
            {
                g2d.setPaint ( new LinearGradientPaint ( 0, 0, 0, separator.getHeight (), fractions,
                        getLightColors () ) );
                g2d.drawLine ( 0, 0, 0, separator.getHeight () );
                g2d.drawLine ( 2, 0, 2, separator.getHeight () );
            }

            g2d.setPaint ( new LinearGradientPaint ( 0, 0, 0, separator.getHeight (), fractions,
                    getDarkColors () ) );
            g2d.drawLine ( drawSideLines ? 1 : 0, 0, drawSideLines ? 1 : 0,
                    separator.getHeight () );
        }
        else
        {
            if ( drawSideLines )
            {
                g2d.setPaint ( new LinearGradientPaint ( 0, 0, separator.getWidth (), 0, fractions,
                        getLightColors () ) );
                g2d.drawLine ( 0, 0, separator.getWidth (), 0 );
                g2d.drawLine ( 0, 2, separator.getWidth (), 2 );
            }

            g2d.setPaint ( new LinearGradientPaint ( 0, 0, separator.getWidth (), 0, fractions,
                    getDarkColors () ) );
            g2d.drawLine ( 0, drawSideLines ? 1 : 0, separator.getWidth (), drawSideLines ? 1 : 0 );
        }
    }

    private Color[] getLightColors ()
    {
        return reversedColors ?
                new Color[]{ separatorUpperColor, separatorColor, separatorUpperColor } :
                new Color[]{ separatorLightUpperColor, Color.WHITE, separatorLightUpperColor };
    }

    private Color[] getDarkColors ()
    {
        return reversedColors ?
                new Color[]{ separatorLightUpperColor, Color.WHITE, separatorLightUpperColor } :
                new Color[]{ separatorUpperColor, separatorColor, separatorUpperColor };
    }

    public Dimension getPreferredSize ( JComponent c )
    {
        if ( ( ( JSeparator ) c ).getOrientation () == WebSeparator.VERTICAL )
        {
            return new Dimension ( drawSideLines ? 3 : 1, 0 );
        }
        else
        {
            return new Dimension ( 0, drawSideLines ? 3 : 1 );
        }
    }
}
