/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.tabbedpane;

import com.alee.laf.WebLookAndFeel;
import com.alee.utils.ReflectionUtils;

import javax.swing.*;
import java.awt.*;

/**
 * User: mgarin Date: 28.06.11 Time: 0:52
 */

public class WebTabbedPane extends JTabbedPane
{
    public WebTabbedPane ()
    {
        super ();
    }

    public WebTabbedPane ( int tabPlacement )
    {
        super ( tabPlacement );
    }

    public WebTabbedPane ( int tabPlacement, int tabLayoutPolicy )
    {
        super ( tabPlacement, tabLayoutPolicy );
    }

    public int getRound ()
    {
        return getWebUI ().getRound ();
    }

    public void setRound ( int round )
    {
        getWebUI ().setRound ( round );
    }

    public int getShadeWidth ()
    {
        return getWebUI ().getShadeWidth ();
    }

    public void setShadeWidth ( int shadeWidth )
    {
        getWebUI ().setShadeWidth ( shadeWidth );
    }

    public Insets getContentInsets ()
    {
        return getWebUI ().getContentInsets ();
    }

    public void setContentInsets ( Insets contentInsets )
    {
        getWebUI ().setContentInsets ( contentInsets );
    }

    public Color getSelectedTopBg ()
    {
        return getWebUI ().getSelectedTopBg ();
    }

    public void setSelectedTopBg ( Color selectedTopBg )
    {
        getWebUI ().setSelectedTopBg ( selectedTopBg );
    }

    public Color getSelectedBottomBg ()
    {
        return getWebUI ().getSelectedBottomBg ();
    }

    public void setSelectedBottomBg ( Color selectedBottomBg )
    {
        getWebUI ().setSelectedBottomBg ( selectedBottomBg );
    }

    public Color getTopBg ()
    {
        return getWebUI ().getTopBg ();
    }

    public void setTopBg ( Color topBg )
    {
        getWebUI ().setTopBg ( topBg );
    }

    public Color getBottomBg ()
    {
        return getWebUI ().getBottomBg ();
    }

    public void setBottomBg ( Color bottomBg )
    {
        getWebUI ().setBottomBg ( bottomBg );
    }

    public WebTabbedPaneUI getWebUI ()
    {
        return ( WebTabbedPaneUI ) getUI ();
    }

    public void updateUI ()
    {
        if ( getUI () == null || !( getUI () instanceof WebTabbedPaneUI ) )
        {
            try
            {
                setUI ( ( WebTabbedPaneUI ) ReflectionUtils
                        .createInstance ( WebLookAndFeel.tabbedPaneUI ) );
            }
            catch ( Throwable e )
            {
                setUI ( new WebTabbedPaneUI () );
            }
        }
        else
        {
            setUI ( getUI () );
        }
    }
}
