/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.tabbedpane;

import com.alee.laf.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;
import com.alee.utils.laf.ShapeProvider;
import sun.swing.SwingUtilities2;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicTabbedPaneUI;
import javax.swing.text.View;
import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.util.HashMap;
import java.util.Map;

/**
 * User: mgarin Date: 27.04.11 Time: 18:39
 */

public class WebTabbedPaneUI extends BasicTabbedPaneUI implements ShapeProvider
{
    private Color selectedTopBg = WebTabbedPaneStyle.selectedTopBg;
    private Color selectedBottomBg = WebTabbedPaneStyle.selectedBottomBg;

    private Color topBg = WebTabbedPaneStyle.topBg;
    private Color bottomBg = WebTabbedPaneStyle.bottomBg;

    private Map<Integer, Color> selectedForegroundAt = new HashMap<Integer, Color> ();
    private Map<Integer, BackgroundPainter> backgroundPainterAt =
            new HashMap<Integer, BackgroundPainter> ();

    private int round = WebTabbedPaneStyle.round;
    private int shadeWidth = WebTabbedPaneStyle.shadeWidth;

    private Insets contentInsets = WebTabbedPaneStyle.contentInsets;

    private FocusAdapter focusAdapter;
    //    private MouseAdapter mouseAdapter;
    //    private int rolloverTab = -1;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebTabbedPaneUI ();
    }

    public void installUI ( final JComponent c )
    {
        super.installUI ( c );

        c.setBackground ( StyleConstants.backgroundColor );

        updateBorder ( c );

        focusAdapter = new FocusAdapter ()
        {
            public void focusGained ( FocusEvent e )
            {
                c.repaint ();
            }

            public void focusLost ( FocusEvent e )
            {
                c.repaint ();
            }
        };
        c.addFocusListener ( focusAdapter );

        //        mouseAdapter = new MouseAdapter ()
        //        {
        //            public void mouseEntered ( MouseEvent e )
        //            {
        //                updateRolloverTab ( e );
        //            }
        //
        //            public void mouseExited ( MouseEvent e )
        //            {
        //                updateRolloverTab ( e );
        //            }
        //
        //            public void mouseMoved ( MouseEvent e )
        //            {
        //                updateRolloverTab ( e );
        //            }
        //
        //            public void mouseDragged ( MouseEvent e )
        //            {
        //                updateRolloverTab ( e );
        //            }
        //        };
        //        c.addMouseListener ( mouseAdapter );
        //        c.addMouseMotionListener ( mouseAdapter );
    }

    public void uninstallUI ( JComponent c )
    {
        super.uninstallUI ( c );

        if ( focusAdapter != null )
        {
            c.removeFocusListener ( focusAdapter );
        }
        //        if ( mouseAdapter != null )
        //        {
        //            c.removeMouseListener ( mouseAdapter );
        //            c.removeMouseMotionListener ( mouseAdapter );
        //        }
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( tabPane, getShadeWidth (), getRound () );
    }

    private void updateBorder ( JComponent c )
    {
        c.setBorder ( BorderFactory
                .createEmptyBorder ( shadeWidth, shadeWidth, shadeWidth, shadeWidth ) );
    }

    public int getShadeWidth ()
    {
        return shadeWidth;
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
        updateBorder ( tabPane );
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public Insets getContentInsets ()
    {
        return contentInsets;
    }

    public void setContentInsets ( Insets contentInsets )
    {
        this.contentInsets = contentInsets;
    }

    public Color getSelectedTopBg ()
    {
        return selectedTopBg;
    }

    public void setSelectedTopBg ( Color selectedTopBg )
    {
        this.selectedTopBg = selectedTopBg;
    }

    public Color getSelectedBottomBg ()
    {
        return selectedBottomBg;
    }

    public void setSelectedBottomBg ( Color selectedBottomBg )
    {
        this.selectedBottomBg = selectedBottomBg;
    }

    public Color getTopBg ()
    {
        return topBg;
    }

    public void setTopBg ( Color topBg )
    {
        this.topBg = topBg;
    }

    public Color getBottomBg ()
    {
        return bottomBg;
    }

    public void setBottomBg ( Color bottomBg )
    {
        this.bottomBg = bottomBg;
    }

    public void setSelectedForegroundAt ( int tabIndex, Color foreground )
    {
        selectedForegroundAt.put ( tabIndex, foreground );
    }

    public Color getSelectedForegroundAt ( int tabIndex )
    {
        return selectedForegroundAt.get ( tabIndex );
    }

    public void setBackgroundPainterAt ( int tabIndex, BackgroundPainter backgroundPainter )
    {
        backgroundPainterAt.put ( tabIndex, backgroundPainter );
    }

    public BackgroundPainter getBackgroundPainterAt ( int tabIndex )
    {
        return backgroundPainterAt.get ( tabIndex );
    }

    //    private void updateRolloverTab ( MouseEvent e )
    //    {
    //        if ( tabPane != null )
    //        {
    //            int old = rolloverTab;
    //            rolloverTab = tabForCoordinate ( tabPane, e.getX (), e.getY () );
    //            if ( old != rolloverTab )
    //            {
    //                tabPane.repaint ();
    //            }
    //        }
    //    }

    protected Insets getContentBorderInsets ( int tabPlacement )
    {
        Insets insets;
        if ( tabPlacement == JTabbedPane.TOP )
        {
            insets = new Insets ( 1, 2, 1, 2 );
        }
        else if ( tabPlacement == JTabbedPane.BOTTOM )
        {
            insets = new Insets ( 2, 2, 0, 2 );
        }
        else if ( tabPlacement == JTabbedPane.LEFT )
        {
            insets = new Insets ( 2, 1, 2, 1 );
        }
        else if ( tabPlacement == JTabbedPane.RIGHT )
        {
            insets = new Insets ( 2, 2, 2, 0 );
        }
        else
        {
            insets = new Insets ( 0, 0, 0, 0 );
        }
        insets.top += contentInsets.top - 1;
        insets.left += contentInsets.left - 1;
        insets.bottom += contentInsets.bottom - 1;
        insets.right += contentInsets.right - 1;
        return insets;
    }

    protected Insets getTabAreaInsets ( int tabPlacement )
    {
        Insets targetInsets = new Insets ( 0, 0, 0, 0 );
        rotateInsets ( new Insets ( 0, 2, 0, 2 ), targetInsets, tabPlacement );
        return targetInsets;
    }

    protected Insets getTabInsets ( int tabPlacement, int tabIndex )
    {
        return new Insets ( 1, 4, 2, 4 );
    }

    protected Insets getSelectedTabPadInsets ( int tabPlacement )
    {
        Insets targetInsets = new Insets ( 0, 0, 0, 0 );
        rotateInsets ( new Insets ( 2, 2, 2, 1 ), targetInsets, tabPlacement );
        return targetInsets;
    }

    protected void paintTabBorder ( Graphics g, int tabPlacement, int tabIndex, int x, int y, int w,
                                    int h, boolean isSelected )
    {
        // We don't need this one
    }

    protected void paintTabBackground ( Graphics g, int tabPlacement, int tabIndex, int x, int y,
                                        int w, int h, boolean isSelected )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

        // Border shape
        GeneralPath borderShape = createTabShape ( TabShapeType.border, tabPlacement, x, y, w, h );

        // Tab shade
        GeneralPath shadeShape = createTabShape ( TabShapeType.shade, tabPlacement, x, y, w, h );
        LafUtils.drawShade ( g2d, shadeShape, StyleConstants.shadeColor, shadeWidth,
                new Rectangle2D.Double ( 0, 0, tabPane.getWidth (), y + h ), round > 0 );

        // Tab background     
        GeneralPath bgShape = createTabShape ( TabShapeType.background, tabPlacement, x, y, w, h );
        if ( backgroundPainterAt.containsKey ( tabIndex ) && isSelected )
        {
            Shape old = LafUtils.intersectClip ( g2d, bgShape );
            BackgroundPainter bp = backgroundPainterAt.get ( tabIndex );
            bp.paintBackground ( g2d, new Rectangle ( x, y, w, h ), tabPane );
            LafUtils.restoreClip ( g2d, old );
        }
        else
        {
            Point topPoint = getTopTabBgPoint ( tabPlacement, x, y, w, h );
            Point bottomPoint = getBottomTabBgPoint ( tabPlacement, x, y, w, h );
            if ( isSelected )
            {
                Color bg = tabPane.getBackgroundAt ( tabIndex );
                bg = bg != null ? bg : tabPane.getBackground ();
                g2d.setPaint (
                        new GradientPaint ( topPoint.x, topPoint.y, selectedTopBg, bottomPoint.x,
                                bottomPoint.y, bg ) );
            }
            else
            {
                g2d.setPaint ( new GradientPaint ( topPoint.x, topPoint.y, topBg, bottomPoint.x,
                        bottomPoint.y, bottomBg ) );
            }
            g2d.fill ( isSelected ? borderShape : bgShape );
        }

        // Tab border
        g2d.setPaint ( StyleConstants.darkBorderColor );
        g2d.draw ( borderShape );

        // Tab focus
        LafUtils.drawCustonWebFocus ( g2d, null, StyleConstants.focusType, borderShape, null,
                isSelected && tabPane.isFocusOwner () );

        //todo Paint selected tab together with area
    }

    protected void paintText ( Graphics g, int tabPlacement, Font font, FontMetrics metrics,
                               int tabIndex, String title, Rectangle textRect, boolean isSelected )
    {

        g.setFont ( font );

        View v = getTextViewForTab ( tabIndex );
        if ( v != null )
        {
            // html
            v.paint ( g, textRect );
        }
        else
        {
            // plain text
            int mnemIndex = tabPane.getDisplayedMnemonicIndexAt ( tabIndex );

            if ( tabPane.isEnabled () && tabPane.isEnabledAt ( tabIndex ) )
            {
                Color fg = tabPane.getForegroundAt ( tabIndex );
                if ( isSelected && ( fg instanceof UIResource ) )
                {
                    if ( selectedForegroundAt.containsKey ( tabIndex ) )
                    {
                        fg = selectedForegroundAt.get ( tabIndex );
                    }
                    else
                    {
                        Color selectedFG = UIManager.getColor ( "TabbedPane.selectedForeground" );
                        if ( selectedFG != null )
                        {
                            fg = selectedFG;
                        }
                    }
                }
                g.setColor ( fg );
                SwingUtilities2
                        .drawStringUnderlineCharAt ( tabPane, g, title, mnemIndex, textRect.x,
                                textRect.y + metrics.getAscent () );

            }
            else
            { // tab disabled
                g.setColor ( tabPane.getBackgroundAt ( tabIndex ).brighter () );
                SwingUtilities2
                        .drawStringUnderlineCharAt ( tabPane, g, title, mnemIndex, textRect.x,
                                textRect.y + metrics.getAscent () );
                g.setColor ( tabPane.getBackgroundAt ( tabIndex ).darker () );
                SwingUtilities2
                        .drawStringUnderlineCharAt ( tabPane, g, title, mnemIndex, textRect.x - 1,
                                textRect.y + metrics.getAscent () - 1 );

            }
        }
    }

    private GeneralPath createTabShape ( TabShapeType tabShapeType, int tabPlacement, int x, int y,
                                         int w, int h )
    {
        GeneralPath bgShape = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
        if ( tabPlacement == JTabbedPane.TOP )
        {
            bgShape.moveTo ( x, y + h + getChange ( tabShapeType ) );
            bgShape.lineTo ( x, y + round );
            bgShape.quadTo ( x, y, x + round, y );
            bgShape.lineTo ( x + w - round, y );
            bgShape.quadTo ( x + w, y, x + w, y + round );
            bgShape.lineTo ( x + w, y + h + getChange ( tabShapeType ) );
        }
        else if ( tabPlacement == JTabbedPane.BOTTOM )
        {
            bgShape.moveTo ( x, y - getChange ( tabShapeType ) );
            bgShape.lineTo ( x, y + h - round );
            bgShape.quadTo ( x, y + h, x + round, y + h );
            bgShape.lineTo ( x + w - round, y + h );
            bgShape.quadTo ( x + w, y + h, x + w, y + h - round );
            bgShape.lineTo ( x + w, y - getChange ( tabShapeType ) );
        }
        else if ( tabPlacement == JTabbedPane.LEFT )
        {
            bgShape.moveTo ( x + w + getChange ( tabShapeType ), y );
            bgShape.lineTo ( x + round, y );
            bgShape.quadTo ( x, y, x, y + round );
            bgShape.lineTo ( x, y + h - round );
            bgShape.quadTo ( x, y + h, x + round, y + h );
            bgShape.lineTo ( x + w + getChange ( tabShapeType ), y + h );
        }
        else
        {
            bgShape.moveTo ( x - getChange ( tabShapeType ), y );
            bgShape.lineTo ( x + w - round, y );
            bgShape.quadTo ( x + w, y, x + w, y + round );
            bgShape.lineTo ( x + w, y + h - round );
            bgShape.quadTo ( x + w, y + h, x + w - round, y + h );
            bgShape.lineTo ( x - getChange ( tabShapeType ), y + h );
        }
        return bgShape;
    }

    private int getChange ( TabShapeType tabShapeType )
    {
        if ( tabShapeType.equals ( TabShapeType.shade ) )
        {
            return -( round > 0 ? round : 1 );
        }
        else if ( tabShapeType.equals ( TabShapeType.border ) )
        {
            return -1;
        }
        else if ( tabShapeType.equals ( TabShapeType.backgroundPainter ) )
        {
            return 2;
        }
        else
        {
            return 0;
        }
    }

    private enum TabShapeType
    {
        shade,
        background,
        backgroundPainter,
        border
    }

    private Point getTopTabBgPoint ( int tabPlacement, int x, int y, int w, int h )
    {
        if ( tabPlacement == JTabbedPane.TOP )
        {
            return new Point ( x, y );
        }
        else if ( tabPlacement == JTabbedPane.BOTTOM )
        {
            return new Point ( x, y + h );
        }
        else if ( tabPlacement == JTabbedPane.LEFT )
        {
            return new Point ( x, y );
        }
        else
        {
            return new Point ( x + w, y );
        }
    }

    private Point getBottomTabBgPoint ( int tabPlacement, int x, int y, int w, int h )
    {
        if ( tabPlacement == JTabbedPane.TOP )
        {
            return new Point ( x, y + h - 4 );
        }
        else if ( tabPlacement == JTabbedPane.BOTTOM )
        {
            return new Point ( x, y + 4 );
        }
        else if ( tabPlacement == JTabbedPane.LEFT )
        {
            return new Point ( x + w - 4, y );
        }
        else
        {
            return new Point ( x + 4, y );
        }
    }

    protected void paintContentBorder ( Graphics g, int tabPlacement, int selectedIndex )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

        int tabAreaSize = getTabAreaLength ( tabPlacement );

        Insets bi =
                tabPane.getBorder () != null ? tabPane.getBorder ().getBorderInsets ( tabPane ) :
                        new Insets ( 0, 0, 0, 0 );
        if ( tabPlacement == JTabbedPane.TOP || tabPlacement == JTabbedPane.BOTTOM )
        {
            bi.right += 1;
        }
        else
        {
            bi.bottom += 1;
        }

        Rectangle selected = getTabBounds ( tabPane, selectedIndex );
        Shape gp = createBackgroundShape ( tabPlacement, tabAreaSize, bi, selected );

        // Proper clip
        GeneralPath clip = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
        clip.append ( new Rectangle2D.Double ( 0, 0, tabPane.getWidth (), tabPane.getHeight () ),
                false );
        clip.append ( gp, false );
        LafUtils.drawShade ( g2d, gp, StyleConstants.shadeColor, shadeWidth, clip, round > 0 );

        // Area background
        if ( backgroundPainterAt.containsKey ( selectedIndex ) )
        {
            Shape old = LafUtils.intersectClip ( g2d, gp );
            backgroundPainterAt.get ( selectedIndex )
                    .paintBackground ( g2d, gp.getBounds (), tabPane );
            LafUtils.restoreClip ( g2d, old );
        }
        else
        {
            Color bg = tabPane.getBackgroundAt ( selectedIndex );
            g2d.setPaint ( bg != null ? bg : tabPane.getBackground () );
            g2d.fill ( gp );
        }

        // Area border
        g2d.setPaint ( Color.GRAY );
        g2d.draw ( gp );

        // Area focus
        LafUtils.drawCustonWebFocus ( g2d, null, StyleConstants.focusType, gp, null,
                tabPane.isFocusOwner () );
    }

    private int getTabAreaLength ( int tabPlacement )
    {
        return tabPlacement == JTabbedPane.TOP || tabPlacement == JTabbedPane.BOTTOM ?
                calculateTabAreaHeight ( tabPlacement, runCount, maxTabHeight ) - 1 :
                calculateTabAreaWidth ( tabPlacement, runCount, maxTabWidth ) - 1;
    }

    private GeneralPath createBackgroundShape ( int tabPlacement, int tabAreaSize, Insets bi,
                                                Rectangle selected )
    {
        GeneralPath gp = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
        if ( tabPlacement == JTabbedPane.TOP )
        {
            int topY = bi.top + tabAreaSize;
            gp.moveTo ( selected.x, topY );
            if ( selected.x > bi.left + round && round > 0 )
            {
                gp.lineTo ( bi.left + round, topY );
                gp.quadTo ( bi.left, topY, bi.left, topY + round );
            }
            else
            {
                gp.lineTo ( bi.left, topY );
            }
            if ( round > 0 )
            {
                gp.lineTo ( bi.left, tabPane.getHeight () - bi.bottom - round );
                gp.quadTo ( bi.left, tabPane.getHeight () - bi.bottom, bi.left + round,
                        tabPane.getHeight () - bi.bottom );
                gp.lineTo ( tabPane.getWidth () - bi.right - round,
                        tabPane.getHeight () - bi.bottom );
                gp.quadTo ( tabPane.getWidth () - bi.right, tabPane.getHeight () - bi.bottom,
                        tabPane.getWidth () - bi.right, tabPane.getHeight () - bi.bottom - round );
            }
            else
            {
                gp.lineTo ( bi.left, tabPane.getHeight () - bi.bottom );
                gp.lineTo ( tabPane.getWidth () - bi.right, tabPane.getHeight () - bi.bottom );
            }
            if ( selected.x + selected.width < tabPane.getWidth () - bi.right - round && round > 0 )
            {
                gp.lineTo ( tabPane.getWidth () - bi.right, topY + round );
                gp.quadTo ( tabPane.getWidth () - bi.right, topY,
                        tabPane.getWidth () - bi.right - round, topY );
            }
            else
            {
                gp.lineTo ( tabPane.getWidth () - bi.right, topY );
            }
            gp.lineTo ( selected.x + selected.width, topY );
        }
        else if ( tabPlacement == JTabbedPane.BOTTOM )
        {
            int bottomY = tabPane.getHeight () - bi.bottom - tabAreaSize;
            gp.moveTo ( selected.x, bottomY );
            if ( selected.x > bi.left + round && round > 0 )
            {
                gp.lineTo ( bi.left + round, bottomY );
                gp.quadTo ( bi.left, bottomY, bi.left, bottomY - round );
            }
            else
            {
                gp.lineTo ( bi.left, bottomY );
            }
            if ( round > 0 )
            {
                gp.lineTo ( bi.left, bi.top + round );
                gp.quadTo ( bi.left, bi.top, bi.left + round, bi.top );
                gp.lineTo ( tabPane.getWidth () - bi.right - round, bi.top );
                gp.quadTo ( tabPane.getWidth () - bi.right, bi.top, tabPane.getWidth () - bi.right,
                        bi.top + round );
            }
            else
            {
                gp.lineTo ( bi.left, bi.top );
                gp.lineTo ( tabPane.getWidth () - bi.right, bi.top );
            }
            if ( selected.x + selected.width < tabPane.getWidth () - bi.right - round && round > 0 )
            {
                gp.lineTo ( tabPane.getWidth () - bi.right, bottomY - round );
                gp.quadTo ( tabPane.getWidth () - bi.right, bottomY,
                        tabPane.getWidth () - bi.right - round, bottomY );
            }
            else
            {
                gp.lineTo ( tabPane.getWidth () - bi.right, bottomY );
            }
            gp.lineTo ( selected.x + selected.width, bottomY );
        }
        else if ( tabPlacement == JTabbedPane.LEFT )
        {
            int leftX = bi.left + tabAreaSize;
            gp.moveTo ( leftX, selected.y );
            if ( selected.y > bi.top + round && round > 0 )
            {
                gp.lineTo ( leftX, bi.top + round );
                gp.quadTo ( leftX, bi.top, leftX + round, bi.top );
            }
            else
            {
                gp.lineTo ( leftX, bi.top );
            }
            if ( round > 0 )
            {
                gp.lineTo ( tabPane.getWidth () - bi.right - round, bi.top );
                gp.quadTo ( tabPane.getWidth () - bi.right, bi.top, tabPane.getWidth () - bi.right,
                        bi.top + round );
                gp.lineTo ( tabPane.getWidth () - bi.right,
                        tabPane.getHeight () - bi.bottom - round );
                gp.quadTo ( tabPane.getWidth () - bi.right, tabPane.getHeight () - bi.bottom,
                        tabPane.getWidth () - bi.right - round, tabPane.getHeight () - bi.bottom );
            }
            else
            {
                gp.lineTo ( tabPane.getWidth () - bi.right, bi.top );
                gp.lineTo ( tabPane.getWidth () - bi.right, tabPane.getHeight () - bi.bottom );
            }
            if ( selected.y + selected.height < tabPane.getHeight () - bi.bottom - round &&
                    round > 0 )
            {
                gp.lineTo ( leftX + round, tabPane.getHeight () - bi.bottom );
                gp.quadTo ( leftX, tabPane.getHeight () - bi.bottom, leftX,
                        tabPane.getHeight () - bi.bottom - round );
            }
            else
            {
                gp.lineTo ( leftX, tabPane.getHeight () - bi.bottom );
            }
            gp.lineTo ( leftX, selected.y + selected.height );
        }
        else
        {
            int rightX = tabPane.getWidth () - bi.right - tabAreaSize;
            gp.moveTo ( rightX, selected.y );
            if ( selected.y > bi.top + round && round > 0 )
            {
                gp.lineTo ( rightX, bi.top + round );
                gp.quadTo ( rightX, bi.top, rightX - round, bi.top );
            }
            else
            {
                gp.lineTo ( rightX, bi.top );
            }
            if ( round > 0 )
            {
                gp.lineTo ( bi.left + round, bi.top );
                gp.quadTo ( bi.left, bi.top, bi.left, bi.top + round );
                gp.lineTo ( bi.left, tabPane.getHeight () - bi.bottom - round );
                gp.quadTo ( bi.left, tabPane.getHeight () - bi.bottom, bi.left + round,
                        tabPane.getHeight () - bi.bottom );
            }
            else
            {
                gp.lineTo ( bi.left, bi.top );
                gp.lineTo ( bi.left, tabPane.getHeight () - bi.bottom );
            }
            if ( selected.y + selected.height < tabPane.getHeight () - bi.bottom - round &&
                    round > 0 )
            {
                gp.lineTo ( rightX - round, tabPane.getHeight () - bi.bottom );
                gp.quadTo ( rightX, tabPane.getHeight () - bi.bottom, rightX,
                        tabPane.getHeight () - bi.bottom - round );
            }
            else
            {
                gp.lineTo ( rightX, tabPane.getHeight () - bi.bottom );
            }
            gp.lineTo ( rightX, selected.y + selected.height );
        }
        return gp;
    }

    public Shape getContentClip ()
    {
        Shape clip = null;

        int tabPlacement = tabPane.getTabPlacement ();
        int tabAreaLength = getTabAreaLength ( tabPlacement );
        Insets insets = tabPane.getInsets ();

        if ( tabPlacement == JTabbedPane.TOP )
        {
            clip = new RoundRectangle2D.Double ( insets.left, insets.top + tabAreaLength,
                    tabPane.getWidth () - insets.left - insets.right,
                    tabPane.getHeight () - insets.top - tabAreaLength - insets.bottom, round * 2,
                    round * 2 );
        }
        else if ( tabPlacement == JTabbedPane.BOTTOM )
        {
            clip = new RoundRectangle2D.Double ( insets.left, insets.top,
                    tabPane.getWidth () - insets.left - insets.right,
                    tabPane.getHeight () - insets.top - tabAreaLength - insets.bottom, round * 2,
                    round * 2 );
        }
        else if ( tabPlacement == JTabbedPane.LEFT )
        {
            clip = new RoundRectangle2D.Double ( insets.left + tabAreaLength, insets.top,
                    tabPane.getWidth () - insets.left - tabAreaLength - insets.right,
                    tabPane.getHeight () - insets.top - insets.bottom, round * 2, round * 2 );
        }
        else if ( tabPlacement == JTabbedPane.RIGHT )
        {
            clip = new RoundRectangle2D.Double ( insets.left, insets.top,
                    tabPane.getWidth () - insets.left - tabAreaLength - insets.right,
                    tabPane.getHeight () - insets.top - insets.bottom, round * 2, round * 2 );
        }

        return clip;
    }

    protected void paintFocusIndicator ( Graphics g, int tabPlacement, Rectangle[] rects,
                                         int tabIndex, Rectangle iconRect, Rectangle textRect,
                                         boolean isSelected )
    {
        // We don't need this one
    }

    //    protected void setRolloverTab ( int index )
    //    {
    //        super.setRolloverTab ( index );
    //
    //        // todo Animate rollover
    //    }

    //    public Dimension getPreferredSize ( JComponent c )
    //    {
    //        if ( tabPane.getTabPlacement () == JTabbedPane.TOP ||
    //                tabPane.getTabPlacement () == JTabbedPane.BOTTOM )
    //        {
    //            getTab
    //        }else {
    //
    //        }
    //    }

    //    protected LayoutManager createLayoutManager ()
    //    {
    //        return new TabbedPaneLayout ();
    //    }
    //
    //    public class TabbedPaneLayout extends BasicTabbedPaneUI.TabbedPaneLayout
    //    {
    //
    //        public TabbedPaneLayout ()
    //        {
    //            WebTabbedPaneUI.this.super ();
    //        }
    //
    //        protected void normalizeTabRuns ( int tabPlacement, int tabCount, int start, int max )
    //        {
    //            // Only normalize the runs for top & bottom;  normalizing
    //            // doesn't look right for Metal's vertical tabs
    //            // because the last run isn't padded and it looks odd to have
    //            // fat tabs in the first vertical runs, but slimmer ones in the
    //            // last (this effect isn't noticeable for horizontal tabs).
    //            if ( tabPlacement == TOP || tabPlacement == BOTTOM )
    //            {
    //                super.normalizeTabRuns ( tabPlacement, tabCount, start, max );
    //            }
    //        }
    //
    //        // Don't rotate runs!
    //        protected void rotateTabRuns ( int tabPlacement, int selectedRun )
    //        {
    //        }
    //
    //        // Don't pad selected tab
    //        protected void padSelectedTab ( int tabPlacement, int selectedIndex )
    //        {
    //        }
    //    }
}
