/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.table;

import com.alee.laf.WebLookAndFeel;
import com.alee.utils.ReflectionUtils;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.plaf.UIResource;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import java.awt.*;
import java.util.Vector;

/**
 * User: mgarin Date: 07.07.11 Time: 17:55
 */

public class WebTable extends JTable
{
    public WebTable ()
    {
        super ();
    }

    public WebTable ( TableModel dm )
    {
        super ( dm );
    }

    public WebTable ( TableModel dm, TableColumnModel cm )
    {
        super ( dm, cm );
    }

    public WebTable ( TableModel dm, TableColumnModel cm, ListSelectionModel sm )
    {
        super ( dm, cm, sm );
    }

    public WebTable ( int numRows, int numColumns )
    {
        super ( numRows, numColumns );
    }

    public WebTable ( Vector rowData, Vector columnNames )
    {
        super ( rowData, columnNames );
    }

    public WebTable ( Object[][] rowData, Object[] columnNames )
    {
        super ( rowData, columnNames );
    }

    public WebTableUI getWebUI ()
    {
        return ( WebTableUI ) getUI ();
    }

    public void updateUI ()
    {
        // Update table header UI
        if ( getTableHeader () != null )
        {
            getTableHeader ().updateUI ();
        }

        // Update table scroll view and UI
        configureEnclosingScrollPaneUI ();

        // Update table UI
        if ( getUI () == null || !( getUI () instanceof WebTableUI ) )
        {
            try
            {
                setUI ( ( WebTableUI ) ReflectionUtils.createInstance ( WebLookAndFeel.tableUI ) );
            }
            catch ( Throwable e )
            {
                setUI ( new WebTableUI () );
            }
        }
        else
        {
            setUI ( getUI () );
        }
    }

    private void configureEnclosingScrollPaneUI ()
    {
        Container p = getParent ();
        if ( p instanceof JViewport )
        {
            Container gp = p.getParent ();
            if ( gp instanceof JScrollPane )
            {
                JScrollPane scrollPane = ( JScrollPane ) gp;
                // Make certain we are the viewPort's view and not, for
                // example, the rowHeaderView of the scrollPane -
                // an implementor of fixed columns might do this.
                JViewport viewport = scrollPane.getViewport ();
                if ( viewport == null || viewport.getView () != this )
                {
                    return;
                }
                //  scrollPane.getViewport().setBackingStoreEnabled(true);
                Border border = scrollPane.getBorder ();
                if ( border == null || border instanceof UIResource )
                {
                    Border scrollPaneBorder = UIManager.getBorder ( "Table.scrollPaneBorder" );
                    if ( scrollPaneBorder != null )
                    {
                        scrollPane.setBorder ( scrollPaneBorder );
                    }
                }
                // add JScrollBar corner component if available from LAF and not already set by the user
                Component corner = scrollPane.getCorner ( JScrollPane.UPPER_TRAILING_CORNER );
                if ( corner == null || corner instanceof UIResource )
                {
                    corner = null;
                    Object componentClass = UIManager.get ( "Table.scrollPaneCornerComponent" );
                    if ( componentClass instanceof Class )
                    {
                        try
                        {
                            corner = ( Component ) ( ( Class ) componentClass ).newInstance ();
                        }
                        catch ( Exception e )
                        {
                            // just ignore and don't set corner
                        }
                    }
                    scrollPane.setCorner ( JScrollPane.UPPER_TRAILING_CORNER, corner );
                }
            }
        }
    }
}
