/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.text;

import com.alee.laf.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.laf.WebLookAndFeel;
import com.alee.utils.ReflectionUtils;

import javax.swing.*;
import javax.swing.text.Document;
import java.awt.*;

/**
 * User: mgarin Date: 28.06.11 Time: 1:13
 */

public class WebTextField extends JTextField
{
    public WebTextField ()
    {
        super ();
    }

    public WebTextField ( String text )
    {
        super ( text );
    }

    public WebTextField ( int columns )
    {
        super ( columns );
    }

    public WebTextField ( String text, int columns )
    {
        super ( text, columns );
    }

    public WebTextField ( Document doc, String text, int columns )
    {
        super ( doc, text, columns );
    }

    public boolean isDrawBorder ()
    {
        return getWebUI ().isDrawBorder ();
    }

    public void setDrawBorder ( boolean drawBorder )
    {
        getWebUI ().setDrawBorder ( drawBorder );
    }

    public boolean isDrawFocus ()
    {
        return getWebUI ().isDrawFocus ();
    }

    public void setDrawFocus ( boolean drawFocus )
    {
        getWebUI ().setDrawFocus ( drawFocus );
    }

    public JComponent getLeadingComponent ()
    {
        return getWebUI ().getLeadingComponent ();
    }

    public void setLeadingComponent ( JComponent leadingComponent )
    {
        getWebUI ().setLeadingComponent ( leadingComponent );
    }

    public JComponent getTrailingComponent ()
    {
        return getWebUI ().getTrailingComponent ();
    }

    public void setTrailingComponent ( JComponent trailingComponent )
    {
        getWebUI ().setTrailingComponent ( trailingComponent );
    }

    public int getComponentSpacing ()
    {
        return getWebUI ().getComponentSpacing ();
    }

    public void setComponentSpacing ( int componentSpacing )
    {
        getWebUI ().setComponentSpacing ( componentSpacing );
    }

    public void setLayoutMargin ( Insets margin )
    {
        getWebUI ().setLayoutMargin ( margin );
    }

    public Insets getLayoutMargin ()
    {
        return getWebUI ().getLayoutMargin ();
    }

    public int getRound ()
    {
        return getWebUI ().getRound ();
    }

    public void setRound ( int round )
    {
        getWebUI ().setRound ( round );
    }

    public int getShadeWidth ()
    {
        return getWebUI ().getShadeWidth ();
    }

    public void setShadeWidth ( int shadeWidth )
    {
        getWebUI ().setShadeWidth ( shadeWidth );
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return getWebUI ().getBackgroundPainter ();
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        getWebUI ().setBackgroundPainter ( backgroundPainter );
    }

    public WebTextFieldUI getWebUI ()
    {
        return ( WebTextFieldUI ) getUI ();
    }

    public void updateUI ()
    {
        if ( getUI () == null || !( getUI () instanceof WebTextFieldUI ) )
        {
            try
            {
                setUI ( ( WebTextFieldUI ) ReflectionUtils
                        .createInstance ( WebLookAndFeel.textFieldUI, this ) );
            }
            catch ( Throwable e )
            {
                setUI ( new WebTextFieldUI ( this ) );
            }
        }
        else
        {
            setUI ( getUI () );
        }
        invalidate ();
    }

    public static WebTextField createWebTextField ()
    {
        return createWebTextField ( StyleConstants.drawBorder );
    }

    public static WebTextField createWebTextField ( boolean drawBorder )
    {
        return createWebTextField ( drawBorder, StyleConstants.round );
    }

    public static WebTextField createWebTextField ( boolean drawBorder, int round )
    {
        return createWebTextField ( drawBorder, round, StyleConstants.shadeWidth );
    }

    public static WebTextField createWebTextField ( boolean drawBorder, int round, int shadeWidth )
    {
        WebTextField webTextField = new WebTextField ();
        webTextField.setDrawBorder ( drawBorder );
        webTextField.setRound ( round );
        webTextField.setShadeWidth ( shadeWidth );
        return webTextField;
    }
}
