/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.tooltip;

import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicToolTipUI;
import java.awt.*;

/**
 * User: mgarin Date: 17.08.11 Time: 23:07
 */

public class WebToolTipUI extends BasicToolTipUI implements ShapeProvider
{
    private static WebToolTipUI sharedInstance = new WebToolTipUI ();

    private Color tooltipTextColor = WebTooltipStyle.tooltipTextColor;
    private int round = WebTooltipStyle.round;
    private Insets contentMargin = WebTooltipStyle.contentMargin;

    private JComponent tooltip = null;

    public static ComponentUI createUI ( JComponent c )
    {
        return sharedInstance;
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        this.tooltip = c;

        c.setOpaque ( false );
        c.setForeground ( tooltipTextColor );

        updateBorder ( c );
    }

    public void uninstallUI ( JComponent c )
    {
        this.tooltip = null;

        super.uninstallUI ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( tooltip, 0, getRound () );
    }

    private void updateBorder ( JComponent c )
    {
        c.setBorder ( BorderFactory
                .createEmptyBorder ( contentMargin.top, contentMargin.left, contentMargin.bottom,
                        contentMargin.right ) );
    }

    public Insets getContentMargin ()
    {
        return contentMargin;
    }

    public void setContentMargin ( Insets contentMargin )
    {
        this.contentMargin = contentMargin;
        updateBorder ( tooltip );
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public Color getTooltipTextColor ()
    {
        return tooltipTextColor;
    }

    public void setTooltipTextColor ( Color tooltipTextColor )
    {
        this.tooltipTextColor = tooltipTextColor;
        this.tooltip.setForeground ( tooltipTextColor );
    }

    public void paint ( Graphics g, JComponent c )
    {
        LafUtils.drawWebBorder ( ( Graphics2D ) g, c, StyleConstants.shadeColor, 0, round, true,
                true );

        super.paint ( g, c );
    }
}
