/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.hotkey;

import com.alee.NonObfuscable;

import java.awt.event.KeyEvent;

/**
 * User: mgarin Date: 11.07.11 Time: 13:24
 * <p/>
 * This interface provides list of predefine dotkeys which can be easily used to setup any hotkey
 * through WebLaF HotkeyManager system. More different standard hotkeys will be added to this
 * interface with time to make even more powerful.
 */

public interface Hotkey extends NonObfuscable
{
    public static final HotkeyData F1 = new HotkeyData ( false, false, false, KeyEvent.VK_F1 );
    public static final HotkeyData F2 = new HotkeyData ( false, false, false, KeyEvent.VK_F2 );
    public static final HotkeyData F3 = new HotkeyData ( false, false, false, KeyEvent.VK_F3 );
    public static final HotkeyData F4 = new HotkeyData ( false, false, false, KeyEvent.VK_F4 );
    public static final HotkeyData F5 = new HotkeyData ( false, false, false, KeyEvent.VK_F5 );
    public static final HotkeyData F6 = new HotkeyData ( false, false, false, KeyEvent.VK_F6 );
    public static final HotkeyData F7 = new HotkeyData ( false, false, false, KeyEvent.VK_F7 );
    public static final HotkeyData F8 = new HotkeyData ( false, false, false, KeyEvent.VK_F8 );
    public static final HotkeyData F9 = new HotkeyData ( false, false, false, KeyEvent.VK_F9 );
    public static final HotkeyData F10 = new HotkeyData ( false, false, false, KeyEvent.VK_F10 );
    public static final HotkeyData F11 = new HotkeyData ( false, false, false, KeyEvent.VK_F11 );
    public static final HotkeyData F12 = new HotkeyData ( false, false, false, KeyEvent.VK_F12 );

    public static final HotkeyData BACKSPACE =
            new HotkeyData ( false, false, false, KeyEvent.VK_BACK_SPACE );
    public static final HotkeyData DELETE =
            new HotkeyData ( false, false, false, KeyEvent.VK_DELETE );
    public static final HotkeyData ESCAPE =
            new HotkeyData ( false, false, false, KeyEvent.VK_ESCAPE );
    public static final HotkeyData ENTER =
            new HotkeyData ( false, false, false, KeyEvent.VK_ENTER );
    public static final HotkeyData PRINTSCREEN =
            new HotkeyData ( false, false, false, KeyEvent.VK_PRINTSCREEN );
    public static final HotkeyData PAGE_UP =
            new HotkeyData ( false, false, false, KeyEvent.VK_PAGE_UP );
    public static final HotkeyData PAGE_DOWN =
            new HotkeyData ( false, false, false, KeyEvent.VK_PAGE_DOWN );

    public static final HotkeyData ALT_A = new HotkeyData ( false, true, false, KeyEvent.VK_A );
    public static final HotkeyData ALT_C = new HotkeyData ( false, true, false, KeyEvent.VK_C );
    public static final HotkeyData ALT_D = new HotkeyData ( false, true, false, KeyEvent.VK_D );
    public static final HotkeyData ALT_F = new HotkeyData ( false, true, false, KeyEvent.VK_F );
    public static final HotkeyData ALT_G = new HotkeyData ( false, true, false, KeyEvent.VK_G );
    public static final HotkeyData ALT_H = new HotkeyData ( false, true, false, KeyEvent.VK_H );
    public static final HotkeyData ALT_R = new HotkeyData ( false, true, false, KeyEvent.VK_R );
    public static final HotkeyData ALT_S = new HotkeyData ( false, true, false, KeyEvent.VK_S );

    public static final HotkeyData ALT_F4 = new HotkeyData ( false, true, false, KeyEvent.VK_F4 );

    public static final HotkeyData ALT_ENTER =
            new HotkeyData ( false, true, false, KeyEvent.VK_ENTER );

    public static final HotkeyData ALT_LEFT =
            new HotkeyData ( false, true, false, KeyEvent.VK_LEFT );
    public static final HotkeyData ALT_RIGHT =
            new HotkeyData ( false, true, false, KeyEvent.VK_RIGHT );
    public static final HotkeyData ALT_UP = new HotkeyData ( false, true, false, KeyEvent.VK_UP );
    public static final HotkeyData ALT_DOWN =
            new HotkeyData ( false, true, false, KeyEvent.VK_DOWN );

    public static final HotkeyData CTRL_A = new HotkeyData ( true, false, false, KeyEvent.VK_A );
    public static final HotkeyData CTRL_B = new HotkeyData ( true, false, false, KeyEvent.VK_B );
    public static final HotkeyData CTRL_C = new HotkeyData ( true, false, false, KeyEvent.VK_C );
    public static final HotkeyData CTRL_E = new HotkeyData ( true, false, false, KeyEvent.VK_E );
    public static final HotkeyData CTRL_F = new HotkeyData ( true, false, false, KeyEvent.VK_F );
    public static final HotkeyData CTRL_G = new HotkeyData ( true, false, false, KeyEvent.VK_G );
    public static final HotkeyData CTRL_N = new HotkeyData ( true, false, false, KeyEvent.VK_N );
    public static final HotkeyData CTRL_O = new HotkeyData ( true, false, false, KeyEvent.VK_O );
    public static final HotkeyData CTRL_P = new HotkeyData ( true, false, false, KeyEvent.VK_P );
    public static final HotkeyData CTRL_S = new HotkeyData ( true, false, false, KeyEvent.VK_S );
    public static final HotkeyData CTRL_X = new HotkeyData ( true, false, false, KeyEvent.VK_X );
    public static final HotkeyData CTRL_V = new HotkeyData ( true, false, false, KeyEvent.VK_V );
    public static final HotkeyData CTRL_Z = new HotkeyData ( true, false, false, KeyEvent.VK_Z );

    public static final HotkeyData CTRL_SHIFT_S =
            new HotkeyData ( true, false, true, KeyEvent.VK_S );

    public static final HotkeyData CTRL_ENTER =
            new HotkeyData ( true, false, false, KeyEvent.VK_ENTER );

    public static final HotkeyData CTRL_LEFT =
            new HotkeyData ( true, false, false, KeyEvent.VK_LEFT );
    public static final HotkeyData CTRL_RIGHT =
            new HotkeyData ( true, false, false, KeyEvent.VK_RIGHT );
    public static final HotkeyData CTRL_UP = new HotkeyData ( true, false, false, KeyEvent.VK_UP );
    public static final HotkeyData CTRL_DOWN =
            new HotkeyData ( true, false, false, KeyEvent.VK_DOWN );
}
