/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.proxy;

import com.alee.NonObfuscable;

import java.io.Serializable;

/**
 * User: mgarin Date: 12.10.11 Time: 17:49
 */

public class ProxySettings implements Serializable, NonObfuscable
{
    private boolean useProxy = false;
    private String proxyHost = null;
    private String proxyPort = null;
    private String nonProxyHosts = null;
    private boolean useProxyAuthentification = false;
    private String proxyLogin = null;
    private String proxyPassword = null;

    public ProxySettings ()
    {
        this ( null, null );
    }

    public ProxySettings ( String proxyHost, String proxyPort )
    {
        this ( proxyHost, proxyPort, null, null );
    }

    public ProxySettings ( String proxyHost, String proxyPort, String proxyLogin,
                           String proxyPassword )
    {
        super ();

        if ( proxyHost != null && proxyPort != null )
        {
            this.useProxy = true;
            this.proxyHost = proxyHost;
            this.proxyPort = proxyPort;

            if ( proxyLogin != null && proxyPassword != null )
            {
                this.useProxyAuthentification = true;
                this.proxyLogin = proxyLogin;
                this.proxyPassword = proxyPassword;
            }
            else
            {
                this.useProxyAuthentification = false;
            }
        }
        else
        {
            this.useProxy = false;
            this.useProxyAuthentification = false;
        }
    }

    public boolean isUseProxy ()
    {
        return useProxy;
    }

    public void setUseProxy ( boolean useProxy )
    {
        this.useProxy = useProxy;
    }

    public String getProxyHost ()
    {
        return proxyHost;
    }

    public void setProxyHost ( String proxyHost )
    {
        this.proxyHost = proxyHost;
    }

    public String getProxyPort ()
    {
        return proxyPort;
    }

    public int getProxyPortInt ()
    {
        try
        {
            return Integer.parseInt ( proxyPort );
        }
        catch ( Throwable e )
        {
            return 80;
        }
    }

    public void setProxyPort ( String proxyPort )
    {
        this.proxyPort = proxyPort;
    }

    public String getNonProxyHosts ()
    {
        return nonProxyHosts;
    }

    public void setNonProxyHosts ( String nonProxyHosts )
    {
        this.nonProxyHosts = nonProxyHosts;
    }

    public boolean isUseProxyAuthentification ()
    {
        return useProxyAuthentification;
    }

    public void setUseProxyAuthentification ( boolean useProxyAuthentification )
    {
        this.useProxyAuthentification = useProxyAuthentification;
    }

    public String getProxyLogin ()
    {
        return proxyLogin;
    }

    public void setProxyLogin ( String proxyLogin )
    {
        this.proxyLogin = proxyLogin;
    }

    public String getProxyPassword ()
    {
        return proxyPassword;
    }

    public void setProxyPassword ( String proxyPassword )
    {
        this.proxyPassword = proxyPassword;
    }
}
