/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.proxy;

import com.alee.laf.label.WebLabel;
import com.alee.laf.optionpane.WebOptionPane;
import com.alee.laf.text.WebPasswordField;
import com.alee.laf.text.WebTextField;
import com.alee.utils.SwingUtils;
import info.clearthought.layout.TableLayout;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import java.net.Authenticator;
import java.net.PasswordAuthentication;

/**
 * User: mgarin Date: 19.10.11 Time: 13:29
 */

public class WebProxyAuthenticator extends Authenticator
{
    protected PasswordAuthentication getPasswordAuthentication ()
    {
        ProxySettings proxySettings = ProxyManager.getProxySettings ();
        if ( proxySettings.isUseProxyAuthentification () )
        {
            return new PasswordAuthentication ( proxySettings.getProxyLogin (),
                    proxySettings.getProxyPassword ().toCharArray () );
        }
        else
        {
            JPanel authPanel = new JPanel ();
            authPanel.setOpaque ( false );
            TableLayout authLayout = new TableLayout (
                    new double[][]{ { TableLayout.PREFERRED, TableLayout.PREFERRED },
                            { TableLayout.PREFERRED, TableLayout.PREFERRED } } );
            authLayout.setHGap ( 2 );
            authLayout.setVGap ( 2 );
            authPanel.setLayout ( authLayout );

            authPanel.add ( new WebLabel ( "Login:", WebLabel.RIGHT ), "0,0" );
            final WebTextField loginField = new WebTextField ( 12 );
            loginField.addAncestorListener ( new AncestorListener ()
            {
                public void ancestorAdded ( AncestorEvent event )
                {
                    loginField.requestFocusInWindow ();
                }

                public void ancestorRemoved ( AncestorEvent event )
                {
                    //
                }

                public void ancestorMoved ( AncestorEvent event )
                {
                    //
                }
            } );
            authPanel.add ( loginField, "1,0" );

            authPanel.add ( new WebLabel ( "Password:", WebLabel.RIGHT ), "0,1" );
            WebPasswordField passwordField = new WebPasswordField ( 12 );
            authPanel.add ( passwordField, "1,1" );

            int confirm = WebOptionPane
                    .showConfirmDialog ( SwingUtils.getActiveWindow (), authPanel,
                            "Proxy authentification required", WebOptionPane.OK_CANCEL_OPTION,
                            WebOptionPane.QUESTION_MESSAGE );

            if ( confirm == WebOptionPane.OK_OPTION )
            {
                proxySettings.setUseProxyAuthentification ( true );
                proxySettings.setProxyLogin ( loginField.getText () );
                proxySettings.setProxyPassword ( new String ( passwordField.getPassword () ) );

                return new PasswordAuthentication ( loginField.getText (),
                        passwordField.getPassword () );
            }
            else
            {
                return null;
            }
        }
    }

    //    public static class AuthentificationDialog extends JDialog
    //    {
    //        public AuthentificationDialog ()
    //        {
    //            this ( SwingUtils.getActiveWindow () );
    //        }
    //
    //        public AuthentificationDialog ( Window owner )
    //        {
    //            super ( owner );
    //
    //            setLayout ( new BorderLayout () );
    //
    //            JPanel container = new JPanel ();
    //            container.setOpaque ( false );
    //            container.setLayout ( new BorderLayout () );
    //            container.setBorder ( BorderFactory.createEmptyBorder ( 8, 8, 8, 8 ) );
    //            add ( container, BorderLayout.CENTER );
    //
    //
    //            JPanel centerPanel = new JPanel ();
    //            centerPanel.setLayout ( new BorderLayout ( 10, 10 ) );
    //            container.add ( centerPanel, BorderLayout.CENTER );
    //
    //
    //            centerPanel.add ( new WebLabel ( new ImageIcon (
    //                    WebProxyAuthenticator.class.getResource ( "icons/auth.png" ) ) ) );
    //
    //
    //            JPanel authPanel = new JPanel ();
    //            authPanel.setOpaque ( false );
    //            TableLayout authLayout = new TableLayout (
    //                    new double[][]{ { TableLayout.PREFERRED, TableLayout.PREFERRED },
    //                            { TableLayout.PREFERRED, TableLayout.PREFERRED,
    //                                    TableLayout.PREFERRED } } );
    //            authLayout.setHGap ( 2 );
    //            authLayout.setVGap ( 2 );
    //            authPanel.setLayout ( authLayout );
    //            centerPanel.add ( authPanel, BorderLayout.EAST );
    //
    //            ProxySettings proxySettings = ProxyManager.getProxySettings ();
    //            authPanel.add ( new WebLabel (
    //                    "<html><b>" + proxySettings.getProxyHost () + "</b> (port: " +
    //                            proxySettings.getProxyPort () + ")" ), "1,0" );
    //
    //            authPanel.add ( new WebLabel ( "Login:", WebLabel.RIGHT ), "0,1" );
    //            WebTextField loginField = new WebTextField ( 12 );
    //            authPanel.add ( loginField, "1,1" );
    //
    //            authPanel.add ( new WebLabel ( "Password:", WebLabel.RIGHT ), "0,2" );
    //            WebPasswordField passwordField = new WebPasswordField ( 12 );
    //            authPanel.add ( passwordField, "1,2" );
    //
    //
    //            JPanel southPanel = new JPanel ();
    //            southPanel.setLayout ( new BorderLayout () );
    //            southPanel.setBorder ( BorderFactory.createEmptyBorder ( 8, 0, 0, 0 ) );
    //            container.add ( southPanel, BorderLayout.SOUTH );
    //
    //            WebCheckBox saveSettings = new WebCheckBox ( "Save settings" );
    //            saveSettings.setSelected ( true );
    //            southPanel.add ( saveSettings, BorderLayout.CENTER );
    //
    //            southPanel.add ( new GroupPanel ( new WebButton ( "Ok" ), new WebButton ( "Cancel" ) ),
    //                    BorderLayout.EAST );
    //
    //            setModal ( true );
    //            setResizable ( false );
    //            pack ();
    //            setLocationRelativeTo ( owner );
    //        }
    //    }
}