/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.tooltip;

import com.alee.laf.StyleConstants;

import java.awt.*;

/**
 * User: mgarin Date: 19.10.11 Time: 16:49
 */

public final class CustomTooltipStyle
{
    /*
    * Default tooltip display way
    * If set to null - best display way will be calculated each time tooltip shown
    */
    public static TooltipWay displayWay = null;

    /*
    * Show component (to which shown tooltip attached) hotkeys information at the right side
    * Only non-hidden hotkeys information will be shown
    */
    public static boolean showHotkey = true;

    /*
    * Should use default tooltips close behavior (close on component area exit, mouse/key press e.t.c.)
    * or allow user to define his own close behavior (in this case tooltips won't close by default)
    */
    public static boolean defaultCloseBehavior = true;

    /*
    * Spacing between tooltip visual sides (visual tooltip border) and tooltip content
    * If set to 0 - content will fit tooltip visual bounds
    */
    public static int contentSpacing = StyleConstants.mediumContentSpacing;

    /*
    * Additional left and right sides content spacing
    * Mainly used to improve text readability in tooltips
    */
    public static int leftRightSpacing = StyleConstants.smallLeftRightSpacing;

    /*
    * Minimal spacing between window edge and tooltip
    * Used to avoid tooltips "crashing" into window edge when shown close to it
    */
    public static int windowSideSpacing = 4;

    /*
    * Tooltip corners rounding
    */
    public static int round = StyleConstants.round;

    /*
    * Tooltip border color
    * If set to null border won't be drawn at all
    */
    public static Color borderColor = StyleConstants.darkBorderColor;

    /*
    * Tooltip top background color
    * If set to null background won't be drawn at all
    */
    public static Color topBgColor = StyleConstants.topBgColor;

    /*
    * Tooltip bottom background color
    * If set to null background will be filled with topBgColor
    */
    public static Color bottomBgColor = StyleConstants.bottomBgColor;

    /*
    * Tooltip text color
    */
    public static Color textColor = StyleConstants.tooltipTextColor;

    /*
    * Tooltip view trasparency
    */
    public static float trasparency = StyleConstants.opaque;
}
