/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import java.awt.*;

/**
 * User: mgarin Date: 26.09.11 Time: 19:58
 */

public class ColorUtils
{
    /*
    * Returns Color progressed from color1 to color2 by specified percentage
    */

    public static Color getProgress ( Color color1, Color color2, float progress )
    {
        return new Color ( getProgress ( color1.getRed (), color2.getRed (), progress ),
                getProgress ( color1.getGreen (), color2.getGreen (), progress ),
                getProgress ( color1.getBlue (), color2.getBlue (), progress ) );
    }

    public static int getProgress ( int color1, int color2, float progress )
    {
        return color1 + Math.round ( ( ( float ) color2 - color1 ) * progress );
    }

    /*
    * Web-safe color
    */

    public static int getWebSafe ( int color )
    {
        if ( 0 <= color && color <= 51 )
        {
            return color > 51 - color ? 51 : 0;
        }
        else if ( 51 <= color && color <= 102 )
        {
            return 51 + color > 102 - color ? 102 : 51;
        }
        else if ( 102 <= color && color <= 153 )
        {
            return 102 + color > 153 - color ? 153 : 102;
        }
        else if ( 153 <= color && color <= 204 )
        {
            return 153 + color > 204 - color ? 204 : 153;
        }
        else if ( 204 <= color && color <= 255 )
        {
            return 204 + color > 255 - color ? 255 : 204;
        }
        return color;
    }

    /*
    * RGB to HEX and HEX to RGB parsing
    */

    public static String rgbToHex ( Color color )
    {
        return rgbToHex ( color.getRGB () );
    }

    public static String rgbToHex ( int rgb )
    {
        String hex = Integer.toHexString ( rgb ).toUpperCase ();
        return hex.substring ( 2, hex.length () );
    }

    public static Color hexToColor ( String hex )
    {
        return Color.decode ( "#" + hex );
    }
}
