/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.*;
import java.util.List;

/**
 * User: mgarin Date: 07.10.11 Time: 14:40
 */

public class DragUtils
{
    /*
    * Retrieves Images list from Transferable
    */

    public static Image getImportedImage ( Transferable t )
    {
        if ( t.isDataFlavorSupported ( DataFlavor.imageFlavor ) )
        {
            try
            {
                Object data = t.getTransferData ( DataFlavor.imageFlavor );
                if ( data instanceof Image )
                {
                    return ( Image ) data;
                }
            }
            catch ( UnsupportedFlavorException e )
            {
                //
            }
            catch ( IOException e )
            {
                //
            }
        }
        return null;
    }

    /*
    * Retrieves Files list from Transferable
    */

    public static List<File> getImportedFiles ( Transferable t )
    {
        // From URL
        try
        {
            if ( DragUtils.hasURIListFlavor ( t.getTransferDataFlavors () ) )
            {
                // File link
                String url = ( String ) t.getTransferData ( DragUtils.getUriListDataFlavor () );
                final File file = new File ( new URL ( url ).getPath () );

                // Decoding address
                String correctName = WebUtils.decodeUrl ( file.getName () );

                // Returning file
                return Arrays.asList ( file );
            }
        }
        catch ( Throwable e )
        {
            //
        }

        // From files list (Linux/MacOS)
        try
        {
            if ( DragUtils.hasURIListFlavor ( t.getTransferDataFlavors () ) )
            {
                // Parsing incoming files
                return DragUtils.textURIListToFileList (
                        ( String ) t.getTransferData ( DragUtils.getUriListDataFlavor () ) );
            }
        }
        catch ( Throwable e )
        {
            //
        }

        // From files list (Windows)
        try
        {
            // Getting files list
            return ( AbstractList ) t.getTransferData ( DataFlavor.javaFileListFlavor );
        }
        catch ( Throwable e )
        {
            //
        }

        return null;
    }

    /*
    * Retrieves URI list from data
    */

    public static List<File> textURIListToFileList ( String data )
    {
        List<File> list = new ArrayList<File> ( 1 );
        for ( StringTokenizer st = new StringTokenizer ( data, "\r\n" ); st.hasMoreTokens (); )
        {
            String s = st.nextToken ();
            if ( s.startsWith ( "#" ) )
            {
                // the line is a comment (as per the RFC 2483)
                continue;
            }
            try
            {
                URI uri = new URI ( s );
                File file = new File ( uri );
                list.add ( file );
            }
            catch ( Throwable e )
            {
                //
            }
        }
        return list;
    }

    /*
    * Checks if there is files URI list DataFlavor
    */

    public static boolean hasURIListFlavor ( DataFlavor[] flavors )
    {
        for ( DataFlavor flavor : flavors )
        {
            if ( getUriListDataFlavor ().equals ( flavor ) )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Files URI list flavor
     */

    private static final String URI_LIST_MIME_TYPE = "text/uri-list;class=java.lang.String";
    private static DataFlavor uriListFlavor = null;

    public static DataFlavor getUriListDataFlavor ()
    {
        if ( uriListFlavor == null )
        {
            try
            {
                return uriListFlavor = new DataFlavor ( URI_LIST_MIME_TYPE );
            }
            catch ( Throwable e )
            {
                return null;
            }
        }
        else
        {
            return uriListFlavor;
        }
    }
}
