/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * User: mgarin Date: 11/18/11 Time: 4:09 PM
 */

public class ReflectionUtils
{
    /*
    * Retrieves class name only
    */

    public static String getClassName ( Class c )
    {
        String fullName = c.getCanonicalName ();
        int dot = fullName.lastIndexOf ( "." );
        return dot != -1 ? fullName.substring ( dot + 1 ) : fullName;
    }

    /*
    * Creates class instance by its canonical name and constructor arguments
    */

    public static Object createInstanceSafely ( String canonicalClassName, Object... arguments )
    {
        try
        {
            return createInstance ( canonicalClassName, arguments );
        }
        catch ( Throwable e )
        {
            return null;
        }
    }

    public static Object createInstance ( String canonicalClassName, Object... arguments )
            throws ClassNotFoundException, InvocationTargetException, IllegalAccessException,
            InstantiationException
    {
        Class theClass = ReflectionUtils.class.getClassLoader ().loadClass ( canonicalClassName );
        Constructor constructor = theClass.getConstructors ()[ 0 ];
        return constructor.newInstance ( arguments );
    }
}
