/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import com.alee.extended.date.WebCalendar;
import com.alee.extended.date.WebDateField;
import com.alee.extended.filechooser.WebFileChooserField;
import com.alee.extended.filechooser.WebPathField;
import com.alee.extended.panel.WebCollapsiblePane;
import com.alee.managers.hotkey.HotkeyData;
import com.alee.managers.hotkey.HotkeyRunnable;

import javax.swing.*;
import javax.swing.plaf.basic.BasicButtonUI;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 20.07.11 Time: 14:41
 */

public class SwingUtils
{
    /*
    * Creates full component size
    */

    public static Rectangle size ( Component component )
    {
        return new Rectangle ( 0, 0, component.getWidth (), component.getHeight () );
    }

    /*
    * Gets index of object in the array
    */

    public static int indexOf ( String text, String[] array )
    {
        return indexOf ( text, array, false );
    }

    public static int indexOf ( String text, String[] array, boolean ignoreCase )
    {
        for ( int i = 0; i < array.length; i++ )
        {
            String txt = array[ i ];
            if ( txt == null && text == null ||
                    txt != null && text != null && txt.equalsIgnoreCase ( text ) )
            {
                return i;
            }
        }
        return -1;
    }

    public static int indexOf ( Object object, Object[] array )
    {
        for ( int i = 0; i < array.length; i++ )
        {
            Object obj = array[ i ];
            if ( obj == null && object == null ||
                    obj != null && object != null && obj.equals ( object ) )
            {
                return i;
            }
        }
        return -1;
    }

    public static int indexOf ( int number, int[] array )
    {
        for ( int i = 0; i < array.length; i++ )
        {
            int num = array[ i ];
            if ( num == number )
            {
                return i;
            }
        }
        return -1;
    }

    public static int indexOf ( char character, char[] array )
    {
        for ( int i = 0; i < array.length; i++ )
        {
            char ch = array[ i ];
            if ( ch == character )
            {
                return i;
            }
        }
        return -1;
    }

    /*
    * Gets max Insets out of two
    */

    public static Insets max ( Insets insets1, Insets insets2 )
    {
        return new Insets ( Math.max ( insets1.top, insets2.top ),
                Math.max ( insets1.left, insets2.left ),
                Math.max ( insets1.bottom, insets2.bottom ),
                Math.max ( insets1.right, insets2.right ) );
    }

    /*
    * Gets min Insets out of two
    */

    public static Insets min ( Insets insets1, Insets insets2 )
    {
        return new Insets ( Math.min ( insets1.top, insets2.top ),
                Math.min ( insets1.left, insets2.left ),
                Math.min ( insets1.bottom, insets2.bottom ),
                Math.min ( insets1.right, insets2.right ) );
    }

    /*
    * Gets max Dimension out of two
    */

    public static Dimension max ( Dimension dimension1, Dimension dimension2 )
    {
        if ( dimension1 == null && dimension2 == null )
        {
            return null;
        }
        else if ( dimension1 == null )
        {
            return dimension2;
        }
        else if ( dimension2 == null )
        {
            return dimension1;
        }
        else
        {
            return new Dimension ( Math.max ( dimension1.width, dimension2.width ),
                    Math.max ( dimension1.height, dimension2.height ) );
        }
    }

    /*
    * Gets min Dimension out of two
    */

    public static Dimension min ( Dimension dimension1, Dimension dimension2 )
    {
        if ( dimension1 == null || dimension2 == null )
        {
            return null;
        }
        else
        {
            return new Dimension ( Math.min ( dimension1.width, dimension2.width ),
                    Math.min ( dimension1.height, dimension2.height ) );
        }
    }

    /*
    * Sets opaque state of component and all its childs
    */

    public static void setOpaqueRecursively ( Component component, boolean opaque )
    {
        if ( component instanceof JComponent )
        {
            ( ( JComponent ) component ).setOpaque ( opaque );
            for ( Component child : ( ( JComponent ) component ).getComponents () )
            {
                setOpaqueRecursively ( child, opaque );
            }
        }
    }

    /*
    * Sets enabled state of component and all its childs
    */

    public static void setEnabledRecursively ( Component component, boolean enabled )
    {
        component.setEnabled ( enabled );
        if ( component instanceof Container )
        {
            for ( Component child : ( ( Container ) component ).getComponents () )
            {
                setEnabledRecursively ( child, enabled );
            }
        }
    }

    /*
    * Sets foreground of component and all its childs
    */

    public static void setForegroundRecursively ( JComponent component, Color foreground )
    {
        component.setForeground ( foreground );
        for ( int i = 0; i < component.getComponentCount (); i++ )
        {
            if ( component.getComponent ( i ) instanceof JComponent )
            {
                setForegroundRecursively ( ( JComponent ) component.getComponent ( i ),
                        foreground );
            }
        }
    }

    /*
    * Sets component and all its childs focusable or not
    */

    public static void setFocusableRecursively ( JComponent component, boolean focusable )
    {
        component.setFocusable ( focusable );
        for ( int i = 0; i < component.getComponentCount (); i++ )
        {
            if ( component.getComponent ( i ) instanceof JComponent )
            {
                setFocusableRecursively ( ( JComponent ) component.getComponent ( i ), focusable );
            }
        }
    }

    /*
    * Sets font of component and all its childs
    */

    public static void setFontRecursively ( JComponent component, Font font )
    {
        component.setFont ( font );

        for ( int i = 0; i < component.getComponentCount (); i++ )
        {
            if ( component.getComponent ( i ) instanceof JComponent )
            {
                setFontRecursively ( ( JComponent ) component.getComponent ( i ), font );
            }
        }
    }

    /*
    * Creates component snapshot image (component must be showing)
    */

    public static BufferedImage createComponentSnapshot ( Component content )
    {
        return createComponentSnapshot ( content, content.getWidth (), content.getHeight () );
    }

    public static BufferedImage createComponentSnapshot ( Component content, int width, int height )
    {
        BufferedImage bi = new BufferedImage ( width, height, BufferedImage.TYPE_INT_ARGB );
        Graphics2D g2d = bi.createGraphics ();
        content.setSize ( width, height );
        content.paintAll ( g2d );
        g2d.dispose ();
        return bi;
    }

    /*
    * Setups menu item accelerator from standart Web hotkey data
    */

    public static void setHotkey ( JMenuItem menuItem, HotkeyData hotkey )
    {
        if ( hotkey.isHotkeySet () )
        {
            menuItem.setAccelerator ( KeyStroke.getKeyStroke ( hotkey.getKeyCode (),
                    ( hotkey.isCtrl () ? getSystemShortcut () : 0 ) |
                            ( hotkey.isAlt () ? KeyEvent.ALT_MASK : 0 ) |
                            ( hotkey.isShift () ? KeyEvent.SHIFT_MASK : 0 ) ) );
        }
    }

    /*
    * Returns top active window
    */

    public static Window getActiveWindow ()
    {
        Window[] windows = Window.getWindows ();
        Window window = null;
        for ( Window w : windows )
        {
            if ( w.isVisible () && w.isActive () && w.isFocused () )
            {
                window = w;
                break;
            }
        }
        return window;
    }

    /*
    * Nullifies button styles
    */

    public static void nullifyButtonUI ( JButton button )
    {
        button.setUI ( new BasicButtonUI () );
        button.setMargin ( new Insets ( 0, 0, 0, 0 ) );
        button.setBorder ( BorderFactory.createEmptyBorder ( 0, 0, 0, 0 ) );
        button.setBorderPainted ( false );
        button.setContentAreaFilled ( false );
        button.setFocusable ( false );
        button.setOpaque ( false );
    }

    /*
    * Determines if system shortcut modifier used in the specified event
    */

    public static boolean isShortcut ( InputEvent event )
    {
        return ( event.getModifiers () & getSystemShortcut () ) != 0;
    }

    /*
    * Returns system shortcut modifier
    */

    public static int getSystemShortcut ()
    {
        return Toolkit.getDefaultToolkit ().getMenuShortcutKeyMask ();
    }

    /*
    * Determines if modifier used in the specified event
    */

    public static boolean isCtrl ( InputEvent event )
    {
        return ( event.getModifiers () & InputEvent.CTRL_MASK ) != 0;
    }

    public static boolean isAlt ( InputEvent event )
    {
        return ( event.getModifiers () & InputEvent.ALT_MASK ) != 0;
    }

    public static boolean isShift ( InputEvent event )
    {
        return ( event.getModifiers () & InputEvent.SHIFT_MASK ) != 0;
    }

    /*
    * Returns default label Font
    */

    private static JLabel label = null;

    public static Font getDefaultLabelFont ()
    {
        if ( label == null )
        {
            label = new JLabel ();
        }
        return label.getFont ();
    }

    /*
    * Returns scroll if components is inside it
    */

    public static JScrollPane getComponentScroll ( Component component )
    {
        if ( component != null && component.getParent () != null &&
                component.getParent () instanceof JViewport &&
                component.getParent ().getParent () != null &&
                component.getParent ().getParent () instanceof JScrollPane )
        {
            return ( JScrollPane ) component.getParent ().getParent ();
        }
        else
        {
            return null;
        }
    }

    /*
    * Find component on container with specified text
    */

    public static List<Component> findComponentsWithText ( String text, Component component )
    {
        return findComponentsWithText ( text, component, new ArrayList<Component> () );
    }

    public static List<Component> findComponentsWithText ( String text, Component component,
                                                           List<Component> components )
    {
        try
        {
            if ( text == null || text.equals ( "" ) )
            {
                return components;
            }
            if ( component instanceof WebPathField || component instanceof WebFileChooserField ||
                    component instanceof WebDateField || component instanceof WebCalendar )
            {
                // Check childs for text in composite components
                for ( Component child : ( ( Container ) component ).getComponents () )
                {
                    if ( findComponentsWithText ( text, child ).size () > 0 )
                    {
                        components.add ( component );
                        break;
                    }
                }
            }
            else if ( component instanceof WebCollapsiblePane )
            {
                // Check header panel for text
                if ( findComponentsWithText ( text,
                        ( ( WebCollapsiblePane ) component ).getHeaderPanel () ).size () > 0 )
                {
                    components.add ( component );
                }
            }
            else if ( component instanceof JComboBox )
            {
                // todo Check each renderer for text
                JComboBox comboBox = ( JComboBox ) component;
                if ( comboBox.getSelectedItem ().toString ().toLowerCase ()
                        .contains ( text.toLowerCase () ) )
                {
                    components.add ( component );
                }
                else
                {
                    if ( comboBox.isEditable () )
                    {
                        if ( findComponentsWithText ( text,
                                comboBox.getEditor ().getEditorComponent () ).size () > 0 )
                        {
                            components.add ( component );
                        }
                    }
                    else
                    {
                        if ( findComponentsWithText ( text, comboBox.getRenderer ()
                                .getListCellRendererComponent ( null, comboBox.getSelectedItem (),
                                        -1, true, true ) ).size () > 0 )
                        {
                            components.add ( component );
                        }
                    }
                }
            }
            else if ( component instanceof JSpinner )
            {
                // Check value for text
                if ( ( ( JSpinner ) component ).getValue ().toString ().toLowerCase ()
                        .contains ( text.toLowerCase () ) )
                {
                    components.add ( component );
                }
            }
            else if ( component instanceof JLabel )
            {
                // Check value for text
                if ( ( ( JLabel ) component ).getText ().toLowerCase ()
                        .contains ( text.toLowerCase () ) )
                {
                    components.add ( component );
                }
            }
            else if ( component instanceof AbstractButton )
            {
                // Check value for text
                if ( ( ( AbstractButton ) component ).getText ().toLowerCase ()
                        .contains ( text.toLowerCase () ) )
                {
                    components.add ( component );
                }
            }
            else if ( component instanceof JTextComponent )
            {
                // Check value for text
                if ( ( ( JTextComponent ) component ).getText ().toLowerCase ()
                        .contains ( text.toLowerCase () ) )
                {
                    // Return scroll instead of the text component
                    if ( component.getParent () != null &&
                            component.getParent ().getParent () != null &&
                            component.getParent ().getParent () instanceof JScrollPane )
                    {
                        components.add ( component.getParent ().getParent () );
                    }
                    else
                    {
                        components.add ( component );
                    }
                }
            }
            else if ( component instanceof JTabbedPane )
            {
                JTabbedPane tp = ( JTabbedPane ) component;

                // Only content check
                for ( int i = 0; i < tp.getTabCount (); i++ )
                {
                    if ( tp.getComponentAt ( i ) instanceof Container )
                    {
                        checkContent ( text, ( Container ) tp.getComponentAt ( i ), components );
                    }
                }

                // Content and titles check
                //                boolean found = false;
                //                for ( int i = 0; i < tp.getTabCount (); i++ )
                //                {
                //                    // Checking tab title
                //                    if ( tp.getTitleAt ( i ).toLowerCase ().contains ( text.toLowerCase () ) )
                //                    {
                //                        components.add ( component );
                //                        found = true;
                //                        break;
                //                    }
                //                    // Checking tab button content
                //                    Component tabComponent = tp.getTabComponentAt ( i );
                //                    if ( tabComponent != null )
                //                    {
                //                        if ( findComponentsWithText ( text, tabComponent ).size () > 0 )
                //                        {
                //                            components.add ( component );
                //                            found = true;
                //                            break;
                //                        }
                //                    }
                //                }
                //
                //                if (!found) {
                //                    return
                //                }
            }
            else if ( component instanceof Container )
            {
                checkContent ( text, ( Container ) component, components );
            }
        }
        catch ( Throwable e )
        {
            //
        }
        return components;
    }

    private static void checkContent ( String text, Container container,
                                       List<Component> components )
    {
        // Check childs for text
        for ( Component child : container.getComponents () )
        {
            findComponentsWithText ( text, child, components );
        }
    }

    /*
    * Component coordinates relative to another component
    */

    public static Rectangle getRelativeBounds ( Component component, Component relativeTo )
    {
        return new Rectangle ( getRelativeLocation ( component, relativeTo ),
                component.getSize () );
    }

    public static Point getRelativeLocation ( Component component, Component relativeTo )
    {
        Point los = component.getLocationOnScreen ();
        Point rt = relativeTo.getLocationOnScreen ();
        return new Point ( los.x - rt.x, los.y - rt.y );
    }

    /*
    * Are components ancestor same
    */

    public static boolean isSameAncestor ( Component component1, Component component2 )
    {
        return SwingUtilities.getWindowAncestor ( component1 ) ==
                SwingUtilities.getWindowAncestor ( component2 );
    }

    /*
    * Applies ComponentOrientation to component and all its subcomponents
    */

    public static void applyComponentOrientation ( Component c, ComponentOrientation co )
    {
        c.setComponentOrientation ( co );
        if ( c instanceof JMenu )
        {
            JMenu menu = ( JMenu ) c;
            int ncomponents = menu.getMenuComponentCount ();
            for ( int i = 0; i < ncomponents; ++i )
            {
                applyComponentOrientation ( menu.getMenuComponent ( i ), co );
            }
        }
        else if ( c instanceof Container )
        {
            Container container = ( Container ) c;
            int ncomponents = container.getComponentCount ();
            for ( int i = 0; i < ncomponents; ++i )
            {
                applyComponentOrientation ( container.getComponent ( i ), co );
            }
        }
    }

    /*
    * Makes all passed components sizes (both width and height) equal
    */

    public static void equalizeComponentsSize ( Component... components )
    {
        Dimension maxSize = new Dimension ( 0, 0 );
        for ( Component c : components )
        {
            Dimension ps = c.getPreferredSize ();
            maxSize.width = Math.max ( maxSize.width, ps.width );
            maxSize.height = Math.max ( maxSize.height, ps.height );
        }
        for ( Component c : components )
        {
            c.setPreferredSize ( maxSize );
        }
    }

    public static void equalizeComponentsWidths ( Component... components )
    {
        int maxWidth = 0;
        for ( Component c : components )
        {
            maxWidth = Math.max ( maxWidth, c.getPreferredSize ().width );
        }
        for ( Component c : components )
        {
            c.setPreferredSize ( new Dimension ( maxWidth, c.getPreferredSize ().height ) );
        }
    }

    public static void equalizeComponentsHeights ( Component... components )
    {
        int maxHeight = 0;
        for ( Component c : components )
        {
            maxHeight = Math.max ( maxHeight, c.getPreferredSize ().height );
        }
        for ( Component c : components )
        {
            c.setPreferredSize ( new Dimension ( c.getPreferredSize ().width, maxHeight ) );
        }
    }

    /*
    * Returns true if first component is equal to second one or any of its childs
    * Otherwise returns false
    */

    public static boolean isEqualOrChild ( Component component, Component compared )
    {
        if ( component == compared )
        {
            return true;
        }
        else
        {
            if ( component instanceof JComponent )
            {
                for ( Component c : ( ( JComponent ) component ).getComponents () )
                {
                    if ( isEqualOrChild ( c, compared ) )
                    {
                        return true;
                    }
                }
                return false;
            }
            else
            {
                return false;
            }
        }
    }
    /*
    * Returns true if component or any of its childs has focus
    * Otherwise returns false
    */

    public static boolean hasFocusOwner ( Component component )
    {
        if ( component == null )
        {
            return false;
        }
        //       else if ( component instanceof JDialog )
        //        {
        //            return hasFocusOwner ( ( ( JDialog ) component ).getRootPane () );
        //        }
        //       else if ( component instanceof JFrame )
        //        {
        //            return hasFocusOwner ( ( ( JFrame ) component ).getRootPane () );
        //        }
        //       else if ( component instanceof JWindow )
        //        {
        //            return hasFocusOwner ( ( ( JWindow ) component ).getRootPane () );
        //        }     else
        else if ( component.isFocusOwner () )
        {
            return true;
        }
        else if ( component instanceof Container )
        {
            for ( Component c : ( ( Container ) component ).getComponents () )
            {
                if ( hasFocusOwner ( c ) )
                {
                    return true;
                }
            }
        }
        return false;
    }

    /*
    * Returns available system fonts list
    */

    private static String[] fontNames;
    private static Font[] fonts;

    public static String[] getFontNames ()
    {
        if ( fontNames == null )
        {
            fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment ()
                    .getAvailableFontFamilyNames ();
        }
        return fontNames;
    }

    public static Font[] getFonts ()
    {
        if ( fonts == null )
        {
            fonts = createFonts ( getFontNames () );
        }
        return fonts;
    }

    /*
    * Creates fonts by their font-names
    */

    public static Font[] createFonts ( String[] fontNames )
    {
        Font[] fonts = new Font[ fontNames.length ];
        for ( int i = 0; i < fontNames.length; i++ )
        {
            fonts[ i ] = new Font ( fontNames[ i ], Font.PLAIN, 13 );
        }
        return fonts;
    }

    /*
    * Improved standard invokeLater method
    */

    public static void invokeLater ( Runnable runnable )
    {
        if ( SwingUtilities.isEventDispatchThread () )
        {
            runnable.run ();
        }
        else
        {
            SwingUtilities.invokeLater ( runnable );
        }
    }

    public static void invokeLater ( final HotkeyRunnable runnable, final KeyEvent e )
    {
        if ( SwingUtilities.isEventDispatchThread () )
        {
            runnable.run ( e );
        }
        else
        {
            SwingUtilities.invokeLater ( new Runnable ()
            {
                public void run ()
                {
                    runnable.run ( e );
                }
            } );
        }
    }

    /*
    * Smoothly scrolls scroll pane to destination point
    */

    private static Thread lastCalledThread1;
    private static Thread lastCalledThread2;

    public static void scrollSmoothly ( final JScrollPane scrollPane, int xValue, int yValue )
    {
        final JScrollBar hor = scrollPane.getHorizontalScrollBar ();
        final JScrollBar ver = scrollPane.getVerticalScrollBar ();

        Dimension viewportSize = scrollPane.getViewport ().getSize ();
        xValue = xValue > hor.getMaximum () - viewportSize.width ?
                hor.getMaximum () - viewportSize.width : xValue;
        yValue = yValue > ver.getMaximum () - viewportSize.height ?
                ver.getMaximum () - viewportSize.height : yValue;
        final int x = xValue < 0 ? 0 : xValue;
        final int y = yValue < 0 ? 0 : yValue;


        final int xSign = hor.getValue () > x ? -1 : 1;
        final int ySign = ver.getValue () > y ? -1 : 1;

        //        System.out.println ( "x:" + x + " y:" + y + " hor.getMaximum ():" + hor.getMaximum () +
        //                " ver.getMaximum ():" + ver.getMaximum () );

        new Thread ( new Runnable ()
        {
            public void run ()
            {
                lastCalledThread1 = Thread.currentThread ();
                int lastValue = hor.getValue ();
                while ( lastValue != x )
                {
                    try
                    {
                        if ( lastCalledThread1 != Thread.currentThread () )
                        {
                            Thread.currentThread ().interrupt ();
                        }
                        if ( lastValue != x )
                        {
                            final int value = lastValue +
                                    xSign * Math.max ( Math.abs ( lastValue - x ) / 4, 1 );
                            lastValue = value;
                            SwingUtilities.invokeLater ( new Runnable ()
                            {
                                public void run ()
                                {
                                    hor.setValue ( value );
                                }
                            } );
                            if ( xSign < 0 && value == hor.getMinimum () ||
                                    xSign > 0 && value == hor.getMaximum () )
                            {
                                break;
                            }
                        }
                        Thread.sleep ( 25 );
                    }
                    catch ( InterruptedException ex )
                    {
                        //
                    }
                }
            }
        } ).start ();
        new Thread ( new Runnable ()
        {
            public void run ()
            {
                lastCalledThread2 = Thread.currentThread ();
                int lastValue = ver.getValue ();
                while ( lastValue != y )
                {
                    try
                    {
                        if ( lastCalledThread2 != Thread.currentThread () )
                        {
                            Thread.currentThread ().interrupt ();
                        }
                        if ( lastValue != y )
                        {
                            final int value = lastValue +
                                    ySign * Math.max ( Math.abs ( lastValue - y ) / 4, 1 );
                            lastValue = value;
                            SwingUtilities.invokeLater ( new Runnable ()
                            {
                                public void run ()
                                {
                                    ver.setValue ( value );
                                }
                            } );
                            if ( ySign < 0 && value == ver.getMinimum () ||
                                    ySign > 0 && value == ver.getMaximum () )
                            {
                                break;
                            }
                        }
                        Thread.sleep ( 25 );
                    }
                    catch ( InterruptedException ex )
                    {
                        //
                    }
                }
            }
        } ).start ();
    }
}
