/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;

/**
 * User: mgarin Date: 05.07.11 Time: 14:51
 */

public class TextUtils
{
    /**
     * Parses point from text value
     */

    public static Point parsePoint ( String text )
    {
        text = text.trim ();
        StringTokenizer st = new StringTokenizer ( text, ",", false );
        if ( st.countTokens () == 2 )
        {
            return new Point ( Integer.parseInt ( st.nextToken ().trim () ),
                    Integer.parseInt ( st.nextToken ().trim () ) );
        }
        else
        {
            return null;
        }
    }

    /**
     * Removes all control symbols from string (fast example of replaceAll("\\p{Cntrl}", ""))
     */

    public static char[] buf = new char[ 1024 ];

    public static String removeControlSymbols ( String text )
    {
        int length = text.length ();
        char[] oldChars = ( length < 1024 ) ? buf : new char[ length ];
        text.getChars ( 0, length, oldChars, 0 );
        int newLen = 0;
        for ( int j = 0; j < length; j++ )
        {
            char ch = oldChars[ j ];
            if ( ch >= ' ' )
            {
                oldChars[ newLen ] = ch;
                newLen++;
            }
        }
        return text;
    }

    /**
     * Creates shortened text
     */

    public static String shortenText ( String text, int maxLength, boolean addDots )
    {
        if ( text.length () <= maxLength )
        {
            return text;
        }
        else
        {
            return text.substring ( 0, maxLength > 3 ? maxLength - 3 : maxLength ) +
                    ( addDots ? "..." : "" );
        }
    }

    /**
     * Converts string to list
     */

    public static List<String> stringToList ( String string, String separator )
    {
        List<String> imageTags = new ArrayList<String> ();
        if ( string != null )
        {
            StringTokenizer tokenizer = new StringTokenizer ( string, separator, false );
            while ( tokenizer.hasMoreTokens () )
            {
                imageTags.add ( tokenizer.nextToken ().trim () );
            }
        }
        return imageTags;
    }

    /**
     * Converts list to string
     */

    public static String listToString ( List list, String separator )
    {
        String string = "";
        if ( list != null )
        {
            for ( Object object : list )
            {
                string += object.toString () +
                        ( list.indexOf ( object ) != list.size () - 1 ? separator : "" );
            }
        }
        return string;
    }

    /**
     * Creates unique IDs
     */

    private static final int partLength = 5;
    private static final String prefix = "WebLaF";
    private static final String suffix = "ID";
    private static final String tooltipPrefix = "TT";
    private static final String checkboxListPrefix = "CBL";
    private static final String webGlassPanePrefix = "WGP";
    private static final String webFileListPrefix = "WFL";
    private static final String webSettingsPrefix = "WS";

    public static String generateTooltipId ()
    {
        return generateId ( tooltipPrefix, null );
    }

    public static String generateCheckboxListId ()
    {
        return generateId ( checkboxListPrefix, null );
    }

    public static String generateWebGlassPaneId ()
    {
        return generateId ( webGlassPanePrefix, null );
    }

    public static String generateWebFileListId ()
    {
        return generateId ( webFileListPrefix, null );
    }

    public static String generateWebSettingsId ()
    {
        return generateId ( webSettingsPrefix, null );
    }

    public static String generateId ( String prefix )
    {
        return generateId ( prefix, null );
    }

    public static String generateId ( String prefix, String suffix )
    {
        return ( prefix == null ? TextUtils.prefix : prefix ) + "-" + generateIdBody () + "-" +
                ( suffix == null ? TextUtils.suffix : suffix );
    }

    private static String generateIdBody ()
    {
        return generateId ( partLength ) + "-" + generateId ( partLength ) + "-" +
                generateId ( partLength ) + "-" + generateId ( partLength );
    }

    private static String generateId ( int length )
    {
        String pass = "";
        Random ran = new Random ();
        for ( int i = 0; i < length; i++ )
        {
            char next = 0;
            int range = 10;
            switch ( ran.nextInt ( 3 ) )
            {
                case 0:
                {
                    next = '0';
                    range = 10;
                }
                break;
                case 1:
                {
                    next = 'a';
                    range = 26;
                }
                break;
                case 2:
                {
                    next = 'A';
                    range = 26;
                }
                break;
            }
            pass += ( char ) ( ( ran.nextInt ( range ) ) + next );
        }
        return pass;
    }
}
