/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import java.util.Calendar;
import java.util.Date;

/**
 * User: mgarin Date: 02.11.11 Time: 11:10
 */

public class TimeUtils
{
    /*
    * Returns current system time
    */

    public static long currentTime ()
    {
        return System.currentTimeMillis ();
    }

    /*
    * Returns current system date
    */

    public static Date currentDate ()
    {
        return new Date ( currentTime () );
    }

    /*
    * Compares two specified dates if they are representing the same day
    */

    public static boolean isSameDay ( Date date1, Date date2 )
    {
        return isSameDay ( date1.getTime (), date2.getTime () );
    }

    public static boolean isSameDay ( Long date1, Long date2 )
    {
        Calendar calendar = Calendar.getInstance ();

        // Saving calendar time
        Date tmp = calendar.getTime ();

        calendar.setTimeInMillis ( date1 );
        boolean same = isSameDay ( calendar, date2 );

        // Restoring calendar time
        calendar.setTime ( tmp );

        return same;
    }

    /*
    * Compares calendar date with specified one if they are representing the same day
    */

    public static boolean isSameDay ( Calendar calendar, Date date )
    {
        return isSameDay ( calendar, date.getTime () );
    }

    public static boolean isSameDay ( Calendar calendar, Long date )
    {
        // Saving calendar time
        long time = calendar.getTimeInMillis ();

        int year = calendar.get ( Calendar.YEAR );
        int month = calendar.get ( Calendar.MONTH );
        int day = calendar.get ( Calendar.DAY_OF_MONTH );

        calendar.setTimeInMillis ( date );
        boolean sameDay = year == calendar.get ( Calendar.YEAR ) &&
                month == calendar.get ( Calendar.MONTH ) &&
                day == calendar.get ( Calendar.DAY_OF_MONTH );

        // Restoring calendar time
        calendar.setTimeInMillis ( time );

        return sameDay;
    }

    /*
    * Changes calendar date by days
    */

    public static void increaseByDay ( Calendar calendar )
    {
        changeByDays ( calendar, 1 );
    }

    public static void decreaseByDay ( Calendar calendar )
    {
        changeByDays ( calendar, -1 );
    }

    public static void changeByDays ( Calendar calendar, int days )
    {
        calendar.set ( Calendar.DAY_OF_MONTH, calendar.get ( Calendar.DAY_OF_MONTH ) + days );
    }
}
