/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils.image;

import com.alee.utils.laf.DefaultBackgroundPainter;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.net.URL;

/**
 * User: mgarin Date: 14.12.11 Time: 15:20
 */

public class NinePatchBackgroundPainter extends DefaultBackgroundPainter
{
    private NinePatchIcon ninePatchIcon;

    public NinePatchBackgroundPainter ()
    {
        this ( ( NinePatchIcon ) null );
    }

    public NinePatchBackgroundPainter ( URL url )
    {
        this ( new NinePatchIcon ( url ) );
    }

    public NinePatchBackgroundPainter ( String iconSrc )
    {
        this ( new NinePatchIcon ( iconSrc ) );
    }

    public NinePatchBackgroundPainter ( ImageIcon imageIcon )
    {
        this ( new NinePatchIcon ( imageIcon ) );
    }

    public NinePatchBackgroundPainter ( Image image )
    {
        this ( new NinePatchIcon ( image ) );
    }

    public NinePatchBackgroundPainter ( BufferedImage bufferedImage )
    {
        this ( new NinePatchIcon ( bufferedImage ) );
    }


    public NinePatchBackgroundPainter ( NinePatchIcon ninePatchIcon )
    {
        super ();
        this.ninePatchIcon = ninePatchIcon;
    }

    public NinePatchIcon getNinePatchIcon ()
    {
        return ninePatchIcon;
    }

    public void setNinePatchIcon ( NinePatchIcon ninePatchIcon )
    {
        this.ninePatchIcon = ninePatchIcon;
    }

    public void paintBackground ( Graphics g,Rectangle bounds, JComponent c )
    {
        if ( ninePatchIcon != null )
        {
            ninePatchIcon.setComponent ( c );
            ninePatchIcon.paintIcon ( c, g, 0, 0 );
        }
    }

    public Dimension getBackgroundPreferredSize ( JComponent c )
    {
        if ( ninePatchIcon != null )
        {
            ninePatchIcon.setComponent ( c );
            return ninePatchIcon.getPreferredSize ();
        }
        else
        {
            return super.getBackgroundPreferredSize ( c );
        }
    }

    public Insets getBackgroundMargin ( JComponent c )
    {
        if ( ninePatchIcon != null )
        {
            ninePatchIcon.setComponent ( c );
            return ninePatchIcon.getMargin ();
        }
        else
        {
            return super.getBackgroundMargin ( c );
        }
    }
}
