/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils.image;

import com.alee.utils.LafUtils;
import com.alee.utils.laf.DefaultBackgroundPainter;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * User: mgarin Date: 26.01.12 Time: 12:31
 */

public class TextureBackgroundPainter extends DefaultBackgroundPainter
{
    private TexturePaint paint;

    public TextureBackgroundPainter ( BufferedImage image )
    {
        super ();
        updatePainter ( image );
    }

    public void setImage ( BufferedImage image )
    {
        updatePainter ( image );
    }

    public BufferedImage getImage ()
    {
        return paint.getImage ();
    }

    private void updatePainter ( BufferedImage image )
    {
        this.paint = new TexturePaint ( image,
                new Rectangle ( 0, 0, image.getWidth (), image.getHeight () ) );
    }

    public void paintBackground ( Graphics g, Rectangle bounds, JComponent c )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        Object old = LafUtils.setupImageQuality ( g2d );
        g2d.setPaint ( paint );
        g2d.fillRect ( bounds.x, bounds.y, bounds.width, bounds.height );
        LafUtils.restoreImageQuality ( g2d, old );
    }
}